package com.rube;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;

/**
 * Created by wuhao on 4/24/17.
 */
public class Drawer {
    public interface ImageVertex {
        float[] getCornerX();

        float[] getCornerY();

        String name();

        boolean getFlipX();
    }

    private float phy2pix = 1;
    private static final float[] vertex = new float[20];

    public float getPhy2pix() {
        return phy2pix;
    }

    public void setPhy2pix(float phy2pix) {
        this.phy2pix = phy2pix;
    }

    public void draw(Batch batch, Body body, ImageData imageVertex, TextureRegion region, float bodyX, float bodyY) {
        if(region == null){
            region = imageVertex.reigon;
        }
        if(region == null) return;
        float angle = 0;
        float x = 0;
        float y = 0;
        if(body != null){
            x = bodyX;
            y = bodyY;
            angle = body.getAngle();
        }
        float []vx = imageVertex.getCornerX();
        float []vy = imageVertex.getCornerY();
        final int N = imageVertex.getCornerX().length;

        if (angle == 0) {
            for (int k = 0; k < N; k++) {
                vertex[Batch.X1 + 5 * k] = (x + vx[k]) * phy2pix;
                vertex[Batch.Y1 + 5 * k] = (y + vy[k]) * phy2pix;
            }
        } else {
            float cos = (float) Math.cos(angle);
            float sin = (float) Math.sin(angle);
            for (int k = 0; k < N; k++) {
                float xx = cos * vx[k] - sin * vy[k];
                float yy = sin * vx[k] + cos * vy[k];
                vertex[Batch.X1 + 5 * k] = (x + xx) * phy2pix;
                vertex[Batch.Y1 + 5 * k] = (y + yy) * phy2pix;
            }
        }

        boolean flipX = imageVertex.getFlipX();
        if (flipX) { // flipX
            vertex[Batch.U1] = region.getU2();
            vertex[Batch.U2] = region.getU();
            vertex[Batch.U3] = region.getU();
            vertex[Batch.U4] = region.getU2();
        } else {
            vertex[Batch.U1] = region.getU();
            vertex[Batch.U2] = region.getU2();
            vertex[Batch.U3] = region.getU2();
            vertex[Batch.U4] = region.getU();
        }
        vertex[Batch.V1] = vertex[Batch.V2] = region.getV2();
        vertex[Batch.V3] = vertex[Batch.V4] = region.getV();

        float c = batch.getColor().toFloatBits();
        if(imageVertex.color!=null){
            if (imageVertex.color.a < 0.7F){
                imageVertex.color.a = 0.4F;
            }
            c = imageVertex.color.toFloatBits();
        }
        for (int k = 0; k < N; k++)
            vertex[Batch.C1 + 5 * k] = c;

        batch.draw(region.getTexture(), vertex, 0, 20);
    }
}
