package com.rube;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.utils.Array;

/**
 * Created by Alan on 2017/4/13.
 */
public class ImageDrawer {
    public ImageDrawer() {
    }
    // the order of bodies in box2d and body data in the `rube.json` generated by rube is not the same,
    // so, I track the body order.
    public ImageDrawer setup(Array<ImageData> drawOrder, Array<Body> bodies, com.rube.TextureMapper ta) {
        this.drawOrder.clear();
        this.drawOrder.addAll(drawOrder);
        this.bodies.clear();
        this.bodies.addAll(bodies);
        this.regions.clear();
        if (ta!=null) {
            for (int i = 0; i < drawOrder.size; i++) {
                regions.add(ta.mapR(drawOrder.get(i).getFilename()));
            }
        }
        return this;
    }
    public ImageDrawer updateSkin(com.rube.TextureMapper mapper) {
        this.regions.clear();
        for (int i=0; i<drawOrder.size; i++) {
            regions.add(mapper.mapR(drawOrder.get(i).getFilename()));
        }
        return this;
    }
    public void clear() {
        this.drawOrder.clear();
        this.bodies.clear();
        this.regions.clear();
    }
    // meter(unit in box2d/physics) => pixel(used when rendering)
    public ImageDrawer setPhy2Pix(float phy2Pix) {
        this.phy2pix = phy2Pix;
        return this;
    }
    public void draw(Batch batch, float alpha) {
        color.a = alpha;
        draw(batch, color);
    }
    public void draw(Batch batch, Color cc) {
        for (int i=0; i<drawOrder.size; i++) {
            draw(batch, cc, i);
        }
    }
    public void draw(Batch batch, float alpha, float rotation) {
        color.a = alpha;
        draw(batch, color, rotation);
    }
    public void draw(Batch batch, Color cc, float rotation) {
        for (int i=0; i<drawOrder.size; i++) {
            draw(batch, cc, i, rotation);
        }
    }
    void draw(Batch batch, Color cc, int i) {
        com.rube.ImageInterface imageData = drawOrder.get(i);
        TextureRegion region = regions.get(i);
        if (region==null) return;
        cc.a *= imageData.getOpacity();
        Body body = bodies.get(imageData.getBodyIndex());
        Vector2 pos = body.getPosition();
        final float x = pos.x;
        final float y = pos.y;
        float angle = body.getAngle();
        float []vx = imageData.getCornerX();
        float []vy = imageData.getCornerY();
        final int N = vx.length;
        if (angle==0) {
            for (int k = 0; k < N; k++) {
                vertex[SpriteBatch.X1 + 5 * k] = (x + vx[k])*phy2pix;
                vertex[SpriteBatch.Y1 + 5 * k] = (y + vy[k])*phy2pix;
            }
        }else {
            float cos = (float)Math.cos(angle);
            float sin = (float)Math.sin(angle);
            for (int k=0; k<N; k++) {
                float xx = cos * vx[k] - sin * vy[k];
                float yy = sin * vx[k] + cos * vy[k];
                vertex[SpriteBatch.X1 + 5*k] = (x+xx) * phy2pix;
                vertex[SpriteBatch.Y1 + 5*k] = (y+yy) * phy2pix;
            }
        }
        boolean flipX = imageData.getFlipX();
        float u = region.getU(), u2 = region.getU2();
        if (flipX) { // flipX
            vertex[SpriteBatch.U1] = u2;
            vertex[SpriteBatch.U2] = u;
            vertex[SpriteBatch.U3] = u;
            vertex[SpriteBatch.U4] = u2;
        }else {
            vertex[SpriteBatch.U1] = u;
            vertex[SpriteBatch.U2] = u2;
            vertex[SpriteBatch.U3] = u2;
            vertex[SpriteBatch.U4] = u;
        }
        vertex[SpriteBatch.V1] = vertex[SpriteBatch.V2] = region.getV2();
        vertex[SpriteBatch.V3] = vertex[SpriteBatch.V4] = region.getV();

//        color.a = alpha;
        float c = color.toFloatBits();
        for (int k=0; k<N; k++)
            vertex[SpriteBatch.C1 + 5*k] = c;

        batch.draw(region.getTexture(), vertex, 0, 5*N);

    }
    void draw(Batch batch, Color cc, int i, float rotation) {
        com.rube.ImageInterface imageData = drawOrder.get(i);
        TextureRegion region = regions.get(i);
        if (region==null) return;
        cc.a *= imageData.getOpacity();
        Body body = bodies.get(imageData.getBodyIndex());
        Vector2 pos = body.getPosition();
        final float x = pos.x;
        final float y = pos.y;
        float angle = ((float) (rotation * Math.PI / 180));
        float []vx = imageData.getCornerX();
        float []vy = imageData.getCornerY();
        final int N = vx.length;
        if (angle==0) {
            for (int k = 0; k < N; k++) {
                vertex[SpriteBatch.X1 + 5 * k] = (x + vx[k])*phy2pix;
                vertex[SpriteBatch.Y1 + 5 * k] = (y + vy[k])*phy2pix;
            }
        }else {
            float cos = (float)Math.cos(angle);
            float sin = (float)Math.sin(angle);
            for (int k=0; k<N; k++) {
                float xx = cos * vx[k] - sin * vy[k];
                float yy = sin * vx[k] + cos * vy[k];
                vertex[SpriteBatch.X1 + 5*k] = (x+xx) * phy2pix;
                vertex[SpriteBatch.Y1 + 5*k] = (y+yy) * phy2pix;
            }
        }
        boolean flipX = imageData.getFlipX();
        float u = region.getU(), u2 = region.getU2();
        if (flipX) { // flipX
            vertex[SpriteBatch.U1] = u2;
            vertex[SpriteBatch.U2] = u;
            vertex[SpriteBatch.U3] = u;
            vertex[SpriteBatch.U4] = u2;
        }else {
            vertex[SpriteBatch.U1] = u;
            vertex[SpriteBatch.U2] = u2;
            vertex[SpriteBatch.U3] = u2;
            vertex[SpriteBatch.U4] = u;
        }
        vertex[SpriteBatch.V1] = vertex[SpriteBatch.V2] = region.getV2();
        vertex[SpriteBatch.V3] = vertex[SpriteBatch.V4] = region.getV();

//        color.a = alpha;
        float c = color.toFloatBits();
        for (int k=0; k<N; k++)
            vertex[SpriteBatch.C1 + 5*k] = c;

        batch.draw(region.getTexture(), vertex, 0, 5*N);

    }
    public void addDrawable(Body body, com.rube.ImageInterface image, TextureRegion region) {
        this.bodies.add(body);
        this.drawOrder.add(image);
        this.regions.add(region);
    }

    float phy2pix=1;
    final Array<com.rube.ImageInterface> drawOrder = new Array<com.rube.ImageInterface>();
    final Array<Body> bodies = new Array<Body>();
    final Array<TextureRegion> regions = new Array<TextureRegion>();

    private static final Color color = new Color(Color.WHITE);
    private static final float []vertex = new float[20];
}
