package com.zyb.loveball;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.esotericsoftware.spine.*;
import com.esotericsoftware.spine.AnimationState.AnimationStateListener;
import com.zyb.loveball.assets.Assets;

public class AnimationHandler extends Actor {
    private SkeletonRenderer render;
    private SkeletonData sData;
    AnimationStateData animationStateData;
    private AnimationState state;
    private Skeleton skeleton;
    private SkeletonJson skeletonJson;
    boolean end = false;
    boolean init = false;
    String fileName;
    TextureAtlas atlas;

    public AnimationHandler(String fileName) {
        this(fileName, null);
    }

    public AnimationHandler(String fileName, TextureAtlas atlas) {
        this.fileName = fileName;
        this.atlas = atlas;
        setTouchable(Touchable.disabled);
        init();
    }
    public AnimationHandler(String fileName, float scale) {
        this.fileName = fileName;
        setScale(scale);
        init();
    }

    public void setAnimation(String name, boolean loop) {
        skeleton.setToSetupPose();
        state.setAnimation(0, name, loop);

    }

    @Override
    public void setRotation(float degrees) {
        skeleton.getRootBone().setRotation(degrees);
    }

    public void setRender(SkeletonRenderer render) {
        this.render = render;
    }

    public void addAnimationStateListener(AnimationStateListener listener) {
        state.addListener(listener);
    }

    public void removeAnimationStateListener(AnimationStateListener listener) {
        state.removeListener(listener);
    }

    private String atlasName;

    private void init() {
//        if (atlas == null) {
//            atlasName = fileName.substring(0, fileName.length() - 5) + ".atlas";
//            Assets.instance.getManager().load(atlasName, TextureAtlas.class);
//            Assets.instance.getManager().finishLoading();
//            atlas = Assets.instance.getManager().get(fileName.substring(0, fileName.length() - 5) + ".atlas");
//        }
//        skeletonJson = new SkeletonJson(atlas);
//        skeletonJson.setScale(getScaleX());
//        sData = skeletonJson.readSkeletonData(Gdx.files.internal(fileName));
//        animationStateData = new AnimationStateData(sData);
//        state = new AnimationState(animationStateData);
//        skeleton = new Skeleton(sData);
//        init = true;
//        setPolygonSpriteBatch(TrivialGame.getPolygonSpriteBatch());
//        setRender(TrivialGame.getRenderer());
    }

    public boolean isEnd() {
        return end;
    }

    public void draw(PolygonSpriteBatch batch, float parentAlpha) {
        if (init) {
            state.update(Gdx.graphics.getDeltaTime());
            // 动画控制器控制骨骼动画
            state.apply(skeleton);
            // 骨骼逐级进行矩阵变换
            skeleton.updateWorldTransform();
            // 绘制
            render.draw(batch, skeleton);
            batch.setBlendFunction(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA); //
        }
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        if(!isVisible()) return;
        if (init) {
            state.update(Gdx.graphics.getDeltaTime());
            // 动画控制器控制骨骼动画
            state.apply(skeleton);
            // 骨骼逐级进行矩阵变换
            skeleton.updateWorldTransform();
            render.draw(batch,skeleton);
            // 绘制
//            if (polygonSpriteBatch != null) {
//                batch.end();
//                polygonSpriteBatch.setProjectionMatrix(batch.getProjectionMatrix());
//                polygonSpriteBatch.setTransformMatrix(batch.getTransformMatrix());
//                polygonSpriteBatch.begin();
//                render.draw(polygonSpriteBatch, skeleton);
//                polygonSpriteBatch.setBlendFunction(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA);
//                polygonSpriteBatch.end();
//                batch.begin();
//            } else {
//                //render.draw(batch, skeleton);
//            }
            batch.setBlendFunction(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA); //
        }
    }


    public void dispose() {
        if (atlasName != null) {
         //   Assets.instance.getManager().unload(atlasName);
            atlasName = null;
        }
    }

    @Override
    public void setPosition(float x, float y) {
        super.setPosition(x, y);
        skeleton.setPosition(x, y);
    }

    @Override
    public void setY(float y) {
        super.setY(y);
        skeleton.setY(y);
    }

    @Override
    public void setX(float x) {
        super.setX(x);
        skeleton.setX(x);
    }
}
