package com.zyb.loveball;

import com.badlogic.gdx.Application;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.objects.ProgressBar;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.screens.ChallengeScreen;
import com.zyb.loveball.utils.LayerUtils;
import com.zyb.loveball.utils.listener.SoundButtonListener;

public class ChallengeUIPanel extends UIPanel {
    private ProgressBar inkProgress;
    private ProgressBar challengeProgress;
    private boolean countdown;
    private int countdownTime;

    public ChallengeUIPanel(Group scene, GameScreen screen) {
        super(scene, screen);
        this.root = scene.findActor("challenge");
        root.setVisible(true);
        inkProgress = new ProgressBar(root.findActor("ink_progress"), (Label) root.findActor("has_remain_percent"));
        challengeProgress = new ProgressBar(root.findActor("challenge_progress_bar"), null);
        initButtons();
        initUI();
    }

    private void initUI() {
        for (int i = 4; i >= GameInfo.life; i--) {
            root.findActor("life_mask_" + i).setVisible(false);
        }
        challengeProgress.update(GameInfo.id / 10F);
        root.findActor("challenge_mask").setX(GameInfo.id * 19F + 90);
        ((Label)root.findActor("challenge_title_label")).setText("CHALLENGE "+GameInfo.id);
        Actor title = root.findActor("challenge_title_group");
        title.setX(-500);
        Actor layer = LayerUtils.createLayer();
        layer.setVisible(false);
        layer.addAction(Actions.sequence(Actions.delay(0.5F),Actions.visible(true),Actions.delay(2.5F),Actions.visible(false)));
        title.addAction(Actions.sequence(Actions.delay(0.5F),Actions.moveBy(500,0,0.3F),Actions.delay(2),Actions.moveBy(500,0,0.2F),Actions.run(new Runnable() {
            @Override
            public void run() {
                start();
            }
        })));
        Group bottom = root.findActor("bottom");
        if (bottom != null) {
            bottom.setY(800 - HelloZombieGame.getViewport().getWorldHeight());
        }
        Group center = root.findActor("center");
        if (center != null) {
            center.setY((800 - HelloZombieGame.getViewport().getWorldHeight())/2);
        }
        center.addActorBefore(title,layer);
        root.findActor("challenge"+GameInfo.chapter).setVisible(true);
    }

    private void start(){
        if(GameInfo.chapter == 2){
            countdownTime = Constant.CHALLENGE_2_COUNTDOWN;
            final Group countdownGroup = root.findActor("time_countdown");
            final Label timeLabel2 = root.findActor("time_label2");
            timeLabel2.setText(""+Constant.CHALLENGE_2_COUNTDOWN);
            timeLabel2.addAction(Actions.sequence(Actions.forever(Actions.sequence(Actions.delay(1),Actions.run(new Runnable() {
                @Override
                public void run() {
                    if(gamePanel.state !=GamePanel.GameState.pause) {
                        countdownTime -= 1;
                    }
                    if(countdownTime == 5){
                        countdownGroup.setVisible(true);
                    }
                    if(countdownTime>=0) {
                        timeLabel2.setText("" + countdownTime);
                        ((Label) countdownGroup.findActor("time_label")).setText("" + countdownTime);
                    }
                }
            }))),Actions.run(new Runnable() {
                @Override
                public void run() {
                    gamePanel.end(false);
                }
            })));
        }
    }

    @Override
    protected void initButtons() {
        super.initButtons();
        root.findActor("btn_clear", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                clearDraws();
            }
        });
    }

    public void countdown(boolean show,int time){
        if(countdown == show) return;
        countdown = show;
        final Group countdownGroup = root.findActor("time_countdown");
        if(countdown){
            countdownTime = time;
            countdownGroup.setVisible(true);
            countdownGroup.addAction((Actions.forever(Actions.sequence(Actions.run(new Runnable() {
                @Override
                public void run() {
                    if(countdownTime>=0) {
                        ((Label) countdownGroup.findActor("time_label")).setText("" + countdownTime);
                    }
                    if(countdownTime < 0){
                        gamePanel.end(false);
                    }
                    if(gamePanel.state !=GamePanel.GameState.pause) {
                        countdownTime -= 1;
                    }
                }
            }),Actions.delay(1)))));
        }else{
            countdownGroup.setVisible(false);
            countdownGroup.getActions().clear();
        }
    }

    private void clearDraws(){
        if (Gdx.app.getType() == Application.ApplicationType.Desktop) {
            screen.gamePanel.paintPanel.clearLines();
        } else {
            screen.setPendingAction(BaseScreen.PendingAction.HINT);
            HelloZombieGame.launcherListener.playUnityAds();
        }
    }


    @Override
    public void update() {
        float progress = gamePanel.inkLeftPercent();
        if(GameInfo.chapter == 1) {
            inkProgress.update(progress);
        }
        System.out.println(progress);
    }

    public void success() {
        countdown(false,0);
        if (GameInfo.id < 10) {
            GameInfo.id++;
            screen.end(GameScreen.class);
        } else {
            Configuration.settingData.challengePass(GameInfo.chapter);
            screen.end(ChallengeScreen.class);
        }
    }

    public void failure() {
        GameInfo.life--;
        if (GameInfo.life >= 1) {
            System.out.println("life ---->" + GameInfo.life);
            screen.end(GameScreen.class);
        } else {
            screen.gamePanel.state = GamePanel.GameState.failure;
            screen.showFailureDialog();
        }
        root.findActor("life_mask_" + GameInfo.life).setVisible(false);
        countdown(false,0);
    }
}
