package com.zyb.loveball;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.*;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.dialogs.*;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.screens.ChallengeScreen;
import com.zyb.loveball.screens.ChapterScreen;
import com.zyb.loveball.screens.LevelScreen;
import com.zyb.loveball.screens.MenuScreen;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.MScreenUtil;
import com.zyb.loveball.utils.listener.ButtonListener;
import com.zyb.loveball.utils.Matrix4s;
import com.zyb.loveball.utils.listener.SoundButtonListener;


public class GameScreen extends BaseScreen {

    GamePanel gamePanel;
    UIPanel uiPanel;
    private CircleLayer circleLayer;

    public GameScreen(HelloZombieGame game) {
        super(game);
        TAG = "game";
        inTime = 1;
    }

    @Override
    public void show() {
        super.show();
        GameInfo.start();
        if (GameInfo.challenge) {
            uiPanel = new ChallengeUIPanel(scene, this);
        } else {
            uiPanel = new GameUIPanel(scene, this);
        }
        gamePanel = new GamePanel(stage.getCamera().viewportWidth * Constant.SCALE, stage.getCamera().viewportHeight * Constant.SCALE, uiPanel);
        uiPanel.setGamePanel(gamePanel);
        stage.addActor(gamePanel);
        stage.addActor(scene);
        scene.setTouchable(Touchable.childrenOnly);
        initButtons();
        layer.remove();
        Matrix4s.ui.set(stage.getCamera().combined);
        circleLayer = new CircleLayer(inTime);
        Vector3 vector3 = gamePanel.getZombieScreenPosition();
        Vector2 zomPosition = screenToStageCoordinates(vector3.x, vector3.y);
        circleLayer.open(zomPosition.x - 10, zomPosition.y + 50);
        stage.addActor(circleLayer);
    }

    @Override
    protected void start() {
        super.start();
        if(circleLayer!=null){
            circleLayer.remove();
        }
        HelloZombieGame.launcherListener.showBanner(GameInfo.levelData.isBanner());
    }

    @Override
    protected void initButtons() {
        super.initButtons();
        uiPanel.getRoot().findActor("btn_retry", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                uiPanel.failure();
            }
        });

        uiPanel.getRoot().findActor("btn_pause", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                pauseGame();
            }
        });
//        findActor("test_hint", Touchable.enabled).addListener(new ButtonListener() {
//            @Override
//            public void touchUpEffect() {
//                super.touchUpEffect();
//                gamePanel.hint();
//            }
//        });
        if (Configuration.debug) {
            findActor("debug").setVisible(true);
            findActor("btn_pre", Touchable.enabled).addListener(new SoundButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    GameInfo.preLevel();
                    gamePanel.end(false);
                }
            });
            findActor("btn_next", Touchable.enabled).addListener(new ButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    GameInfo.nextLevel();
                    gamePanel.end(false);
                }
            });
            findActor("test_hint", Touchable.enabled).addListener(new ButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    gamePanel.hint();
                }
            });
        } else {
            findActor("debug").setVisible(false);
        }
    }

    @Override
    public void pause() {
        Log.log("GameScreen", "pause()");
        pauseGame();
    }

    private void pauseGame() {
        if (state != ScreenState.end) {
            if (getPendingAction() == PendingAction.NONE && gamePanel.pause()) {
                showPauseDialog();
            }
        }
    }

    private void showPauseDialog() {
        PauseDialog pauseDialog = (PauseDialog) showDialog("ui/dialogs/pause.json", PauseDialog.class);
        pauseDialog.setPauseDialogListener(new PauseDialog.PauseDialogListener() {
            @Override
            public void home() {
                if (GameInfo.challenge) {
                    ChallengeQuitDialog confirmDialog = (ChallengeQuitDialog) showDialog("ui/dialogs/quit_challenge.json", ChallengeQuitDialog.class);
                    confirmDialog.setConfirmListener(new ChallengeQuitDialog.ConfirmListener() {
                        @Override
                        public void confirm() {
                            end(MenuScreen.class);
                        }
                    });
                } else {
                    end(MenuScreen.class);
                }
            }

            @Override
            public void menu() {
                if (GameInfo.challenge) {
                    ChallengeQuitDialog confirmDialog = (ChallengeQuitDialog) showDialog("ui/dialogs/quit_challenge.json", ChallengeQuitDialog.class);
                    confirmDialog.setConfirmListener(new ChallengeQuitDialog.ConfirmListener() {
                        @Override
                        public void confirm() {
                            end(ChallengeScreen.class);
                        }
                    });
                } else {
                    end(LevelScreen.class);
                }
            }
        });
    }

    public void showFullScreenAd(boolean show) {
        if (GameInfo.id <= 10 && GameInfo.chapter == 1) return;
        if (Configuration.full_screen_time < 50 && Configuration.retry_times_full_screen < 3) return;
        HelloZombieGame.launcherListener.showInterstitial();
        Configuration.full_screen_time = 0;
        Configuration.retry_times_full_screen = 0;
    }

    @Override
    public BaseDialog showDialog(String attribute, Class<? extends BaseDialog> tClass) {
        gamePanel.pause();
        return super.showDialog(attribute, tClass);
    }

    @Override
    public void update() {
        super.update();
        uiPanel.update();
    }

    @Override
    protected void out() {
        super.out();
    }

    public void showSuccessDialog() {
        SuccessDialog dialog = (SuccessDialog) ((GameScreen) game.getScreen()).showDialog("ui/dialogs/success.json", SuccessDialog.class);
        dialog.setSuccessDialogListener(new SuccessDialog.SuccessDialogListener() {
            @Override
            public void next() {
                if (!(GameInfo.chapter == 30 && GameInfo.id == 10)) {
                    GameInfo.nextLevel();
                    end((GameScreen.class));
                } else {
                    end(ChapterScreen.class);
                }
                GameScreen.this.showFullScreenAd(true);
            }

            @Override
            public void playVideo(PendingAction action, int pendingValue) {
                setPendingAction(action);
                GameScreen.this.pendingValue = pendingValue;
                HelloZombieGame.launcherListener.playUnityAds();
            }

            @Override
            public void showCoinDialog() {
                GameScreen.this.showCoinDialog();
            }

            @Override
            public void showFullScreenAd() {
                GameScreen.this.showFullScreenAd(true);
            }
        });
    }

    public void showFailureDialog() {
        FailureDialog dialog = (FailureDialog) ((GameScreen) game.getScreen()).showDialog("ui/dialogs/revive.json", FailureDialog.class);
        dialog.setReviveDialogListener(new FailureDialog.ReviveDialogListener() {
            @Override
            public void playVideo(PendingAction action, int pendingValue) {
                if (Configuration.videoAdReady) {
                    setPendingAction(action);
                    GameScreen.this.pendingValue = pendingValue;
                    HelloZombieGame.launcherListener.playUnityAds();
                } else {
                    showMessage("The video ad is not ready!");
                }
            }
        });
    }

    public void showRateDialog() {
        RateDialog dialog = (RateDialog) ((GameScreen) game.getScreen()).showDialog("ui/dialogs/rate.json", RateDialog.class);
    }

    @Override
    public void resume() {
        Log.log("GameScreen", "resume()");
        super.resume();
        if (dialogs.size() == 0) {
            gamePanel.resume();
        }
    }

    @Override
    protected void removeDialog() {
        super.removeDialog();
        if (dialogs.empty()) {
            gamePanel.resume();
        }
    }

    @Override
    public void handlePendingAction(boolean done) {
        Log.log("GameScreen", "handlePendingAction()");
        super.handlePendingAction(done);
        switch (getPendingAction()) {
            case HINT:
                gamePanel.hint();
                FlurryUtils.Ads_Reward("Hint");
                setPendingAction(PendingAction.NONE);
                break;
            case RESTART:
                end((GameScreen.class));
                setPendingAction(PendingAction.NONE);
                break;
            case CLEAR:
                gamePanel.paintPanel.clearLines();
                setPendingAction(PendingAction.NONE);
                break;

        }
    }

    @Override
    protected boolean back() {
        Log.log("GameScreen", "back()");
        if (dialogs.size() > 0) {
            BaseDialog dialog = dialogs.peek();
            if (dialog != null) {
                if (dialog instanceof FailureDialog || dialog instanceof SuccessDialog) {
                    return false;
                } else {
                    dialog.close();
                    return false;
                }
            }
        } else {
            Log.log("GameScreen", "back()--->pause");
            pause();
        }
        return true;
    }


    @Override
    public void dispose() {
        super.dispose();
        gamePanel.dispose();
        gamePanel = null;
        uiPanel = null;
    }

    public Vector2 screenToStageCoordinates(float x, float y) {
        Vector2 temp = stage.screenToStageCoordinates(new Vector2(x, Gdx.graphics.getBackBufferHeight() - y));
        return temp;
    }
}