package com.zyb.loveball;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.objects.ProgressBar;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.MScreenUtil;
import com.zyb.loveball.utils.listener.LClickListener;
import com.zyb.loveball.utils.listener.SoundButtonListener;

public class GameUIPanel extends UIPanel {
    private ProgressBar progressBar;
    int star = 3;
    private CommonAnimation[] starAnimation;
    private CommonAnimation handAnimation;
    boolean handTip;
        Group hintVideo;
    Group hintCoin;
    Actor hintVideoGray;
    Group hintBtn;
    Image arrow;
    Label label;
    private HintState hintState = HintState.none;

    public GameUIPanel(Group scene, GameScreen screen) {
        super(scene, screen);
        this.root = scene.findActor("normal");
        root.setVisible(true);
        progressBar = new ProgressBar(root.findActor("progress_bar"), (Label) root.findActor("has_remain_percent"));
        starAnimation = new CommonAnimation[3];
        for (int i = 0; i < starAnimation.length; i++) {
            starAnimation[i] = new CommonAnimation("animations/star.skel", 1);
            Actor temp = root.findActor("star_small_empty" + i);
            starAnimation[i].setPosition(temp.getX() + 10, temp.getY() + 10);
            root.addActorAfter(temp, starAnimation[i]);
        }
        init();
    }

    private void init() {
        if (screen.getClass() != screen.getGame().getPreScreen()) {
            GameInfo.reset();
        }
        hintBtn = root.findActor("btn_hint");
        hintVideo = root.findActor("video");
        hintVideoGray = root.findActor("video_gray");
        hintCoin = root.findActor("coin");
        Group bottom = root.findActor("bottom");
        if (bottom != null) {
            bottom.setY(800 - HelloZombieGame.getViewport().getWorldHeight());
        }
        initButtons();
        checkHintAni();
        showSkip(false);
    }

    private void initCourse() {
        Vector3 vector3 = gamePanel.getZombieScreenPosition();
        Vector2 zomPosition = screen.screenToStageCoordinates(vector3.x, vector3.y);
        arrow = new Image(Assets.instance.ui.findRegion("zombie_arrow"));
        BitmapFont bitmapFont = new BitmapFont(Gdx.files.internal("font/comicbd.fnt"));
        Label.LabelStyle style = new Label.LabelStyle();
        style.font = bitmapFont;
        style.fontColor = Color.BLACK;
        label = new Label("hit the zombie\nwith the bomb", style);
        arrow.setScale(0.5f);
        label.setFontScale(0.5f);
        arrow.setPosition(zomPosition.x + 20, zomPosition.y + 50);
        arrow.setColor(0, 0, 0, 0.65f);
        label.setPosition(zomPosition.x - 60, zomPosition.y + 90);
        arrow.setTouchable(Touchable.disabled);
        label.setTouchable(Touchable.disabled);
        ((Group) root.findActor("bottom")).addActor(arrow);
        ((Group) root.findActor("bottom")).addActor(label);
    }

    @Override
    public void showHand(boolean show) {
        if (show == handTip) return;
        if (show) {
            if (GameInfo.course) return;
            if (hintState == HintState.none) return;
            if (handAnimation == null) {
                initHandTipAni();
            }
            handAnimation.setVisible(true);
            handAnimation.getState().setAnimation(0, "1", false);
            handAnimation.getState().addAnimation(0, "2", true, 0.5f);
            handTip = true;
        } else {
            if (handAnimation != null)
                handAnimation.getState().setAnimation(0, "3", false);
            handTip = false;
        }
    }

    private void initHandTipAni() {
        handAnimation = new CommonAnimation("animations/dianji.skel", 0.5f);
        root.addActor(handAnimation);
        Actor hint = root.findActor("btn_hint");
        float x = hint.getX();
        float y = hint.getY();
        handAnimation.setPosition(x, y);
        handAnimation.setxOffset(hint.getWidth() / 2);
        handAnimation.setyOffset(hint.getHeight() / 2);
        handAnimation.setVisible(false);
    }

    @Override
    protected void initButtons() {
        super.initButtons();
        hintBtn.addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                hint();
            }
        });
        root.findActor("btn_skip", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                gamePanel.end(true, 0.5F);
                showSkip(false);
                FlurryUtils.level_Skip(GameInfo.chapter, GameInfo.levelId);
            }
        });
        Label levelLabel = root.findActor("level_label");
        levelLabel.setText("Level " + GameInfo.chapter + "-" + GameInfo.id);
        if (GameInfo.levelData.isBanner()) {
            levelLabel.setY(levelLabel.getY() + 60);
        }
        CommonAnimation animation = new CommonAnimation("animations/tishi.skel", 0.5f);
        animation.setPosition(hintBtn.getWidth() / 2 - 3F, hintBtn.getHeight() / 2 + 0.3F);
        animation.setName("hint_animation");
        hintBtn.addActor(animation);
        hintAniReset();
    }

    public void showSkip(boolean show) {
        root.findActor("btn_skip").setVisible(show);
    }

    public void screenShot() {
        root.addAction(Actions.sequence(Actions.delay(0.05F), Actions.run(new Runnable() {
            @Override
            public void run() {
                MScreenUtil.screenShot(screen.getGame(), gamePanel, null, Constant.SCREEN_SHOT_W_BIG,Constant.SCREEN_SHOT_H_BIG, true);
            }
        })));

    }

    private void hint() {
        if (GameInfo.hint) return;
        if (hintState == HintState.coin) {
            Configuration.settingData.cost(Constant.HINT_COST);
            gamePanel.hint();
            FlurryUtils.level_HintCoin(GameInfo.chapter, GameInfo.id);
        } else if (hintState == HintState.video) {
            screen.setPendingAction(BaseScreen.PendingAction.HINT);
            HelloZombieGame.launcherListener.playUnityAds();
            FlurryUtils.level_HintAD(GameInfo.chapter, GameInfo.id);
        } else {
            screen.showMessage("The video ad is not ready!");
        }
        updateHintBtn();
        showHand(false);
        hintAniReset();
    }

    private void checkHintAni() {
        updateHintBtn();
        if (GameInfo.course) return;
        if (GameInfo.hint) return;
        if (GameInfo.failureTimes == 0) return;
        if (hintState == HintState.none) return;
        CommonAnimation animation = root.findActor("hint_animation");
        Group hintGroup = root.findActor("btn_hint");
        animation.getState().setAnimation(0, "animation", true);
        hintGroup.addAction(Actions.forever(Actions.sequence(Actions.scaleTo(1.05F, 1.05F, 0.4F), Actions.scaleTo(1, 1, 0.4F))));
    }

    private void hintAniReset() {
        CommonAnimation animation = root.findActor("hint_animation");
        animation.getState().setAnimation(0, "animation2", true);
        root.findActor("btn_hint").getActions().clear();
    }


    @Override
    public void update() {
        if (gamePanel == null) return;
        float progress = gamePanel.inkLeftPercent();
        progressBar.update(progress);
        if (progress < 0) progress = 0;
        if (progress < 0.7 && star == 3) {
            star--;
            starAnimation[2].getState().setAnimation(0, "animation", false);
        }
        if (progress < 0.4 && star == 2) {
            star--;
            starAnimation[1].getState().setAnimation(0, "animation", false);
        }
        GameInfo.star = star;
        updateHintBtn();
    }

    @Override
    public void setGamePanel(GamePanel gamePanel) {
        super.setGamePanel(gamePanel);
        if (GameInfo.chapter == 1 && GameInfo.id == 1) {
            initCourse();
        }
    }

    public void end(){
        showHand(false);
        hintAniReset();
    }

    private synchronized void updateHintBtn() {
        hintBtn.findActor("hint_animation").setVisible(true);
        if (Configuration.settingData.getCoin() >= GameInfo.levelData.getHintPrice()) {
            hintVideo.setVisible(false);
            hintCoin.setVisible(true);
            hintState = HintState.coin;
        } else {
            hintVideo.setVisible(true);
            hintCoin.setVisible(false);
            if (Configuration.videoAdReady) {
                hintVideoGray.setVisible(false);
                hintVideo.setTouchable(Touchable.enabled);
                hintState = HintState.video;

            } else {
                hintVideoGray.setVisible(true);
                hintVideo.setTouchable(Touchable.childrenOnly);
                hintState = HintState.none;
                hintBtn.findActor("hint_animation").setVisible(false);
            }
        }
    }

    public void success() {
        screen.showSuccessDialog();
        if (!Configuration.settingData.isRate()) {
            for (int i = 0; i < Constant.rateLevel.length; i = i + 2) {
                if (GameInfo.chapter == Constant.rateLevel[i] && GameInfo.id == Constant.rateLevel[i + 1]) {
                    screen.showRateDialog();
                }
            }
        }
        FlurryUtils.level_Win(GameInfo.chapter, GameInfo.id);
        if (GameInfo.star == 3) {
            FlurryUtils.level_3Stars(GameInfo.chapter, GameInfo.id);
        }
        GameInfo.reset();
    }

    public void failure() {
        Configuration.retry_times_full_screen++;
        screen.end(GameScreen.class);
        GameInfo.failureTimes++;
        FlurryUtils.level_Fail(GameInfo.chapter, GameInfo.id);
        if (Configuration.settingData.isFirstPlay()) {
            FlurryUtils.level_FirstRestart(GameInfo.chapter, GameInfo.id);
        }
        screen.showFullScreenAd(true);
    }

    public enum HintState {
        none, coin, video
    }
}
