
package com.zyb.loveball;

import com.badlogic.gdx.*;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.g2d.*;
import com.badlogic.gdx.graphics.glutils.FrameBuffer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Json;
import com.badlogic.gdx.utils.viewport.ExtendViewport;
import com.badlogic.gdx.utils.viewport.StretchViewport;
import com.badlogic.gdx.utils.viewport.Viewport;
import com.esotericsoftware.spine.SkeletonBinary;
import com.esotericsoftware.spine.SkeletonData;
import com.esotericsoftware.spine.SkeletonJson;
import com.esotericsoftware.spine.SkeletonRenderer;
import com.rube.JsonLoader;
import com.rube.WorldData;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.audio.AudioProcess;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.screens.LoadScreen;
import com.zyb.loveball.screens.MenuScreen;
import com.zyb.loveball.spine.MAtlasAttachmentLoader;
import com.zyb.loveball.utils.FlurryUtils;

import com.zyb.loveball.utils.ImageRelate;
import com.zyb.loveball.utils.Log;
import org.freyja.libgdx.cocostudio.ui.CocoStudioUIEditor;

import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.HashSet;

public class HelloZombieGame extends Game {
    private static Viewport viewport;
    private static InputMultiplexer inputMultiplexer;
    private static FrameBuffer frameBuffer;
    private static Batch batch;
    private static ShapeRenderer shapeRenderer;
    private static CocoStudioUIEditor cocoStudioUIEditor;
    private static Json json;
    public static LauncherListener launcherListener;
    private static SkeletonRenderer skeletonRenderer;
    private static SkeletonJson skeletonJson;
    private static SkeletonBinary skeletonBinary;
    private Class preScreen;
    public static ImageRelate imageUtil;
    private boolean autoPauseInput = false;
    private int pendingActionState;
    public HelloZombieGame(LauncherListener launcherListener) {
        this.launcherListener = launcherListener;
    }

    @Override
    public void create() {
        Configuration.prepare();
        AudioProcess.prepare();
        GameInfo.init();
        Assets.prepare();
        inputMultiplexer = new InputMultiplexer();
        if (Gdx.graphics.getWidth() / (float) Gdx.graphics.getHeight() > 480F / 800F) {
            viewport = new StretchViewport(480, 800);
        } else {
            viewport = new ExtendViewport(480, 800, 480, 1100);
        }
        setScreen(new LoadScreen(this));
        Gdx.input.setCatchBackKey(true);
        update();
        Configuration.width = (int)viewport.getWorldWidth();
        Configuration.height = (int)viewport.getWorldHeight();
    }
    @Override
    public void render() {
        Gdx.gl.glClearColor(0F, 0F, 0F, 1f);
        if(Configuration.poor){
            Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        }else {
//            Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
            Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT | GL20.GL_DEPTH_BUFFER_BIT | (Gdx.graphics.getBufferFormat().coverageSampling ? GL20.GL_COVERAGE_BUFFER_BIT_NV : 0));
        }
        super.render();
        AudioProcess.update();
        float deltaTime = Gdx.graphics.getDeltaTime();
        Configuration.game_time+=deltaTime;
        Configuration.full_screen_time +=deltaTime;
    }

    public static Viewport getViewport() {
        return viewport;
    }


    public static void pauseInputProcessor() {
        Gdx.input.setInputProcessor(null);
    }

    public static void clearInputProcessor() {
        Gdx.app.postRunnable(new Runnable() {
            @Override
            public void run() {
                inputMultiplexer.clear();
            }
        });
    }

    public static void addInputProcessor(InputProcessor inputProcessor) {
        inputMultiplexer.addProcessor(inputProcessor);
    }

    public static void resumeInputProcessor() {
        Gdx.input.setInputProcessor(inputMultiplexer);
    }

    public static FrameBuffer getFrameBuffer() {
        if(Configuration.poor) return null;
        if (frameBuffer == null) {
            Log.log("HelloZombie","getFrameBuffer()---> new frame buffer" );
            frameBuffer = new FrameBuffer(Pixmap.Format.RGBA8888, Configuration.width, Configuration.height, false);
        }
        return frameBuffer;
    }

    public static Batch getBatch() {
        if (batch == null) {
            batch = new CpuPolygonSpriteBatch();
         //   batch = new PolygonSpriteBatch();
        }
        return batch;
    }

    public static ShapeRenderer getShapeRenderer() {
        if (shapeRenderer == null) {
            shapeRenderer = new ShapeRenderer();
            shapeRenderer.setAutoShapeType(true);
            shapeRenderer.setColor(Color.BLACK);
            shapeRenderer.setAutoShapeType(true);
        }
        return shapeRenderer;
    }

    public static SkeletonRenderer getSkeletonRenderer() {
        if (skeletonRenderer == null) {
            skeletonRenderer = new SkeletonRenderer();
        }
        return skeletonRenderer;
    }

    public static SkeletonJson getSkeletonJson() {
        if (skeletonJson == null) {
            skeletonJson = new SkeletonJson(new MAtlasAttachmentLoader(Assets.instance.animation));
        }
        return skeletonJson;
    }

    public static SkeletonBinary getSkeletonBinary() {
        if(skeletonBinary == null){
            skeletonBinary = new SkeletonBinary(new MAtlasAttachmentLoader(Assets.instance.animation));
        }
        return skeletonBinary;
    }

    public void update() {
        Screen screen = getScreen();
        if (screen != null) {
            ((BaseScreen) screen).update();
        }
    }

    public static Json getJson() {
        if (json == null) {
            json = new Json();
        }
        return json;
    }

    @Override
    public void pause() {
        Log.log("HelloZombieGame","pause()");
        super.pause();
        if(frameBuffer!=null){
            frameBuffer.dispose();
            frameBuffer = null;
        }
        Configuration.saveData();
        if(Gdx.input.getInputProcessor() !=null){
            pauseInputProcessor();
            autoPauseInput = true;
        }else{
            autoPauseInput = false;
        }
    }

    @Override
    public void resume() {
        super.resume();
        if(autoPauseInput){
            resumeInputProcessor();
        }
    }

    @Override
    public void setScreen(Screen screen) {
        if(getScreen()!=null) {
            preScreen = getScreen().getClass();
        }
        super.setScreen(screen);
        Configuration.saveData();
        update();
    }

    public Class getPreScreen() {
        return preScreen;
    }

    public void handlePendingAction(final boolean done) {
        try {
            Log.log("HelloZombieGame", "handlePendingAction()----> pendingAction : " + ((BaseScreen) screen).getPendingAction() + "  done : " + done);
            if (done) {
                pendingActionState = 1;
            } else {
                pendingActionState = -1;
            }

            Gdx.app.postRunnable(new Runnable() {
                @Override
                public void run() {
                    update();
                }
            });
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    public void unityAdReady(boolean ready) {
        if(ready){
            FlurryUtils.Ads_Reward("Ready");
        }
        Configuration.videoAdReady = ready;
        update();
    }

    public static CocoStudioUIEditor getCocoStudioUIEditor() {
        if (cocoStudioUIEditor == null) {
            HashSet<TextureAtlas> set = new HashSet<TextureAtlas>();
            set.add(Assets.instance.ui);
            set.add(Assets.instance.game);
            HashMap<String, BitmapFont> fonts = new HashMap<String, BitmapFont>();
            for (int i = 0; i < Assets.instance.fonts.length; i++) {
                fonts.put("font/" + Assets.FONT_NAME[i], Assets.instance.fonts[i]);
            }
            cocoStudioUIEditor = new CocoStudioUIEditor(null, fonts, null, set);
            cocoStudioUIEditor.setDirName("ui/");
        }
        return cocoStudioUIEditor;
    }

    @Override
    public void dispose() {
        Log.log("HelloZombieGame","dispose()");
        if(batch!=null){
            batch.dispose();
            batch = null;
        }
        if(shapeRenderer!=null){
            shapeRenderer.dispose();
            shapeRenderer = null;
        }

        if(frameBuffer!=null){
            frameBuffer.dispose();
            frameBuffer = null;
        }

        skeletonJson = null;
        if(shapeRenderer!=null){
            shapeRenderer.dispose();
            shapeRenderer = null;
        }

        if(screen!=null){
            screen.dispose();
        }
//        Assets.dispose();
    }


    public int getPendingActionState() {
        return pendingActionState;
    }

    public void setPendingActionState(int pendingActionState) {
        this.pendingActionState = pendingActionState;
    }
}