package com.zyb.loveball;


import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.*;
import com.badlogic.gdx.utils.ArrayMap;
import com.badlogic.gdx.utils.FloatArray;
import com.badlogic.gdx.utils.ObjectMap;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.objects.*;
import com.zyb.loveball.utils.HintUtil;
import com.zyb.loveball.utils.Log;

import java.util.Iterator;

public class PaintPanel {
    private PromptLine promptLine;
    private ArrayMap<Body, Line> lineBodies = new ArrayMap<>();
    private Line tmpLine;
    private float lastX, lastY;
    boolean created = false;
    GamePanel gamePanel;
    Vector2 worldPosition = new Vector2();
    boolean painting;
    public final Pen pen;
    Hint hint;
    NinePatch linePatch;
    private float hintX, hintY;
    boolean adapt = false;


    public PaintPanel(GamePanel gamePanel) {
        this.gamePanel = gamePanel;
        init();
        hint = new Hint();
        int inkAll = 100;
        int inkLeft = 100;
        if (gamePanel.levelData != null) {
            inkAll = gamePanel.levelData.getInkAll();
            inkLeft = gamePanel.levelData.getInkInitial();
        }
        pen = new Pen(inkAll, inkLeft);
    }

    private void init() {
        promptLine = new PromptLine(0, 0, 0, 0);
        int pendId = Configuration.settingData.getPenId();
        TextureRegion tmpRegion = Assets.instance.game.findRegion(Assets.instance.getGoodData(pendId, 0).getResources() + "_line");
        if (tmpRegion == null) {
            tmpRegion = Assets.instance.game.findRegion("pen_lanqianbi_line");
        }
        linePatch = new NinePatch(tmpRegion, 8, 8, 0, 0);
        if(Configuration.debug) {
            InputAdapter keyAdapter = new InputAdapter() {
                @Override
                public boolean keyUp(int keycode) {
                    if (keycode == Input.Keys.S) {
                        if (tmpLine == null) return false;
                        hint.addHint(HintUtil.saveHintVertexes(tmpLine.getVertexes(), hintX, hintY));
                    } else if (keycode == Input.Keys.R) {
                        hint.clearHint();
                        hint.reset();
                    }
                    return super.keyUp(keycode);
                }
            };
            HelloZombieGame.addInputProcessor(keyAdapter);
        }
    }


    public void start(float x, float y) {
        if (!pen.canWrite()) return;
        updateWorldPosition(x, y);
        created = false;
        adapt = false;
        pen.begin(worldPosition.x, worldPosition.y);
        if (canDraw()) {
            createLine(worldPosition.x, worldPosition.y);
            created = true;
            addVertex(worldPosition.x, worldPosition.y);
        }
        painting = true;
        GamePanel.GameTime.unActiveTime = 0;
    }

    private boolean canDraw() {
        if (gamePanel.isInBodies(worldPosition.x - Constant.LINESIZE, worldPosition.y - Constant.LINESIZE))
            return false;
        if (gamePanel.isInBodies(worldPosition.x - Constant.LINESIZE, worldPosition.y + Constant.LINESIZE))
            return false;
        if (gamePanel.isInBodies(worldPosition.x + Constant.LINESIZE, worldPosition.y - Constant.LINESIZE))
            return false;
        if (gamePanel.isInBodies(worldPosition.x + Constant.LINESIZE, worldPosition.y + Constant.LINESIZE))
            return false;
        return true;
    }


    private void updateWorldPosition(float x, float y) {
        worldPosition.x = x * Constant.SCALE;
        worldPosition.y = y * Constant.SCALE;
        gamePanel.updateWorldPosition(worldPosition);
        pen.update(worldPosition.x, worldPosition.y);
        promptLine.update(lastX, lastY, worldPosition.x, worldPosition.y);
    }

    public void painting(float x, float y) {
        if (!painting) return;
        updateWorldPosition(x, y);
        if (!created && canDraw()) {
            createLine(worldPosition.x, worldPosition.y);
            created = true;
            addVertex(worldPosition.x, worldPosition.y);
        }
        if (created) {
            if (!isOverlapping(worldPosition.x, worldPosition.y)) {
                addVertex(worldPosition.x, worldPosition.y);
            }
        }
    }

    public void end(float x, float y) {
        updateWorldPosition(x, y);
        if (hint.isShowHint() && adapt && created) {
            adaptHintLine();
        }
        if (created)
            hint.nextHint();
        end();
    }

    public void end() {
        promptLine.setVisible(false);
        if (created) {
            if (tmpLine.getVertexes().size < 4) {
                gamePanel.removeObject(tmpLine.getBody());
            } else {
                tmpLine.active();
                gamePanel.start();
            }
        }
        created = false;
        painting = false;
        pen.end();
        GamePanel.GameTime.unActiveTime = 0;
        gamePanel.paintEnd();
    }

    private void adaptHintLine() {
        FloatArray floatArray = hint.getHintLine().getVertexes();
        if (floatArray.size >= 2) {
            tmpLine.clearAll();
            for (int i = 0; i < floatArray.size; i += 2) {
                tmpLine.addVertex(floatArray.get(i), floatArray.get(i + 1));
            }
        }
    }

    private void addVertex(float x, float y) {
        if (Math.abs(worldPosition.x - lastX) + Math.abs(worldPosition.y - lastY) < Constant.MINBOXLENGTH)
            return;
        if (!pen.paint(x, y)) {
            end(worldPosition.x / Constant.SCALE, worldPosition.y / Constant.SCALE);
            return;
        }
        lastX = x;
        lastY = y;
        tmpLine.addVertex(x, y);
        gamePanel.updateUI();
        if (hint.hit(x, y)) {
            adapt = true;
        }
    }

    private void createLine(float xWorld, float yWorld) {
        if (!pen.canWrite()) return;
        tmpLine = new Line(gamePanel);
        tmpLine.setBodyPosition(xWorld, yWorld);
        lineBodies.put(tmpLine.getBody(), tmpLine);
        hintX = xWorld;
        hintY = yWorld;
        promptLine.setVisible(true);
    }


    boolean overLapping;

    private synchronized boolean isOverlapping(float x, float y) {
        float offsetX = x - lastX;           // x改变量
        float offsetY = y - lastY;           // y改变量
        float length = (float) Math.sqrt(offsetX * offsetX + offsetY * offsetY);
        float smallX = Constant.CHECK_SIZE * offsetX / length;
        float smallY = Constant.CHECK_SIZE * offsetY / length;
        float changeXMath = (Constant.CHECK_SIZE) / length * offsetY;
        float changeYMath = -(Constant.CHECK_SIZE) / length * offsetX;
        if (length > 0) {
            overLapping = false;
            gamePanel.getWorld().rayCast(lineCheck, lastX - changeXMath, lastY - changeYMath, x + smallX - changeXMath, y + smallY - changeYMath);
            gamePanel.getWorld().rayCast(lineCheck, lastX + changeXMath, lastY + changeYMath, x + smallX + changeXMath, y + smallY + changeYMath);
        }
        if (overLapping) {
            float checkX = lastX + 2 * smallX;
            float checkY = lastY + 2 * smallY;
            float i = 0;
            while (i < length) {
                if (gamePanel.isInBodies(checkX - changeXMath, checkY - changeYMath) || gamePanel.isInBodies(checkX + changeXMath, checkY + changeYMath)) {
                    addVertex(checkX - 2 * smallX, checkY - 2 * smallY);
                    break;
                }
                checkX += smallX;
                checkY += smallY;
                i += Constant.CHECK_SIZE;
            }
        }
        return overLapping;
    }

    RayCastCallback lineCheck = new RayCastCallback() {
        @Override
        public float reportRayFixture(Fixture fixture, Vector2 point, Vector2 normal, float fraction) {
            String name = gamePanel.scene.getName(fixture.getBody());
            if (name != null && (name.equals("fan") || name.equals("anti_gravity"))) {
                return 1;
            }
            overLapping = true;
            return 1;
        }
    };

    public void clearLines() {
        Iterator<ObjectMap.Entry<Body, Line>> iterator = lineBodies.iterator();
        while (iterator.hasNext()) {
            ObjectMap.Entry<Body, Line> entry = iterator.next();
            gamePanel.destroyBody(entry.key);
        }
        lineBodies.clear();
        pen.reset(gamePanel.levelData.getInkInitial());
        gamePanel.updateUI();
    }

    public void removeObject(Body body) {
        if (tmpLine != null && tmpLine.getBody() == body) {
            tmpLine = null;
            created = false;
        }
        lineBodies.removeKey(body);
    }

    public void draw(Batch batch) {
        Iterator<ObjectMap.Entry<Body, Line>> iterator = lineBodies.iterator();
        while (iterator.hasNext()) {
            ObjectMap.Entry<Body, Line> entry = iterator.next();
            Line line = entry.value;
            if (!line.draw(batch, linePatch)) {
                gamePanel.destroyBody(entry.key);
                iterator.remove();
            }
        }
        promptLine.draw(batch);
        hint.draw(batch);
        pen.draw(batch);
    }
}
