package com.zyb.loveball.animations;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Colors;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.esotericsoftware.spine.*;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.spine.MAtlasAttachmentLoader;

/**
 * Created by Doodle on 2018/10/17.
 */

public class SkeletonAnimation {
    private Skeleton skeleton;
    private AnimationState state;
    SkeletonRenderer renderer;
    float x, y;
    String name;
    float scale;
    float xOffset;
    float yOffset;
    private Color color = new Color(1,1,1,1);
    private String prefix = "";

    public SkeletonAnimation(String name) {
        this.name = name;
        String[]temp = name.split("/");
        String str = temp[1];
        this.prefix = str.substring(0,str.length()-5);
        init();
    }

    public SkeletonAnimation(String name,float scale){
        this.name = name;
        this.scale = scale;
        String[]temp = name.split("/");
        String str = temp[1];
        this.prefix = str.substring(0,str.length()-5);
        init();
    }

    public void setPosition(float x,float y){
        this.x = x;
        this.y = y;
        skeleton.setPosition(x+xOffset, y+yOffset);
    }

    public void setColor(float r,float g,float b,float a) {
        this.color.set(r,g,b,a);
        skeleton.setColor(color);
    }

    public void setxOffset(float xOffset) {
        this.xOffset = xOffset;
        skeleton.setPosition(x+xOffset, y+yOffset);
    }

    public void setyOffset(float yOffset) {
        this.yOffset = yOffset;
        skeleton.setPosition(x+xOffset, y+yOffset);
    }



    private void init() {
        MAtlasAttachmentLoader.prefix =prefix+"_";
        renderer = HelloZombieGame.getSkeletonRenderer();
        SkeletonBinary skeletonJson = HelloZombieGame.getSkeletonBinary();
        skeletonJson.setScale(scale);
        SkeletonData data = skeletonJson.readSkeletonData(Gdx.files.internal(name));
        AnimationStateData animationStateData = new AnimationStateData(data);
        state = new AnimationState(animationStateData);
        skeleton = new Skeleton(data);
        skeleton.setPosition(x, y);
        MAtlasAttachmentLoader.prefix ="";
    }

    public void setX(float x) {
        this.x = x;
        skeleton.setX(x);
    }

    public void setY(float y) {
        this.y = y;
        skeleton.setY(y);
    }

    public void update(){
        state.update(Gdx.graphics.getDeltaTime());
        state.apply(skeleton);
        skeleton.updateWorldTransform();
    }


    public AnimationState getState() {
        return state;
    }

    public void draw(Batch batch) {
        renderer.draw((PolygonSpriteBatch)batch, skeleton);
//        batch.setColor(Color.WHITE);
    }
}
