package com.zyb.loveball.assets;

import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.Log;

public class SettingData {
    private int stars[][] = new int[50][];
    private int coin = 0;
    private int penId = 1;
    private int ballId = 1;
    private boolean[] pens = new boolean[12];
    private boolean[] balls = new boolean[20];
    private boolean[] challenge = new boolean[20];
    private boolean sound;
    private boolean music;
    private boolean notification;
    private int lastLevelInt;
    private int lastChapterInt;
    private int unlockChapter;
    private int unlockLevel;
    private boolean firstPlay =true;
    private boolean rate = false;

    public SettingData() {
        initInt(stars, 31, 0);
        stars[1][1] = 0;
        pens[0] = true;
        balls[0] = true;
        sound = true;
        music = true;
        notification = true;
        coin = 0;
        lastLevelInt = 1;
        lastChapterInt = 1;
        unlockChapter = 1;
        unlockLevel = 1;
    }

    private void initInt(int[][] temp, int size, int defaultValue) {
        for (int i = 0; i < temp.length; i++) {
            temp[i] = new int[size];
            for (int j = 0; j < temp[i].length; j++) {
                temp[i][j] = defaultValue;
            }
        }
    }

    private void initFloat(float[] temp,float defaultValue){
        for (int i = 0;i<temp.length;i++){
            temp[i] = defaultValue;
        }
    }

    public void unlockNextLevel(int chapter,int level) {
        if(chapter == unlockChapter && level == unlockLevel) {
            unlockLevel++;
            unlockChapter += unlockLevel / 11;
            if (unlockLevel >= 11) {
                unlockLevel = unlockLevel - 10;
            }
        }
    }

    public boolean isRate() {
        return rate;
    }

    public void setRate(boolean rate) {
        this.rate = rate;
    }

    public int getUnlockChapter() {
        return unlockChapter;
    }

    public int getUnlockLevel() {
        return unlockLevel;
    }

    public int getLastLevelInt() {
        return lastLevelInt;
    }

    public int getLastChapterInt() {
        return lastChapterInt;
    }

    public void setLastLevelInt(int lastChapter,int lastLevelInt) {
        this.lastChapterInt = lastChapter;
        this.lastLevelInt = lastLevelInt;
    }
    public void setSound(boolean sound) {
        this.sound = sound;
    }

    public void setMusic(boolean music) {
        this.music = music;
    }

    public boolean isMusic() {
        return music;
    }

    public boolean isSound() {
        return sound;
    }

    public boolean isNotification() {
        return notification;
    }

    public void setNotification(boolean notification) {
        this.notification = notification;
    }


    public int getPenId() {
        return penId;
    }

    public int getBallId() {
        return ballId;
    }

    public boolean isGoodUnlocked(int id, int type) {
        if (id < 1) {
            Log.log("SettingData", "get good state error,id can't <1, id = " + id + " type = " + type);
            return false;
        }
        try {
            if (type == 0) {
                return pens[id - 1];
            } else {
                return balls[id - 1];
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            e.printStackTrace();
            if (type == 0) {
                pens = resizeBoolean(pens);
            } else {
                balls = resizeBoolean(balls);
            }
            return isGoodUnlocked(id, type);
        }

    }

    private boolean[] resizeBoolean(boolean[] objects) {
        boolean[] temp = objects;
        objects = new boolean[temp.length * 2];
        for (int i = 0; i < temp.length; i++) {
            objects[i] = temp[i];
        }
        return objects;
    }

    public int getGoodId(int type) {
        if (type == 0) return penId;
        return ballId;
    }

    public void setGoodId(int id, int type) {
        if (type == 0) {
            penId = id;
            FlurryUtils.Skin_Pen(id);
        }
        else {
            ballId = id;
            FlurryUtils.Skin_Bomb(id);
        }
    }

    public void setPenId(int penId) {
        this.penId = penId;
    }

    public void setBallId(int ballId) {
        this.ballId = ballId;
    }

    public int getStars(int chapter, int level) {
        if (level >= stars[chapter].length) {
            int temp[] = new int[level + 10];
            for (int i = 0; i < stars[chapter].length; i++) {
                temp[i] = stars[chapter][i];
            }
            stars[chapter] = temp;
        }
        return stars[chapter][level];
    }

    public int getStarsChapter(int chapter) {
        int total = 0;
        for (int j = 0; j < stars[chapter].length; j++) {
            total += stars[chapter][j];
        }
        return total;
    }

    public void setStar(int chapter, int level, int value) {
        if (value > stars[chapter][level]) {
            stars[chapter][level] = value;
        }
    }

    public boolean cost(int cost) {
        if (coin >= cost) {
            coin -= cost;
            Configuration.saveData();
            return true;
        }
        return false;
    }

    public boolean isChallengePassed(int id){
        return challenge[id-1];
    }

    public void challengePass(int id){
        challenge[id-1] = true;
    }

    public void unlockGood(int id, int type) {
        if (id < 1) {
            Log.log("SettingData", "get good state error,id can't <1, id = " + id + " type = " + type);
        }
        if (type == 0) {
            pens[id - 1] = true;
        } else {
            balls[id - 1] = true;
        }
        setGoodId(id, type);
        Configuration.saveData();
    }

    public int getCoin() {
        if(coin>9999999){
            return 9999999;
        }
        return coin;
    }

    public void addCoin(int offset) {
        if (offset > 0) {
            coin += offset;
            Configuration.saveData();
        }
    }

    public void setFirstPlay(boolean firstPlay) {
        this.firstPlay = firstPlay;
    }

    public boolean isFirstPlay() {
        return firstPlay;
    }

    @Override
    public String toString() {
        return HelloZombieGame.getJson().toJson(this);
    }

}
