package com.zyb.loveball.audio;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.utils.Log;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;

public class AudioProcess {

    public static HashMap<String, SoundAsset> soundAssets = new HashMap<>();
    public static HashMap<String, MusicAsset> musicAssets = new HashMap<>();
    public static HashMap<String, String> musics = new HashMap<>();
    private static String currMusic = "";
    private static HashSet<String> currSoundLoop = new HashSet<>();
    private static boolean onFocus = true;
    private static boolean prepared = false;

    public static void prepare() { // 准备哪些资源将要加载
        clear();
        loadMusicFromData();
        soundAssets.put("UI", Asset.registerSoundAsset("sfx/UI.ogg"));
        soundAssets.put("draw", Asset.registerSoundAsset("sfx/draw.ogg"));
        soundAssets.put("bomb", Asset.registerSoundAsset("sfx/bomb.ogg"));
        soundAssets.put("money", Asset.registerSoundAsset("sfx/money.ogg"));
        soundAssets.put("win1", Asset.registerSoundAsset("sfx/winA-01.ogg"));
        soundAssets.put("win2", Asset.registerSoundAsset("sfx/winA-02.ogg"));
        soundAssets.put("win3", Asset.registerSoundAsset("sfx/winA-03.ogg"));
        musicAssets.put("bgm", Asset.registerMusicAsset("music/bgm.ogg"));
        soundQueue = new ArrayBlockingQueue<>(2);// 4
        new SoundThread().start();
        prepared = true;

    }

    public static boolean isPrepared() {
        return prepared;
    }

    public static void setOnFocus(boolean flag) {
        Log.log("AudioProcess", "setOnFocus()--->" + flag);
        onFocus = flag;
        if (!prepared) return;
        if (onFocus) {
            resumeMusic();
            resumeAllSound();
        } else {
            pauseAllSound();
            pauseAllMusic();
        }
    }

    public static void setPitch(long soundId, float pitch, String name) {
        SoundAsset sa = getSoundAsset(name);
        if (sa != null) {
            sa.setPitch(soundId, pitch);
        }
    }

    public static void setVolume(long soundId, float pitch, String name) {
        SoundAsset sa = getSoundAsset(name);
        if (sa != null) {
            sa.setVolume(soundId, pitch);
//            sa.setPitch(soundId, pitch);
        }
    }


    public static SoundAsset getSoundAssets(String name) {
        return soundAssets.get(name);
    }

    static class SoundThread extends Thread {
        @Override
        public void run() {
            super.run();
            loadSound();
        }
    }


    public static void loadSound() {
        Iterator<SoundAsset> iterator = soundAssets.values().iterator();
        while (iterator.hasNext()) {
            iterator.next().load();
        }
    }

    private static long soundStartTime = 0;
    private static final long soundTimeSpan = 100;

    public static Queue<String> soundQueue;

    public static void playSound(String name, float v) {
        if (name != null) {
            playsound(name, false);
        }
    }


    public static long playSoundLoop(String name,float v) {
        stopSoundLoop(name);
        if (name != null) {
            if (currSoundLoop.add(name)) {
                return playsound(name, true,v);
            }
        }
        return -1;
    }

    public static void stopSoundLoop(String name) {
        currSoundLoop.remove(name);
        if (name != null) {
            if (soundAssets.get(name) != null) {
                soundAssets.get(name).stop();
            }
        }
    }


    public static void stopAllSound() {
        Iterator<String> iterator = currSoundLoop.iterator();
        while (iterator.hasNext()) {
            String soundName = iterator.next();
            if (soundName != null) {
                if (soundAssets.get(soundName) != null) {
                    soundAssets.get(soundName).stop();
                }
            }
        }
        currSoundLoop.clear();
    }


    public static void pauseAllSound() {
        Iterator<String> iterator = currSoundLoop.iterator();
        while (iterator.hasNext()) {
            String soundName = iterator.next();
            if (soundName != null) {
                if (soundAssets.get(soundName) != null) {
                    soundAssets.get(soundName).pause();
                }
            }
        }
    }


    public static void resumeAllSound() {
        Iterator<String> iterator = currSoundLoop.iterator();
        while (iterator.hasNext()) {
            String soundName = iterator.next();
            if (soundName != null) {
                if (soundAssets.get(soundName) != null) {
                    soundAssets.get(soundName).resume();
                }
            }
        }
    }

    public static void playAllSound() {
        Iterator<String> iterator = currSoundLoop.iterator();
        while (iterator.hasNext()) {
            String soundName = iterator.next();
            if (soundName != null) {
                if (soundName.contains("draw")) {
                    playsound(soundName, true);
                } else {
                    playsound(soundName, true);
                }
            }
        }
    }

    private static long playsound(String name, boolean loop) {
        if (soundQueue == null)
            return -1;
        if (!Configuration.settingData.isSound()) return -1;
        SoundAsset sa = getSoundAsset(name);
        if (!loop) {
            if (!soundQueue.offer(name)) {
                return -1;
            }
        } else {
            return sa.loop();
        }
        return -1;
    }

    private static long playsound(String name, boolean loop,float v) {
        if (soundQueue == null)
            return -1;
        if (!Configuration.settingData.isSound()) return -1;
        SoundAsset sa = getSoundAsset(name);
        if(sa!=null){
            sa.setVolume(v);
        }
        if (!loop) {
            if (!soundQueue.offer(name)) {
                return -1;
            }
        } else {
            return sa.loop();
        }
        return -1;
    }


    public static void update() {
        if (soundQueue.isEmpty())
            return;
        if (System.currentTimeMillis() - soundStartTime > soundTimeSpan) {
            String name = soundQueue.poll();// soundQueue.clear();
            SoundAsset sa = getSoundAsset(name);
            if (sa != null) {
                sa.play();
            }
            soundStartTime = System.currentTimeMillis();
        }
    }

    private static SoundAsset getSoundAsset(String name) {
        return soundAssets.get(name);

    }

    public static void reset() {
        if (soundQueue != null) {
            soundQueue.clear();
        }
    }

    public static void clear() {
        if (soundQueue != null) {
            soundQueue.clear();
        }
        soundAssets.clear();
        musicAssets.clear();
        prepared = false;
        currMusic = "";
    }

    public static void playMusicLoop(String name) {
        if (currMusic.equals(name))
            return;
        stopMusic(currMusic);
        currMusic = name;
        playmusicLoop(name);
    }

    public static void playMusicLoop() {
        if (currMusic == null || "".equals(currMusic)) {
            return;
        }
        playmusicLoop(currMusic);

    }


    private static void playmusicLoop(String musicName) {
        if (!Configuration.settingData.isMusic()) return;
        if (musicAssets.get(musicName) != null) {
            musicAssets.get(musicName).load();
            musicAssets.get(musicName).playMusicLoop();
        } else {
            System.out.println("can not find music: " + musicName);
        }
    }

    public static void resumeMusic() {
        if (currMusic == null || currMusic.equals("")) return;
        if (!Configuration.settingData.isMusic()) return;
        Log.log("AudioProcess", "resumemusic()---->");
        musicAssets.get(currMusic).resumeMusic();
    }

    private static void pausemusic(String musicName) {
        musicAssets.get(musicName).pauseMusic();
    }

    public static void stopMusic() {
        if (currMusic != null) {
            stopmusic(currMusic);
            //           currMusic = "";
        }
    }

    public static void stopMusic(String name) {
        if (!currMusic.equals(name))
            return;
        String musicName = musics.get(name);
        if (musicName != null) {
            stopmusic(musicName);
        }

    }

    private static void stopmusic(String musicName) {
        if (musicAssets.get(musicName) != null) {
            musicAssets.get(musicName).stopMusic();
            musicAssets.get(musicName).dispose(Assets.instance.assetManager);
        }
    }

    public static void pauseAllMusic() {
        Iterator<MusicAsset> iteratorMusic = musicAssets.values().iterator();
        while (iteratorMusic.hasNext()) {
            iteratorMusic.next().pauseMusic();
        }
    }

    public static void stopAllMusic() {
        Iterator<MusicAsset> iteratorMusic = musicAssets.values().iterator();
        while (iteratorMusic.hasNext()) {
            iteratorMusic.next().stopMusic();
        }
//        currMusic = "";
    }

    public static FileHandle[] getAllSoundName() {
        return Gdx.files.internal("sfx").list();
    }

    public static FileHandle[] getAllMusciName() {
        return Gdx.files.internal("music").list();
    }

    public static void destory() {
        for (int i = 0; i < musicAssets.size(); i++) {
            musicAssets.get(i).music.dispose();
        }

        for (int i = 0; i < soundAssets.size(); i++) {
            soundAssets.get(i).sound.dispose();
        }
    }

    private static void loadMusicFromData() {
//        CsvReader reader = null; // 定义Csvreader
        //       reader = new CsvReader(Gdx.files.internal("sfx/audio.csv").reader());
        try {
//            reader.readHeaders();
//            while (reader.readRecord()) {
//                String str = null;
//
//                String key = reader.get("key");
//                String audio = reader.get("audio");
//                if (key == null || key.equals("") || audio == null || audio.equals("")) {
////					Log.log("AudioProcess", "loadMusicFromData: load audio name failed:" + key + "   " + audio);
//                }
//                sound.put(key, "sfx/" + audio);
//
//            }
            musics.put("bgm", "music/bgm.ogg");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
