package com.zyb.loveball.audio;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.math.MathUtils;

public class SoundAsset extends Asset {

    public Sound sound;
    float v = 1f;
    boolean load = false;
    float factor = 1F;
    public SoundAsset(Sound sound, String name) {
        this.sound = sound;
        this.name = name;
    }

    public SoundAsset(String name) {
        this.name = name;
    }

    public void setVolume(float v) {
        this.v = MathUtils.clamp(v,0,1);
    }
    public void setVolume(long soundId,float v) {
        v = MathUtils.clamp(v,0,1);
        try {
            sound.setVolume(soundId,v);
        }catch (Exception e){
            e.printStackTrace();
        }
    }
    public long play() {
        if (sound != null) {
            try {
                return sound.play(v * factor);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return -1;
    }

    public void stop() {
        if (sound != null) {
            try {
                sound.stop();
                System.out.println("pause");
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }


    public void pause() {
        if (sound != null) {
            try {
                sound.pause();
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }

    public long loop() {
        if (sound != null) {
            try {
                return sound.loop(v * factor);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return -1;
    }

    public void resume(){
        if (sound != null) {
            try {
                sound.resume();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public long play(float v) {
        v = MathUtils.clamp(v,0,1);
        return this.sound.play(v * factor);
    }

    public void setPitch(long soundId, float pitch) {
        try {
            sound.setPitch(soundId, pitch);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    @Override
    public void load() {
        if (load)
            return;
        try {
            sound = Gdx.audio.newSound(Gdx.files.internal(name));
        } catch (Exception e) {
            e.printStackTrace();
        }
        load = true;
    }

    @Override
    public void loading(AssetManager assetManager) {
        // TODO Auto-generated method stub
        if (assetManager.isLoaded(name, Sound.class)) {
            assetManager.unload(name);
            assetManager.load(name, Sound.class);
        } else {
            assetManager.load(name, Sound.class);
        }
    }

    @Override
    public void finished(AssetManager assetManager) {
        // TODO Auto-generated method stub
        if (assetManager.isLoaded(name)) {
            sound = assetManager.get(name, Sound.class);
        }
    }

    @Override
    public void dispose(AssetManager assetManager) {
        // TODO Auto-generated method stub
        if (sound != null) {
            sound.dispose();
            sound = null;
            load = false;
        }
    }
}
