package com.zyb.loveball.dialogs;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.*;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.ui.Grid;
import com.zyb.loveball.utils.LayerUtils;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.listener.SoundButtonListener;

/**
 * Created by lilexiang on 17-8-23.
 */
public class BaseDialog {
    protected String TAG = "";
    protected Actor layer;
    Group group;
    DialogListener baseListener;
    public static Color color = new Color(232/255f,250/255F,233/255F,1);
    Label coinLabel;


    public void load(Group group) {
        this.group = group;
        Actor bg = group.findActor("bg");
        if(bg!=null){
            float bgWidth = bg.getWidth()/2;
            float bgHeight = bg.getHeight()/2;
            float centerX = bg.getX()+bg.getWidth()/2;
            float centerY = bg.getY()+bg.getHeight()/2;
            Grid grid = new Grid();
            grid.setNeedBg(true);
            grid.setScale(0.9F);
            grid.setColor(color);
            grid.setSize((bgWidth-8)/0.9F,(bgHeight-8)/0.9F);
            grid.setPosition(centerX - grid.getWidth()/2,centerY-grid.getHeight()/2);
            group.addActorAt(0,grid);
        }
        HelloZombieGame.pauseInputProcessor();
        layer =LayerUtils.createLayer(0.7F);
        initButtons();
        layout();
    }

    protected void layout(){

    }

    public void show(Group parent) {
        parent.addActor(layer);
        parent.addActor(group);
        group.setScale(0.3F, 0.3F);
        group.setTouchable(Touchable.childrenOnly);
        group.setOrigin(240, 400);
        group.addAction(Actions.sequence(Actions.scaleTo(1, 1, 0.1F), Actions.run(new Runnable() {
            @Override
            public void run() {
                HelloZombieGame.resumeInputProcessor();
            }
        })));
        Actor actor = group.findActor("coin_font");
        if(actor!=null){
            coinLabel = (Label)actor;
            Log.log("SuccessDialog","find coin label");
        }
    }

    public void update() {
        if(coinLabel!=null){
            coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
        }
    }

    protected void initButtons() {
        Actor btnClose = group.findActor("btn_close", Touchable.enabled);
        if(btnClose!=null) {
            btnClose.addListener(new SoundButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    close();
                }
            });
        }
        layer.addListener(new ClickListener(){
            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                super.touchUp(event, x, y, pointer, button);
                close();
            }
        });
    }

    public void close() {
        layer.remove();
        group.remove();
        if(baseListener!=null){
            baseListener.close();
        }
    }



    public static interface DialogListener {
        public void close();

        public void update(boolean updated);
        public void screenEnd(Class screen);
        public void showDialog(String attribute, Class<? extends BaseDialog> tClass);
        public void showStoreDialog();
    }

    public static class DialogAdapter implements DialogListener {
        @Override
        public void close() {

        }

        @Override
        public void update(boolean updated) {

        }

        @Override
        public void screenEnd(Class screen) {

        }

        @Override
        public void showDialog(String attribute, Class<? extends BaseDialog> tClass) {

        }

        @Override
        public void showStoreDialog() {

        }
    }

    public void handlePendingAction() {

    }

    public void setBaseListener(DialogListener listener) {
        baseListener = listener;
    }
}
