package com.zyb.loveball.dialogs;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.SpriteDrawable;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.esotericsoftware.spine.AnimationState;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.animations.CommonParticleEffect;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.assets.GoodData;
import com.zyb.loveball.audio.AudioProcess;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.screens.BaseScreen;
import com.zyb.loveball.screens.ChallengeScreen;
import com.zyb.loveball.screens.LevelScreen;
import com.zyb.loveball.screens.MenuScreen;
import com.zyb.loveball.screens.StoreScreen;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.MScreenUtil;
import com.zyb.loveball.utils.listener.ButtonListener;
import com.zyb.loveball.utils.listener.LClickListener;
import com.zyb.loveball.utils.listener.SoundButtonListener;

/**
 * Created by Doodle on 2018/9/13.
 */

public class SuccessDialog extends BaseDialog {
    SuccessDialogListener successDialogListener;
    Label coinRewardLabel;
    Actor videoCoinReward;
    Actor videoSkinReward;

    @Override
    public void show(Group parent) {
        super.show(parent);
        group.getActions().clear();
        initSettingData();
        group.setPosition(0, (HelloZombieGame.getViewport().getWorldHeight() - 800) / 2);
        Configuration.settingData.unlockNextLevel(GameInfo.chapter, GameInfo.id);
        AudioProcess.playSound("win" + GameInfo.star, 1);
        coinLabel = group.findActor("coin_font");
        initAnimation();
        videoCoinReward = group.findActor("video_coin_reward");
        videoSkinReward = group.findActor("video_skin_reward");
        coinRewardLabel = group.findActor("coin_value");
        group.findActor("buttons").setVisible(false);
        checkReward();
        group.addAction(Actions.sequence(Actions.scaleTo(1, 1, 0.1F),Actions.delay(0.5F),Actions.run(new Runnable() {
            @Override
            public void run() {
                String pngFileName = "level_pic_" + GameInfo.levelId;
                FileHandle file = Gdx.files.local(Constant.LEVEL_PIC_PATH + pngFileName + ".png");;
                MScreenUtil.writePngFromBuffer(file,Constant.SCREEN_SHOT_W_BIG,Constant.SCREEN_SHOT_H_BIG,true);
                HelloZombieGame.resumeInputProcessor();
            }
        })));

    }

    private void initSettingData() {
        int id = GameInfo.id + 1;
        int chapter = GameInfo.chapter;
        if (id > 10) {
            id = 1;
            chapter++;
            if (chapter == 31) {
                chapter = 1;
            }
        }
        Configuration.settingData.setLastLevelInt(chapter, id);
    }

    public void setSuccessDialogListener(SuccessDialogListener successDialogListener) {
        this.successDialogListener = successDialogListener;
    }

    private void checkReward() {
        coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
        videoSkinReward.setVisible(false);
        videoCoinReward.setVisible(false);
        int oldStar = Configuration.settingData.getStars(GameInfo.chapter, GameInfo.id);
        if(GameInfo.star>oldStar){
            Configuration.settingData.setStar(GameInfo.chapter,GameInfo.id,GameInfo.star);
        }
        if (oldStar == 0) {
            baseReward();
            extraReward();
        } else {
            noReward();
        }
    }

    private void extraReward() {
        if (!Configuration.videoAdReady) {
            group.findActor("buttons").setVisible(true);
            return;
        }
        int goodNum = Assets.instance.getGoodsNum(1);
        boolean coinEnough = false;
        for (int i = 1; i <= goodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 1)) continue;
            GoodData goodData = Assets.instance.getGoodData(i, 1);
            if (Configuration.settingData.getCoin() >= goodData.getCoin()) {
                coinEnough = true;
            }
            break;
        }
        Log.log("SuccessDialog","extraReward()----> coinEnough " + coinEnough);
        int rewardId = GameInfo.levelData.getPushId(coinEnough);
        Log.log("SuccessDialog","extraReward()----> rewardId " + rewardId);
        if (rewardId == 0) {
            group.findActor("buttons").setVisible(true);
            return;
        } else if (rewardId <= 3) {
            initCoinReward(rewardId);
        } else {
            initSkinReward(rewardId);
        }
        group.findActor("buttons").addAction(Actions.sequence(Actions.delay(1), Actions.visible(true)));
    }


    private void baseReward() {
        Configuration.settingData.addCoin(GameInfo.levelData.getReward());
        coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
        coinRewardLabel.setText(Configuration.numType.format(GameInfo.levelData.getReward()));
    }

    private void noReward() {
        group.findActor("coin_reward").setVisible(false);
        group.findActor("buttons").setVisible(true);
    }

    @Override
    public void update() {
    }

    int pendingReward;

    private void initCoinReward(final int rewardId) {
        if (rewardId == 1) {
            pendingReward = 10 * GameInfo.levelData.getReward();
            FlurryUtils.Event_CoinPush("Coin10");
        } else if (rewardId == 2) {
            pendingReward = 8 * GameInfo.levelData.getReward();
            FlurryUtils.Event_CoinPush("Coin8");
        } else if (rewardId == 3) {
            pendingReward = 5 * GameInfo.levelData.getReward();
            ;
            FlurryUtils.Event_CoinPush("Coin5");
        }
        Label label = group.findActor("coin_times");
        label.setText("+" + Configuration.numType.format(pendingReward));
        videoCoinReward.addListener(new ButtonListener() {
            @Override
            public void touchDownEffect() {
                super.touchDownEffect();
                successDialogListener.playVideo(BaseScreen.PendingAction.SUCCESS_COIN, pendingReward);
                FlurryUtils.Event_CoinClick("Coin" + (pendingReward));
                videoCoinReward.setVisible(false);
            }
        });

        videoCoinReward.setScale(4);
        videoCoinReward.addAction(Actions.sequence(Actions.delay(0.5F), Actions.visible(true), Actions.scaleTo(1, 1, 0.2F)));
    }

    private void initSkinReward(final int rewardId) {
        int goodNum = Assets.instance.getGoodsNum(1);
        int id = 1;
        for (int i = 1; i <= goodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 1)) continue;
            id = i;
            break;
        }
        int discountValue = 0;
        final GoodData goodData = Assets.instance.getGoodData(id, 1);
        if (rewardId == 4) {
            discountValue = (int) (0.85F * goodData.getCoin());
        } else if (rewardId == 5) {
            TextureRegion region = Assets.instance.ui.findRegion("discount_25");
            Image image = group.findActor("discount_pic");
            TextureRegionDrawable drawable = (TextureRegionDrawable) image.getDrawable();
            drawable.setRegion(region);
            image.setSize(region.getRegionWidth() / 2, region.getRegionHeight() / 2);
            discountValue = (int) (0.75F * goodData.getCoin());
        }
        ((Label) group.findActor("cost_money")).setText("" + discountValue);
        ((Label) group.findActor("cost_discount")).setText("" + goodData.getCoin());
        Image image = group.findActor("skin_mask");
        Sprite sprite = Assets.instance.ui.createSprite(goodData.getResources());
        SpriteDrawable spriteDrawable = new SpriteDrawable(sprite);
        image.setDrawable(spriteDrawable);
        videoSkinReward.addListener(new ButtonListener() {
            @Override
            public void touchDownEffect() {
                super.touchDownEffect();
                float discount = 1;
                if (rewardId == 4) {
                    discount = goodData.getCoin() * 0.85F;
                } else if (rewardId == 5) {
                    discount = goodData.getCoin() * 0.75F;
                }
                successDialogListener.playVideo(BaseScreen.PendingAction.BALL_UNLOCK, (int) discount);
                videoSkinReward.setVisible(false);
            }
        });
        videoSkinReward.setScale(4);
        videoSkinReward.addAction(Actions.sequence(Actions.delay(0.5F), Actions.visible(true), Actions.scaleTo(1, 1, 0.2F)));
    }

    public void initAnimation() {
        CommonAnimation starAnimation = new CommonAnimation("animations/jiesuan.skel", 0.5f);
        group.addActor(starAnimation);
        starAnimation.setPosition(0, 0);
        starAnimation.getState().setAnimation(0, GameInfo.star + "", false);
        if (!Configuration.poor) {
            CommonParticleEffect effect = new CommonParticleEffect("animations/particle/c3", Assets.instance.ui);
            effect.setPosition(0, 800);
            group.addActorAfter(layer, effect);
        }
    }

    @Override
    protected void initButtons() {
        group.findActor("coin_group", Touchable.enabled).addListener(new LClickListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                successDialogListener.showCoinDialog();
            }
        });
        group.findActor("btn_next", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                successDialogListener.next();
            }
        });
        group.findActor("btn_home", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                baseListener.screenEnd(MenuScreen.class);
            }
        });
        group.findActor("btn_level", Touchable.enabled).addListener(new SoundButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                if (GameInfo.challenge) {
                    baseListener.screenEnd(ChallengeScreen.class);
                } else {
                    baseListener.screenEnd(LevelScreen.class);
                }
            }
        });
        group.findActor("btn_skin", Touchable.enabled).addListener(new ButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                baseListener.screenEnd(StoreScreen.class);
            }
        });
        group.findActor("btn_retry", Touchable.enabled).addListener(new ButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                baseListener.screenEnd(GameScreen.class);
                successDialogListener.showFullScreenAd();
            }
        });
    }

    int srcReward;
    int dstReward;

    public void extraCoinGot() {
        srcReward = GameInfo.levelData.getReward();
        dstReward = srcReward + pendingReward;
        Configuration.settingData.addCoin(pendingReward);
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                if (srcReward < dstReward - 5) {
                    srcReward += 5;
                } else {
                    srcReward = dstReward;
                    extraCoinEnd();
                    coinRewardLabel.getActions().clear();
                }
                coinRewardLabel.setText(srcReward + "");
            }
        };
        if (coinRewardLabel != null) {
            coinRewardLabel.addAction(Actions.forever(Actions.sequence(Actions.run(runnable), Actions.delay(0.01F))));
        }
    }

    private void extraCoinEnd() {
        final CommonAnimation coin = new CommonAnimation("animations/jinbi_jiesuan.skel", 0.5F, "jinbi_fly");
        coin.getState().setTimeScale(1.5F);
        coin.getState().setAnimation(0, "2", false);
        coin.getState().addListener(new AnimationState.AnimationStateAdapter() {
            @Override
            public void complete(AnimationState.TrackEntry entry) {
                super.complete(entry);
                coin.remove();
                coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
            }
        });
        group.addActor(coin);
    }


    public interface SuccessDialogListener {
        public void next();

        public void showCoinDialog();

        public void playVideo(BaseScreen.PendingAction action, int pendingValue);

        public void showFullScreenAd();
    }
}
