package com.zyb.loveball.loader;

import com.badlogic.gdx.assets.AssetDescriptor;
import com.badlogic.gdx.assets.AssetLoaderParameters;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.assets.loaders.AsynchronousAssetLoader;
import com.badlogic.gdx.assets.loaders.FileHandleResolver;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.ArrayMap;
import com.badlogic.gdx.utils.Json;
import com.zyb.loveball.assets.ChapterData;
import com.zyb.loveball.utils.CsvReader;
import org.freyja.libgdx.cocostudio.ui.model.CCExport;

import java.nio.charset.Charset;

public class ChapterDataLoader extends AsynchronousAssetLoader<ChapterDataLoader.ChaptersData, ChapterDataLoader.ChapterParameter> {

    private ChaptersData data;

    public ChapterDataLoader(FileHandleResolver resolver) {
        super(resolver);
    }

    @Override
    public void loadAsync(AssetManager manager, String fileName, FileHandle file, ChapterParameter parameter) {
        data = new ChaptersData();
        CsvReader reader = new CsvReader(file.read(), Charset.defaultCharset());
        try {
            reader.skipLine();
            reader.readHeaders();
            while (reader.readRecord()) {
                ChapterData chapterData = parse(reader);
                if (chapterData != null) {
                    data.put(chapterData.getId(), chapterData);
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private ChapterData parse(CsvReader reader) {
        ChapterData chapterData = new ChapterData();
        try {
            chapterData.setId(Integer.parseInt(reader.get("id")));
            chapterData.setName(reader.get("chapter_name"));
            chapterData.setChapterType(Integer.parseInt(reader.get("chapter_type")));
            chapterData.setLife(Integer.parseInt(reader.get("life")));
            String tmpStr = "";
            for (int i = 0; i < 10; i++) {
                tmpStr = reader.get("stage" + (i + 1));
                int levelId = Integer.parseInt(tmpStr);
                chapterData.setLevel(i, levelId);
            }
            return chapterData;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    public ChaptersData loadSync(AssetManager manager, String fileName, FileHandle file, ChapterParameter parameter) {
        return data;
    }

    @Override
    public Array<AssetDescriptor> getDependencies(String fileName, FileHandle file, ChapterParameter parameter) {
        return null;
    }

    static public class ChapterParameter extends AssetLoaderParameters<ChaptersData> {

    }

    static public class ChaptersData extends ArrayMap<Integer, ChapterData> {
    }
}
