package com.zyb.loveball.loader;

import com.badlogic.gdx.assets.AssetDescriptor;
import com.badlogic.gdx.assets.AssetLoaderParameters;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.assets.loaders.AsynchronousAssetLoader;
import com.badlogic.gdx.assets.loaders.FileHandleResolver;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.ArrayMap;
import com.zyb.loveball.assets.GoodData;
import com.zyb.loveball.assets.LevelData;
import com.zyb.loveball.utils.CsvReader;

import java.nio.charset.Charset;

/**
 * Created by Doodle on 2018/11/14.
 */

public class GoodDataLoader extends AsynchronousAssetLoader<GoodDataLoader.GoodsData, GoodDataLoader.ChapterParameter> {
    private GoodsData data;
    public GoodDataLoader(FileHandleResolver resolver) {
        super(resolver);
    }

    @Override
    public void loadAsync(AssetManager manager, String fileName, FileHandle file, ChapterParameter parameter) {
        data = new GoodsData();
        CsvReader reader = new CsvReader(file.read(), Charset.defaultCharset());
        try {
            reader.skipLine();
            reader.readHeaders();
            while (reader.readRecord()) {
                GoodData goodData = parse(reader);
                if(goodData!=null){
                    data.put(goodData.getId(),goodData);
                    if(goodData.getType() == 0){
                        data.penNum++;
                    }else{
                        data.ballNum++;
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private GoodData parse(CsvReader reader) {
        GoodData goodData = new GoodData();
        try {
            goodData.setId(Integer.parseInt(reader.get("id")));
            String tempStr = reader.get("type");
            if(tempStr!=null&& !tempStr.equals("")) {
                goodData.setType(Integer.parseInt(tempStr));
            }else{
                goodData.setType(0);
            }
            tempStr = reader.get("coin");
            if(tempStr!=null&& !tempStr.equals("")) {
                goodData.setCoin(Integer.parseInt(tempStr));
            }else{
                goodData.setCoin(999999);
            }
            tempStr = reader.get("resources");
            if(tempStr!=null&& !tempStr.equals("")) {
                goodData.setResources(tempStr);
            }else{
                goodData.setResources("pen_lanqianbi");
            }
            tempStr = reader.get("animation");
            if(tempStr!=null&& !tempStr.equals("")) {
                goodData.setAnimation(tempStr);
            }else{
                goodData.setAnimation("boom");
            }
            tempStr = reader.get("icon");
            if(tempStr!=null&& !tempStr.equals("")) {
                goodData.setIcon(tempStr);
            }else{
                goodData.setIcon("nullIcon");
            }
            return goodData;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    public GoodsData loadSync(AssetManager manager, String fileName, FileHandle file, ChapterParameter parameter) {
        return data;
    }

    @Override
    public Array<AssetDescriptor> getDependencies(String fileName, FileHandle file, ChapterParameter parameter) {
        return null;
    }

    static public class ChapterParameter extends AssetLoaderParameters<GoodsData> {

    }

    static public class GoodsData extends ArrayMap<Integer, GoodData> {
        public int penNum;
        public int ballNum;
    }
}
