package com.zyb.loveball.loader;

import com.badlogic.gdx.assets.AssetDescriptor;
import com.badlogic.gdx.assets.AssetLoaderParameters;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.assets.loaders.AsynchronousAssetLoader;
import com.badlogic.gdx.assets.loaders.FileHandleResolver;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.ArrayMap;
import com.zyb.loveball.assets.ChapterData;
import com.zyb.loveball.assets.LevelData;
import com.zyb.loveball.utils.CsvReader;

import java.nio.charset.Charset;

public class LevelDataLoader extends AsynchronousAssetLoader<LevelDataLoader.LevelsData, LevelDataLoader.ChapterParameter> {

    private LevelsData data;

    public LevelDataLoader(FileHandleResolver resolver) {
        super(resolver);
    }

    @Override
    public void loadAsync(AssetManager manager, String fileName, FileHandle file, ChapterParameter parameter) {
        data = new LevelsData();
        CsvReader reader = new CsvReader(file.read(), Charset.defaultCharset());
        try {
            reader.skipLine();
            reader.readHeaders();
            while (reader.readRecord()) {
                LevelData levelData = parse(reader);
                if (levelData != null) {
                    data.put(levelData.getId(), levelData);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private LevelData parse(CsvReader reader) {
        LevelData levelData = new LevelData();
        try {
            levelData.setId(Integer.parseInt(reader.get("id")));
            String tempStr = reader.get("inkall");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setInkAll(Integer.parseInt(tempStr));
            } else {
                levelData.setInkAll(Integer.MAX_VALUE);
            }
            tempStr = reader.get("reward_gold");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setReward(Integer.parseInt(tempStr));
            }
            tempStr = reader.get("ink2");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setInk2(Integer.parseInt(tempStr));
            }
            tempStr = reader.get("ink3");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setInk3(Integer.parseInt(tempStr));
            }
            tempStr = reader.get("initial");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setInkInitial(Integer.parseInt(tempStr));
            } else {
                levelData.setInkInitial(levelData.getInkAll());
            }
            tempStr = reader.get("times");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setTimes(Integer.parseInt(tempStr));
            } else {
                levelData.setTimes(Integer.MAX_VALUE);
            }
            tempStr = reader.get("pushnull");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setPush(0, Integer.parseInt(tempStr));
            }
            tempStr = reader.get("push1");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setPush(1, Integer.parseInt(tempStr));
            }
            tempStr = reader.get("push2");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setPush(2, Integer.parseInt(tempStr));
            }
            tempStr = reader.get("push3");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setPush(3, Integer.parseInt(tempStr));
            }
            tempStr = reader.get("push4");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setPush(4, Integer.parseInt(tempStr));
            }
            tempStr = reader.get("push5");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setPush(5, Integer.parseInt(tempStr));
            }
            tempStr = reader.get("hintprice");
            if (tempStr != null && !tempStr.equals("")) {
                levelData.setHintPrice(Integer.parseInt(tempStr));
            }
            tempStr = reader.get("Banner");
            if (tempStr != null && !tempStr.equals("")) {
                int tempInt = Integer.parseInt(tempStr);
                if (tempInt == 0)
                    levelData.setBanner(true);
            }


            boolean[] magnifier = new boolean[4];
            for (int i = 1; i <= 4; i++) {
                int temp = Integer.parseInt(reader.get("magnifier" + i));
                if (temp == 1) {
                    magnifier[i - 1] = true;
                }
            }

            levelData.setMagnifier(magnifier);
            return levelData;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    public LevelsData loadSync(AssetManager manager, String fileName, FileHandle file, ChapterParameter parameter) {
        return data;
    }

    @Override
    public Array<AssetDescriptor> getDependencies(String fileName, FileHandle file, ChapterParameter parameter) {
        return null;
    }

    static public class ChapterParameter extends AssetLoaderParameters<LevelsData> {

    }

    static public class LevelsData extends ArrayMap<Integer, LevelData> {

    }
}
