package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyContactAdapter;
import com.badlogic.gdx.physics.box2d.Contact;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.constants.Constant;

/**
 * Created by Doodle on 2018/11/7.
 */

public class AreaDanger extends BaseObject {
    Vector2 tmpVector = new Vector2();
    float[] vertexes;
    Vector2 position = new Vector2();
    float width;
    float height;
    float borderWidth;
    float borderHeight;
    TextureRegion regionOrigin;
    int regionSize;
    TextureRegion clipRegion;
    NinePatch ninePatch;
    TextureRegion borderRegion;
    float regionHeight;
    float regionWidth;
    float clipHeight;
    float scale;
    float originX,originY;
    float rotation;
    public AreaDanger(Body body,GamePanel gamePanel){
        super(gamePanel);
        this.body = body;
        rotation = body.getAngle()/MathUtils.PI*180;
        position.set(body.getPosition());
        borderRegion = new TextureRegion(Assets.instance.game.findRegion("area_danger_border"));
        ninePatch = new NinePatch(borderRegion,15,15,15,15);
        scale = Constant.SCALE/2;
        init(body);
        initListener();
    }

    private void initListener(){
        body.setContactListener(new BodyContactAdapter() {
            public void beginContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
                super.beginContact(contact,fixtureA,fixtureB, isA);
                Body tmpBody;
                if (isA) {
                    tmpBody = fixtureB.getBody();
                } else {
                    tmpBody = fixtureA.getBody();
                }
                String name = gamePanel.getScene().getName(tmpBody);
                if (name != null) {
                    if (name.startsWith("ball")) {
                        gamePanel.getBall().bomb(false);
                    }
//                    else if (name.startsWith("heart")) {
//                        gamePanel.getZombie().out();
//                    }
                }
            }
        });
    }

    private void init(Body body){
        PolygonShape shape = (PolygonShape)body.getFixtureList().get(0).getShape();
        int count = shape.getVertexCount();
        vertexes = new float[count*2];
        for(int i = 0;i<count;i++){
            shape.getVertex(i,tmpVector);
            vertexes[2*i] = tmpVector.x;
            vertexes[2*i+1] = tmpVector.y;
        }
        width = (vertexes[0] - vertexes[4]) / Constant.SCALE;
        height = (vertexes[3]- vertexes[1]) / Constant.SCALE;
        originX = width/2;
        originY = height/2;
        borderWidth = width*2+3;
        borderHeight = height*2;
        regionOrigin = new TextureRegion(Assets.instance.game.findRegion("area_danger"));
        regionHeight = regionOrigin.getRegionHeight();
        regionWidth = regionOrigin.getRegionWidth();
        float u2 = regionOrigin.getU2();
        float v2 = regionOrigin.getV2();
        float u = regionOrigin.getU();
        float v = regionOrigin.getV();
        if(width<regionWidth){
            u2 = u + width/regionWidth * (u2-u);
            regionOrigin.setRegion(u,v,u2,v2);
        }
        regionWidth = regionOrigin.getRegionWidth();
        regionSize = (int)(height/regionHeight);

        clipHeight = height - regionSize* regionHeight;
        v = v2 - clipHeight/regionHeight * (v2-v);
        clipRegion = new TextureRegion(regionOrigin);
        clipRegion.setRegion(u,v,u2,v2);
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        for (int i = 0;i<regionSize;i++){
            batch.draw(regionOrigin,position.x-originX,position.y-originY+ i*regionHeight*Constant.SCALE,originX,originY,regionWidth,regionHeight,Constant.SCALE,Constant.SCALE,rotation);
        }
        batch.draw(clipRegion,position.x-originX,position.y-originY+regionHeight*Constant.SCALE*regionSize,originX,originY,regionWidth,clipHeight,Constant.SCALE,Constant.SCALE,rotation);
        ninePatch.draw(batch,position.x - borderWidth / 2, position.y - borderHeight / 2, borderWidth / 2, borderHeight / 2, borderWidth, borderHeight, Constant.SCALE/2, Constant.SCALE/2, rotation);
    }
}
