package com.zyb.loveball.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.*;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.animations.BallAnimation;
import com.zyb.loveball.animations.NormalBallAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.attributes.BallAttribute;
import com.zyb.loveball.audio.AudioProcess;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.utils.Log;

/**
 * Created by Doodle on 2018/9/10.
 */

public class Ball extends BaseObject {
    TextureRegion region;
    Vector2 position = new Vector2();
    Vector2 velocity = new Vector2();
    float angularVelocity;
    float rotation;
    BallAnimation ballAnimation;
    private BallState state = BallState.idle;
    private BombListener bombListener;
    int id;

    public Ball(Body body, final GamePanel gamePanel, int type) {
        super(gamePanel);
        id = Configuration.settingData.getGoodId(1);
        this.body = body;
        position.set(body.getPosition());
        body.setTransform(position,0);
        this.gamePanel = gamePanel;
        //       if(type == 1) {
        ballAnimation = new NormalBallAnimation(id);
        //     }else if(type == 2){
        //       ballAnimation = new BouncyBallAnimation();
        //    }
        region = Assets.instance.game.findRegion("ball_blue");
        body.setContactListener(contactListener);
        body.destroyFixture(body.getFixtureList().get(0));
        CircleShape circleShape = new CircleShape();
        circleShape.setRadius(0.21f);
        Fixture fixture = body.createFixture(circleShape, Constant.ball.density);
        fixture.setFriction(Constant.ball.friction);
        fixture.setFilterData(BallAttribute.filter);
        fixture.setRestitution(0.15F);
        if (type == 2) {
            body.getFixtureList().get(0).setRestitution(Constant.ball_bouncy.restitution);
        } else if (type == 3) {
            body.setGravityScale(-1);
        }
        body.setBullet(true);
    }

    public Zombie.ZombieListener initZombieListener() {
        return new Zombie.ZombieListener() {
            @Override
            public void getPosition(Vector2 position) {
                if (state == BallState.idle) {
                    if (position.x > Ball.this.position.x) {
                        ballAnimation.setFaceRight(true);
                    } else {
                        ballAnimation.setFaceRight(false);
                    }
                }
            }
        };
    }

    public void setBombListener(BombListener bombListener) {
        this.bombListener = bombListener;
    }

    @Override
    public void updateUI(float delta) {
        position.set(body.getPosition());
        rotation = body.getAngle() / (float) Math.PI * 180;
        velocity.set(body.getLinearVelocity());
        angularVelocity = body.getAngularVelocity();
        if (angularVelocity != 0 && state == BallState.idle) {
            state = BallState.run;
            ballAnimation.run(angularVelocity);
        }
//        else if(state == BallState.run && angularVelocity == 0 ){
//            state = BallState.idle;
//            ballAnimation.idle();
//        }
        bombListener.getPosition(position);
        ballAnimation.update(position.x, position.y, rotation);
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        batch.setColor(Color.WHITE);
        ballAnimation.draw(batch);
    }

    private void drawBallRegion(Batch batch) {
        batch.draw(region, position.x - 17.5F, position.y - 17.5F, 17.5F, 17.5F, 44, 47, 1 / 80f, 1 / 80f, rotation);
    }

    BodyContactAdapter contactListener = new BodyContactAdapter() {

        public void beginContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
            super.beginContact(contact, fixtureA, fixtureB, isA);
            Body tmpBody;
            if (isA) {
                tmpBody = fixtureB.getBody();
            } else {
                tmpBody = fixtureA.getBody();
            }
            String name = gamePanel.getScene().getName(tmpBody);
            if (name != null && name.startsWith("heart")) {
                bomb(true);

            }
        }
    };

    public void bomb(boolean isZombie) {
        if (state == BallState.contact) return;
        Log.log("Ball","bomb()");
        state = BallState.contact;
        gamePanel.activeBody(body, false);
        ballAnimation.boom();
        body.setContactListener(null);
        AudioProcess.playSound("bomb", 1);
        if(isZombie) {
            gamePanel.end(true);
            bombListener.boom(id);
        }else{
            gamePanel.end(false);
        }
    }

    public void out() {
        gamePanel.end(false);
        bombListener.ballOut();
    }

    public enum BallState {
        idle, run, contact, boom
    }

    public interface BombListener {
        public void getPosition(Vector2 position);

        public void boom(int ballId);

        public void ballOut();
    }
}
