package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.*;
import com.badlogic.gdx.utils.Array;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.constants.Constant;

import java.util.Iterator;

public class Border extends BaseObject {

    Vector2 position;

    public Border(Body body, GamePanel panel) {
        super(panel);
        this.body = body;
        Array<Fixture> fixtures = new Array<>();
        fixtures.addAll(body.getFixtureList());
        initBounds(fixtures);
    }

    private void initBounds(Array<Fixture> fixtures) {
        float height = gamePanel.getCamera().viewportHeight;
        body.setTransform(0, 0, 0);
        for (Fixture temp : fixtures) {
            body.destroyFixture(temp);
        }
        FixtureDef fixtureDef = new FixtureDef();
        fixtureDef.isSensor = true;
        EdgeShape shape = new EdgeShape();
        fixtureDef.shape = shape;
        float offset = 0.2F;
        shape.set(0 - offset, 0 - offset, 0 - offset, height + offset);
        body.createFixture(fixtureDef);
        shape.set(0 - offset, 0 - offset, 6 + offset, 0 - offset);
        body.createFixture(fixtureDef);
        shape.set(6 + offset, 0 - offset, 6 + offset, height + offset);
        body.createFixture(fixtureDef);
        shape.set(0 - offset, height + offset, 6 + offset, height + offset);
        body.createFixture(fixtureDef);
        body.setContactListener(new BodyContactAdapter() {
            public void beginContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
                super.beginContact(contact, fixtureA, fixtureB, isA);
                Body tmpBody;
                if (isA) {
                    tmpBody = fixtureB.getBody();
                } else {
                    tmpBody = fixtureA.getBody();
                }
                String name = gamePanel.getScene().getName(tmpBody);
                if (name != null) {
                    if (name.startsWith("ball")) {
                        gamePanel.getBall().out();
                    } else if (name.startsWith("heart")) {
                        gamePanel.getZombie().out();
                    }
                }
            }
        });
    }

}
