package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyContactAdapter;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.CircleShape;
import com.badlogic.gdx.physics.box2d.Contact;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.physics.box2d.Manifold;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.constants.Constant;

public class ConveyorBelt extends BaseObject {
    NinePatch ninePatch;
    TextureRegion wheelRegion;
    float rotation;
    Vector2 tmpVectorBody = new Vector2();
    PolygonShape fixture0;
    Vector2 point1;
    float bodyWidth, bodyHeight;
    float rotationSpeed = 1;
    int direction = 1;
    float targetScale = 1/160f;

    public ConveyorBelt(Body beltBody, GamePanel gamePanel) {
        super(gamePanel);
        this.body = beltBody;
        if (beltBody.getType() == BodyDef.BodyType.DynamicBody
                || beltBody.getType() == BodyDef.BodyType.KinematicBody) {

            rotationSpeed = beltBody.getAngularVelocity() / (float) Math.PI * 180;
            if (rotationSpeed == 0) rotationSpeed = 1;
            beltBody.setType(BodyDef.BodyType.StaticBody);
        }
        point1 = new Vector2();
        ((PolygonShape) beltBody.getFixtureList().get(0).getShape()).getVertex(1, point1);
        fixture0 = (PolygonShape) beltBody.getFixtureList().get(0).getShape();
        bodyWidth = Math.abs(point1.x * 2 * 80);
        bodyHeight = Math.abs(point1.y * 2 * 80);
        TextureRegion bodyRegion = Assets.instance.game.findRegion("conveyor_belt_body");
        wheelRegion = Assets.instance.game.findRegion("conveyor_belt_wheel");
        ninePatch = new NinePatch(bodyRegion, 60, 60, 6, 6);
        BodyDef bodyDef = new BodyDef();
        bodyDef.type = BodyDef.BodyType.KinematicBody;
        rotation = beltBody.getAngle();
        tmpVectorBody.set(beltBody.getPosition().x, beltBody.getPosition().y);
        beltBody.setContactListener(contactListener);
        updateConveyorBelt(beltBody);
        float tempRotation = rotation * 180 / (float) Math.PI;
        while (tempRotation < 0) {
            tempRotation += 360;
        }
        if (tempRotation > 90 && tempRotation < 270) {
            direction = -1;
        }

    }

    private void updateConveyorBelt(Body beltBody) {
        float[] polygonShapePoints = new float[8];
        bodyHeight = 27f;
        tmpVectorBody.set(beltBody.getPosition().x - (float) Math.sin(rotation) * (Math.abs(point1.y) - 13.5f / 80f),
                beltBody.getPosition().y + (float) Math.cos(rotation) * (Math.abs(point1.y) - 13.5f / 80f));
        for (int i = 0; i < polygonShapePoints.length / 2; i++) {
            Vector2 temp = new Vector2();
            fixture0.getVertex(i, temp);
            if (temp.x > 0) {
                polygonShapePoints[i * 2] = temp.x - 0.17f;
            } else {
                polygonShapePoints[i * 2] = temp.x + 0.17f;
            }
            if (temp.y < 0) {
                polygonShapePoints[i * 2 + 1] = Math.abs(temp.y) - 27f / 80f;
            } else {
                polygonShapePoints[i * 2 + 1] = temp.y;
            }

        }
        Fixture tempFixture = beltBody.getFixtureList().get(0);
        beltBody.destroyFixture(tempFixture);
        PolygonShape polygonShape = new PolygonShape();
        polygonShape.set(polygonShapePoints);
        beltBody.createFixture(polygonShape, 1);
        CircleShape circleShapeLeft = new CircleShape();
        CircleShape circleShapeRight = new CircleShape();
        circleShapeLeft.setRadius(13.5f / 80f);
        circleShapeRight.setRadius(13.5f / 80f);
        circleShapeLeft.setPosition(new Vector2(polygonShapePoints[0], Math.abs(polygonShapePoints[3]) - 13.5f / 80f));
        circleShapeRight.setPosition(new Vector2(-polygonShapePoints[0], Math.abs(polygonShapePoints[3]) - 13.5f / 80f));
        beltBody.createFixture(circleShapeLeft, 1);
        beltBody.createFixture(circleShapeRight, 1);
    }

    BodyContactAdapter contactListener = new BodyContactAdapter() {
        public void beginContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
            super.beginContact(contact, fixtureA, fixtureB, isA);
            contact.setTangentSpeed(2 * rotationSpeed);
        }

        @Override
        public void preSolve(Contact contact, Fixture fixtureA, Fixture fixtureB, Manifold oldManifold, boolean isA) {
            Body tmpBody;
            if (isA) {
                tmpBody = fixtureB.getBody();
            } else {
                tmpBody = fixtureA.getBody();
            }
            if (tmpBody.getLinearVelocity().len() <= Math.abs(3 * rotationSpeed)) {
                float force = rotationSpeed * tmpBody.getMass();
                tmpBody.applyForceToCenter(force * (float) Math.cos(rotation) * direction, force * (float) Math.sin(rotation) * direction, false);
            }
        }
    };

    float wheelAngle;

    @Override
    public void updateUI(float delta) {
        super.updateUI(delta);
        wheelAngle += 360 - 3 * rotationSpeed;
        wheelAngle %= 360;
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);

        //((PolygonShape)body.getFixtureList().get(0).getShape()).getVertex(2,point2);

        float bodyWidth = this.bodyWidth*(Constant.SCALE/targetScale);
        float bodyHeight = this.bodyHeight*(Constant.SCALE/targetScale);
        float wheelWidth = wheelRegion.getRegionWidth();
        float wheelHeight = wheelRegion.getRegionHeight();
        ninePatch.draw(batch,
                tmpVectorBody.x - bodyWidth / 2,
                tmpVectorBody.y - bodyHeight / 2,
                bodyWidth / 2,
                bodyHeight / 2,
                bodyWidth,
                bodyHeight,
                1 / 160F,
                1 / 160F,
                rotation / (float) Math.PI * 180f);

        batch.draw(wheelRegion,
                tmpVectorBody.x - wheelWidth / 2 - (float) Math.cos(rotation) * (bodyWidth  / 2 *targetScale-0.175f),
                tmpVectorBody.y - wheelHeight / 2 - (float) Math.sin(rotation) * (bodyWidth / 2 *targetScale-0.175f),
                wheelWidth / 2,
                wheelHeight / 2,
                wheelWidth,
                wheelHeight,
                1 / 160F,
                1 / 160F,
                wheelAngle);
        batch.draw(wheelRegion,
                tmpVectorBody.x - wheelWidth / 2 + (float) Math.cos(rotation) * (bodyWidth / 2 *targetScale-0.175f),
                tmpVectorBody.y - wheelHeight / 2 + (float) Math.sin(rotation) * (bodyWidth / 2 *targetScale-0.175f),
                wheelWidth / 2,
                wheelHeight / 2,
                wheelWidth,
                wheelHeight,
                1 / 160F,
                1 / 160F,
                wheelAngle);
    }
}
