package com.zyb.loveball.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.*;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.*;
import com.badlogic.gdx.utils.Array;
import com.rube.CustomProperties;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.constants.Constant;

/**
 * Created by Doodle on 2018/10/10.
 */

public class Fan extends BaseObject {
    private TextureRegion region;
    Animation<TextureRegion> fanAnimation;
    Array<TextureRegion> regions = new Array<>();
    float windPower = 0;
    Vector2 tmpVector = new Vector2();
    float width, height;
    float x, y;
    float originX, originY;
    float rotation;
    ParticleEffect effect;
    CustomProperties properties;
    Vector2 bodyPosition;
    float angle;
    float windPowerBase = 5;
    float range;
    float[][] rayLines = new float[5][4];
    private float targetScale = 1 / 160f;
    private float aniFactor = 0.25F;

    public Fan(Body body, GamePanel gamePanel, CustomProperties properties) {
        super(gamePanel);
        this.properties = properties;
        this.body = body;
        angle = body.getAngle();
        rotation = angle / (float) Math.PI * 180;
        bodyPosition = body.getPosition();
        init();
        initParticle();
    }

    private void initParticle() {
        effect = new ParticleEffect();
        effect.load(Gdx.files.internal("animations/particle/lizi_fan"), Assets.instance.game);
        ParticleEmitter emitter = effect.getEmitters().get(0);
        float fanX = bodyPosition.x + 0.5F * (float) Math.cos(angle);
        float fanY = bodyPosition.y + 0.5F * (float) Math.sin(angle);
        effect.setPosition(fanX, fanY);
        emitter.scaleSize(1 / 80F);
        emitter.scaleMotion(1 / 80F);
        emitter.getAngle().setLow(rotation);
        float velocityMax = 300 * windPower / windPowerBase * Constant.SCALE * aniFactor;
        int count = (int) (range / 2 * 20 * windPower / windPowerBase * aniFactor);
        count = MathUtils.clamp(count, 15, 100);
        emitter.setMaxParticleCount(count);
        velocityMax = MathUtils.clamp(velocityMax,150*Constant.SCALE,1500*Constant.SCALE);
        float velocityMin = velocityMax/6;
        emitter.getVelocity().setHighMin(velocityMin);
        emitter.getVelocity().setHighMax(velocityMax);
        emitter.getLife().setLow(range / velocityMax * 1000 - 100);
        emitter.getLife().setHigh(range / velocityMax * 1000 - 100);
    }

    private void init() {
        PolygonShape shape = (PolygonShape) body.getFixtureList().get(0).getShape();
        shape.getVertex(1, tmpVector);
        for (int i = 0; i < 4; i++) {
            shape.getVertex(i, tmpVector);
            if (tmpVector.x > range) {
                range = tmpVector.x;
            }
        }
        for (int i = 0; i < 6; i++) {
            regions.add(Assets.instance.animation.findRegion("fan" + (i + 1)));
        }
        width = regions.get(0).getRegionWidth();
        height = regions.get(0).getRegionHeight();
        tmpVector.set(body.getPosition());
        x = tmpVector.x;
        y = tmpVector.y;
        originX = width / 2;
        originY = height / 2;
        if (properties.get("windPower") == null) windPower = windPowerBase;
        else windPower = properties.get("windPower");
//        if(windPower == 5){
//            windPower = windPowerBase;
//        }
        fanAnimation = new Animation(1 / (20 * windPower*aniFactor + 10), regions);
        fanAnimation.setPlayMode(Animation.PlayMode.LOOP);
        for (int i = 0; i < 3; i++) {
            createRayLine(bodyPosition.x - i * 0.17F * (float) Math.sin(angle), bodyPosition.y + i * 0.17F * (float) Math.cos(angle), i);
        }
        createRayLine(bodyPosition.x + 1 * 0.17F * (float) Math.sin(angle), bodyPosition.y - 1 * 0.17F * (float) Math.cos(angle), 3);
        createRayLine(bodyPosition.x + 2 * 0.17F * (float) Math.sin(angle), bodyPosition.y - 2 * 0.17F * (float) Math.cos(angle), 4);

    }

    private void createRayLine(float x, float y, int index) {
        rayLines[index][0] = x;
        rayLines[index][1] = y;
        rayLines[index][2] = x + range * (float) Math.cos(angle);
        rayLines[index][3] = y + range * (float) Math.sin(angle);
    }


    float stateTime = 0;

    @Override
    public void updateUI(float delta) {
        super.updateUI(delta);
        stateTime += Gdx.graphics.getDeltaTime();
        region = fanAnimation.getKeyFrame(stateTime);
        effect.update(Gdx.graphics.getDeltaTime());
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        effect.draw(HelloZombieGame.getBatch());
        float offset = 0.2f;
        batch.draw(region, x - originX + offset * MathUtils.cos(angle), y - originY + offset * MathUtils.sin(angle), originX, originY, width, height, targetScale, targetScale, rotation);

    }

    RayCastCallback rayCastCallback = new RayCastCallback() {
        @Override
        public float reportRayFixture(Fixture fixture, Vector2 point, Vector2 normal, float fraction) {
            Body tmpBody = fixture.getBody();
            float force = (1 - fraction) *0.1F* windPower / windPowerBase;
            float forceX = force * (float) Math.cos(angle);
            float forceY = force * (float) Math.sin(angle);
            tmpBody.applyForce(forceX, forceY, point.x, point.y, true);
            return 1;
        }
    };

    @Override
    public void updateWorld(float delta) {
        super.updateWorld(delta);
        for (int i = 0; i < rayLines.length; i++) {
            gamePanel.getWorld().rayCast(rayCastCallback, rayLines[i][0], rayLines[i][1], rayLines[i][2], rayLines[i][3]);
        }
    }
}
