package com.zyb.loveball.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.FloatArray;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.HintData;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.utils.MathUtil;

public class Hint {
    HintLine hintLine;
    private int hintIndex;
    private boolean showHint;
    Array<float[]> hints;
    private float[] currentVertexes;
    float hintLength;
    private TextureRegion handRegion;
    float leftBottomX;
    float leftBottomY;
    float rightTopX;
    float rightTopY;
    private boolean showLineHand;

    public Hint() {
        initHintData();
        if(GameInfo.course){
            showLineHand = true;
        }
    }

    private void initHintData() {
        FileHandle handle = Gdx.files.internal("data/chapter/level_hint" + GameInfo.levelId + ".json");
        if (handle.exists()) {
            String str = handle.readString();
            try {
                hints = HelloZombieGame.getJson().fromJson(HintData.class, str);
                initHintLength();
            } catch (Exception e) {
                e.printStackTrace();
                handle.delete();
            }
        }
        if (hints == null) {
            hints = new Array<>();
        }
    }

    private void initHintLength() {
        for (Object temp : hints) {
            float[] points;
            if (temp instanceof float[]) {
                points = (float[]) temp;
            } else {
                Array array = (Array) temp;
                points = new float[array.size];
                for (int i = 0; i < array.size; i++) {
                    points[i] = (Float) array.get(i);
                }
            }
            for (int i = 0; i < points.length - 2; i = i + 2) {
                if (points.length < 4) break;
                float x = Math.abs(points[i + 2] - points[i]);
                float y = Math.abs(points[i + 3] - points[i + 1]);
                hintLength += (float) Math.sqrt(x * x + y * y) * 10;
            }
        }
    }

    public boolean hit(float x, float y) {
        if (!showHint) return false;
        if (x >= leftBottomX && y >= leftBottomY && x <= rightTopX && y <= rightTopY) {
            return true;
        }
        return false;
    }

    public void clearHint() {
        hints.clear();
        writeHint();
    }

    public void addHint(FloatArray vertexes) {
        hints.add(vertexes.toArray());
        writeHint();
    }

    private void writeHint() {
        FileHandle handle = Gdx.files.local("data/chapter/level_hint" + GameInfo.levelId + ".json");
        try {
            handle.writeString(HelloZombieGame.getJson().toJson(hints), false);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public void hint() {
        showHint = true;
        if (hintIndex >= hints.size) {
            hintEnd();
            return;
        }
        if (hintLine == null) {
            hintLine = new HintLine();
        }
        currentVertexes = getCurrentVertexes();
        System.out.println("length---->"+currentVertexes.length);
        hintLine.setHintVertexes(currentVertexes);
        hintLine.setVisible(true);
        initBounds();
        if (showLineHand) {
            showHand();
        }
    }

    private void showHand() {
        handRegion = Assets.instance.animation.findRegion("shou2");
        pickPointLength = MathUtil.length(currentVertexes[0], currentVertexes[1], currentVertexes[2], currentVertexes[3]);
        handWidth = handRegion.getRegionWidth();
        handHeight = handRegion.getRegionHeight();
    }

    private void initBounds() {
        if (currentVertexes == null) return;
        leftBottomX = 6;
        leftBottomY = 10;
        rightTopX = 0;
        rightTopY = 0;
        for (int i = 0; i < currentVertexes.length; i += 2) {
            float tmpX, tmpY;
            tmpX = currentVertexes[i];
            tmpY = currentVertexes[i + 1];
            if (tmpX < leftBottomX) leftBottomX = tmpX;
            if (tmpX > rightTopX) rightTopX = tmpX;
            if (tmpY < leftBottomY) leftBottomY = tmpY;
            if (tmpY > rightTopY) rightTopY = tmpY;
            float minHintLength = 0.5f;
            if (rightTopX - leftBottomX < minHintLength) {
                rightTopX += (minHintLength - (rightTopX - leftBottomX)) / 2;
                leftBottomX -= (minHintLength - (rightTopX - leftBottomX)) / 2;
            }
            if (rightTopY - leftBottomY < minHintLength) {
                rightTopY += (minHintLength - (rightTopY - leftBottomY)) / 2;
                leftBottomY -= (minHintLength - (rightTopY - leftBottomY)) / 2;
            }
        }
    }

    public void reset() {
        hintIndex = 0;
        hintLine = null;
        showHint = false;
    }

    public void nextHint() {
        if (!showHint) return;
        hintIndex++;
        if (hintIndex >= hints.size) {
            hintEnd();
            return;
        }
        currentVertexes = getCurrentVertexes();
        hintLine.setHintVertexes(currentVertexes);
        initBounds();
    }

    public HintLine getHintLine() {
        return hintLine;
    }

    public boolean isShowHint() {
        return showHint;
    }

    private float[] getCurrentVertexes() {
        Object temp = hints.get(hintIndex);
        float[] vertexes = null;
        if (temp instanceof float[]) {
            vertexes = (float[]) temp;
        } else {
            Array array = (Array) temp;
            vertexes = new float[array.size];
            for (int i = 0; i < array.size; i++) {
                vertexes[i] = (Float) array.get(i);
            }
        }
        return vertexes;
    }

    public void hintEnd() {
        hintIndex = 0;
        if (hintLine != null) {
            hintLine.setVisible(false);
        }
        showHint = false;
        showLineHand = false;
        currentVertexes = null;
    }

    private int pickPoint = 0;      //教程小手画到哪两个点之间
    private float pickPointLength = 0;       //两点之间剩余的没有绘制的长度
    private float leftLength = 0;            //剩余leftLength长度再次绘制小手
    public static float courseHandVel = 0.01f;
    private float handX, handY;
    private float handWidth, handHeight;

    public void updateHint(){
        if(showLineHand){
            leftLength = courseHandVel;
            while (leftLength > pickPointLength) {
                leftLength -= pickPointLength;
                pickPoint += 2;
                pickPoint %= currentVertexes.length - 4;
                pickPointLength = MathUtil.length(currentVertexes[pickPoint], currentVertexes[pickPoint + 1], currentVertexes[pickPoint + 2], currentVertexes[pickPoint + 3]);
            }
            pickPointLength -= leftLength;
            float length = MathUtil.length(currentVertexes[pickPoint], currentVertexes[pickPoint + 1], currentVertexes[pickPoint + 2], currentVertexes[pickPoint + 3]);
            handX = currentVertexes[pickPoint + 2] - pickPointLength * (currentVertexes[pickPoint + 2] - currentVertexes[pickPoint]) / length;
            handY = currentVertexes[pickPoint + 3] - pickPointLength * (currentVertexes[pickPoint + 3] - currentVertexes[pickPoint + 1]) / length;
        }
    }

    public void draw(Batch batch) {
        if (showHint && hintLine != null) {
            hintLine.draw(batch, 1);
        }
        if (showLineHand) {
            batch.draw(handRegion, handX, handY - handHeight, 0, handHeight, handWidth, handHeight, Constant.SCALE / 3, Constant.SCALE / 3, 0);
        }
    }


    public float getHintLength() {
        return hintLength;
    }
}
