package com.zyb.loveball.objects;


import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.*;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.FloatArray;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.attributes.LineAttribute;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.MathUtil;

/**
 * Created by Doodle on 2018/8/9.
 */

public class Line {
    protected Vector2 tempVector = new Vector2();
    protected Vector2 tempVector2 = new Vector2();
    protected Body body;
    protected float angle;
    protected FloatArray lengthPicFloats = new FloatArray();           //每个方块的长度
    protected FloatArray localLengthArray = new FloatArray();          //点到body初始点的位置坐标
    protected FloatArray localAngleArray = new FloatArray();           //点和初始点连线的角度坐标
    protected FloatArray aSinDouble = new FloatArray();               //每个放块与水平线的夹角大小
    protected FloatArray vertexes = new FloatArray();
    boolean created;
    static BodyDef bodyDef;
    static PolygonShape polygonShape;
//    static CircleShape circleShape;
    static FixtureDef fixtureDef;
    private float lineBoxMin = 0.05F;
    float x, y;

    public FloatArray getVertexes() {
        return vertexes;
    }

    GamePanel panel;

    public Line(GamePanel panel) {
        init();
        bodyDef.linearDamping = 1;
        bodyDef.angularDamping = 2;
        bodyDef.position.set(0, 0);
        bodyDef.active = false;
        this.panel = panel;
        body = panel.createBody(bodyDef);
        body.setType(BodyDef.BodyType.DynamicBody);
        body.setBullet(false);
        body.setSleepingAllowed(true);
    }

    private void init() {
        if (bodyDef != null) return;
        bodyDef = new BodyDef();
        bodyDef.type = BodyDef.BodyType.DynamicBody;
        bodyDef.position.set(0, 0);
        polygonShape = new PolygonShape();
//        circleShape = new CircleShape();
//        circleShape.setRadius(Constant.LINESIZE);
        fixtureDef = new FixtureDef();
        fixtureDef.friction = Constant.line.friction;
        fixtureDef.filter.maskBits = LineAttribute.filter.maskBits;
        fixtureDef.filter.categoryBits = LineAttribute.filter.categoryBits;
        fixtureDef.filter.groupIndex = LineAttribute.filter.groupIndex;
        fixtureDef.shape = polygonShape;
        fixtureDef.density = Constant.line.density;
    }

    public void setBodyPosition(float x, float y) {
        body.setTransform(x, y, 0);
        this.x = x;
        this.y = y;
    }

    public void addVertex(float x, float y) {
        x -= this.x;
        y -= this.y;
        int size = vertexes.size;
        if (size < 4) {
            insert(x, y);
        } else {
            float x1 = vertexes.get(size - 4);
            float y1 = vertexes.get(size - 3);
            float x2 = vertexes.get(size - 2);
            float y2 = vertexes.get(size - 1);
            boolean sameLine = MathUtil.isOnSameLine(x1, y1, x2, y2, x, y);
            if (sameLine) {
                tempVector.set(x2 - x1, y2 - y1);
                tempVector2.set(x - x2, y - y2);
                sameLine = tempVector.hasSameDirection(tempVector2);
            }
            if (sameLine) {
                removeVertex();
//                System.out.println("same line , first point:" + x1+ " " + y1 + "   + second point: " + x2 + " " + y2 + "   new point: " + x + "  " + y);
            }
            insert(x, y);
        }
    }


    private void insert(float x, float y) {
        pretreatment();
        if (vertexes.size == 0) {
            vertexes.add(x);
            vertexes.add(y);
            return;
        }
        int size = vertexes.size;
        float x1 = vertexes.get(size - 2);
        float y1 = vertexes.get(size - 1);
        float x2 = x;
        float y2 = y;
        float length = (float) Math.sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
        vertexes.add(x);
        vertexes.add(y);
        float localX;
        float localY;
        float localLength;
        lengthPicFloats.add(length);
        aSinDouble.add((float) (MathUtil.getAngle(x1, y1, x2, y2) * 180 / Math.PI));
        localX = x1;
        localY = y1;
        localLength = (float) Math.sqrt(localX * localX + localY * localY);
        localLengthArray.add(localLength);
        localAngleArray.add((float) MathUtil.getAngle(0, 0, x1, y1));
        if (length >= lineBoxMin) {
            createRectShape(x1, y1, x2, y2);
        }
    }

    private void pretreatment() {
        if (vertexes.size < 4 || body.getFixtureList().size == vertexes.size / 2 - 1) return;
        int size = vertexes.size;
        float x1 = vertexes.get(size - 4);
        float y1 = vertexes.get(size - 3);
        float x2 = vertexes.get(size - 2);
        float y2 = vertexes.get(size - 1);
        float length = (float) Math.sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
        if (length < lineBoxMin) {
            removeVertex();
        }
    }


    public Body active() {
        if (body.isActive()) return body;
        if (vertexes.size < 4) return null;
        if (body.getFixtureList().size < vertexes.size / 2 - 1) {
            createRectShape(vertexes.get(vertexes.size - 4), vertexes.get(vertexes.size - 3), vertexes.get(vertexes.size - 2), vertexes.get(vertexes.size - 1));
        }
        panel.transformBody(body, x, y, 0);
        panel.activeBody(body, true);
        created = true;
        return body;
    }

    float[] temp = new float[8];

    public void clearAll() {
        lengthPicFloats.clear();
        localLengthArray.clear();
        localAngleArray.clear();
        aSinDouble.clear();
        vertexes.clear();
        Array<Fixture> fixtures = body.getFixtureList();
        float num = fixtures.size;
        for (int i = 0; i < num; i++) {
            body.destroyFixture(fixtures.get(0));
        }
    }

    private void removeVertex() {
        int size = vertexes.size / 2;
        vertexes.removeIndex(2 * size - 1);
        vertexes.removeIndex(2 * size - 2);
        lengthPicFloats.removeIndex(size - 2);
        localAngleArray.removeIndex(size - 2);
        localLengthArray.removeIndex(size - 2);
        aSinDouble.removeIndex(size - 2);
        if (body.getFixtureList().size == size - 1) {
            Fixture fixture = body.getFixtureList().get(size - 2);
            body.destroyFixture(fixture);
        }
    }

    private void createRectShape(float x1, float y1, float x2, float y2) {
        float offsetX = x2 - x1;           // x改变量
        float offsetY = y2 - y1;           // y改变量
        float length = (float) Math.sqrt(offsetX * offsetX + offsetY * offsetY);
        if(length<=Constant.MINBOXLENGTH){
            removeVertex();
            return;
        }
        float changeXMath = (Constant.LINESIZE / length * offsetY);     //绘制矩形时x相对于鼠标位置的改变量
        float changeYMath = (Constant.LINESIZE / length * offsetX);    //绘制矩形时y相对于鼠标位置的改变量
        temp[0] = x1 + changeXMath;
        temp[1] = y1 - changeYMath;
        temp[2] = x1 - changeXMath;
        temp[3] = y1 + changeYMath;
        temp[4] = x2 - changeXMath;
        temp[5] = y2 + changeYMath;
        temp[6] = x2 + changeXMath;
        temp[7] = y2 - changeYMath;
        polygonShape.set(temp);
        fixtureDef.shape = polygonShape;
        panel.createFixture(body, fixtureDef);
//        float rotation = aSinDouble.get(aSinDouble.size-1);
//        if(rotation>40 || rotation < -40){
//            tempVector.set(x1,y1);
//            circleShape.setPosition(tempVector);
//            fixtureDef.shape = circleShape;
//            panel.createFixture(body,fixtureDef);
//        }
    }

    public boolean draw(Batch batch, NinePatch ninePatch) {
        if (created) {
            angle = body.getAngle();
            tempVector.set(body.getPosition());
            x = tempVector.x;
            y = tempVector.y;
        }
        if (x > 12 || x < -6 || y > 20 || y < -10) return false;
        float sumX;
        float sumY;
        float degreesAngle = (float) Math.toDegrees(angle);
        tempVector.set(x, y);
        for (int i = 0; i < vertexes.size - 2; i += 2) {
            if (vertexes.size <= 2) break;
            float rotation = (float) ((double) aSinDouble.get(i / 2)) + degreesAngle;
            float localLength = localLengthArray.get(i / 2);
            float localAngle = localAngleArray.get(i / 2);
            sumX = tempVector.x + (float) (Math.cos(localAngle + angle) * localLength);
            sumY = tempVector.y + (float) (Math.sin(localAngle + angle) * localLength);
            float x = sumX;
            float y = sumY - 8;
            ninePatch.draw(batch, x - 4, y, 4, 8, lengthPicFloats.get(i / 2) * 250 + 8, 16F, 0.004f, 0.004f, rotation);
        }
        return true;
    }

    public float getX() {
        return x;
    }

    public float getY() {
        return y;
    }

    public Body getBody() {
        return body;
    }
}
