package com.zyb.loveball.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.utils.ScissorStack;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.utils.Matrix4s;

public class MagnifierUtil extends Actor {
    OrthographicCamera magnifierCamera;
    Rectangle scissors = new Rectangle();
    Rectangle clipBounds;
    private int magnifierId = -1;
    Rectangle magnifierArea = new Rectangle();
    private GamePanel gamePanel;
    float cameraWidth, cameraHeight;
    private boolean[] magnifierOpen;
    NinePatch magnifierBg;
    Vector2 position = new Vector2();
    float xOffset = 10F / Configuration.width;
    float yOffset = 10F / Configuration.height;
    private boolean magnifier;
    public Rectangle[] area = new Rectangle[4];
    static float penOffsetX =0 ;
    static float penOffsetY = 0;
    private float zoom = 1.5F;
    //    public Rectangle[] area = {
//            new Rectangle(-1F,0.5F,0.5F,0.5F),
//            new Rectangle(0.5F,0.5F,0.5F,0.5F),
//            new Rectangle(-1F,-1F,0.5F,0.5F),
//            new Rectangle(0.5F,-1F,0.5F,0.5F)
//    };
    public static float[][] offset;

    public MagnifierUtil(boolean[] magnifierOpen, GamePanel gamePanel) {
        initArea();
        this.magnifierOpen = magnifierOpen;
        this.gamePanel = gamePanel;
        if (magnifierOpen == null) {
            magnifierOpen = new boolean[4];
        }
        initCamera();
        checkMagnifierArea();
        setTouchable(Touchable.disabled);
    }

    private void initArea() {
        width = 150;
        height = 170;
        magnifierArea.width = (width + xOffset * Configuration.width * 2) * Constant.SCALE;
        magnifierArea.height = (height + yOffset * Configuration.height * 2) * Constant.SCALE;
        this.cameraHeight = Configuration.height * Constant.SCALE / zoom;
        this.cameraWidth = Configuration.width * Constant.SCALE / zoom;
        Rectangle leftTop = new Rectangle();
        leftTop.width = width / (Configuration.width / 2);
        leftTop.height = height / (Configuration.height / 2);
        leftTop.x = -1 + xOffset;
        leftTop.y = 1 - yOffset - leftTop.height;
        area[0] = leftTop;
        Rectangle rightTop = new Rectangle();
        rightTop.width = width / (Configuration.width / 2);
        rightTop.height = height / (Configuration.height / 2);
        rightTop.x = 1 - xOffset - rightTop.width;
        rightTop.y = 1 - yOffset - rightTop.height;
        area[1] = rightTop;
        Rectangle leftBottom = new Rectangle();
        leftBottom.width = width / (Configuration.width / 2);
        leftBottom.height = height / (Configuration.height / 2);
        leftBottom.x = -1 + xOffset;
        leftBottom.y = -1 + yOffset;
        area[2] = leftBottom;
        Rectangle rightBottom = new Rectangle();
        rightBottom.width = width / (Configuration.width / 2);
        rightBottom.height = height / (Configuration.height / 2);
        rightBottom.x = 1 - xOffset - rightTop.width;
        rightBottom.y = -1 + yOffset;
        area[3] = rightBottom;
        penOffsetX = (Configuration.width - width)/2*Constant.SCALE/zoom;
        penOffsetY = (Configuration.height - height)/2*Constant.SCALE/zoom;
        offset = new float[4][2];
        offset[0][0] = penOffsetX;
        offset[0][1] = -penOffsetY;
        offset[1][0] = -penOffsetX;
        offset[1][1] = -penOffsetY;
        offset[2][0] = penOffsetX;
        offset[2][1] = penOffsetY;
        offset[3][0] = -penOffsetX;
        offset[3][1] = penOffsetY;

    }

    private void initCamera() {
        magnifierCamera = new OrthographicCamera(cameraWidth, cameraHeight);
        magnifierCamera.position.set(2, 6, 0);
        magnifierCamera.update();
    }

    private void changeMagnifierArea() {
        if (magnifierId == -1) return;
        clipBounds = area[magnifierId];
        if (magnifierId == 0) {
            magnifierArea.x = 0;
            magnifierArea.y = (Configuration.height - height - yOffset * Configuration.height ) * Constant.SCALE;
        } else if (magnifierId == 1) {
            magnifierArea.x = (Configuration.width - width - xOffset * Configuration.width ) * Constant.SCALE;
            magnifierArea.y = (Configuration.height - height - yOffset * Configuration.height ) * Constant.SCALE;
        } else if (magnifierId == 2) {
            magnifierArea.x = 0;
            magnifierArea.y = 0;
        } else {
            magnifierArea.x = (Configuration.width - width - xOffset * Configuration.width ) * Constant.SCALE;
            magnifierArea.y = 0;
        }
    }

    public void update(Vector2 position) {
        this.position.set(position);
        if (magnifierArea.contains(position)) {
            checkMagnifierArea();
        }
        tmpVector2.set(position).scl(1/Constant.SCALE);
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        if (magnifier) {
            magnifier(batch);
        }
    }
    private Vector2 tmpVector2 = new Vector2();

    public void magnifier(Batch batch) {
        if (magnifierId == -1) return;
        if (magnifierBg == null) {
            magnifierBg = new NinePatch(Assets.instance.game.findRegion("bg_frame_green"), 15, 15, 15, 15);
        }
        magnifierCamera.position.set(position.x + offset[magnifierId][0], position.y + offset[magnifierId][1], 0);
//        magnifierCamera.position.set(3,5, 0);
        magnifierCamera.update();
        batch.flush();
        ScissorStack.calculateScissors(magnifierCamera, magnifierCamera.invProjectionView, clipBounds, scissors);
        ScissorStack.pushScissors(scissors);
        batch.setProjectionMatrix(magnifierCamera.combined);
        gamePanel.drawMagnifier(batch);
        batch.flush();
        ScissorStack.popScissors();
        batch.setProjectionMatrix(Matrix4s.ui);

        magnifierBg.draw(batch, (area[magnifierId].getX() + 1) * Configuration.width / 2 - 5, (area[magnifierId].getY() + 1) * Configuration.height / 2 - 5, 0, 0, width + 10, height + 10, 1, 1, 0);
    }

    private void checkMagnifierArea() {
        int mask = 0;
        magnifierId++;
        magnifierId %= 4;
        while (!magnifierOpen[magnifierId] && mask++ < 4) {
            magnifierId++;
            magnifierId %= 4;
        }
        if (mask >= 4) {
            magnifierId = -1;
        }
        changeMagnifierArea();
    }

    public void setMagnifier(boolean magnifier) {
        this.magnifier = magnifier;
    }
}
