package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.CircleShape;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.rube.CustomProperties;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.constants.Constant;

/**
 * Created by Doodle on 2018/10/9.
 */

public class MovePlatform extends BaseObject {
    private Vector2 lineVelocity;
    private float distance;
    private float stateTime = 0;
    private float time = 0;
    TextureRegion region;
    TextureRegion lineRegion;
    float width, height;
    float lineRegionHeight;
    Body body;
    Vector2 bodyPosition = new Vector2();
    float angle;
    float rotationLine;
    float rotation;

    public MovePlatform(Body body, CustomProperties properties, GamePanel panel) {
        super(panel);
        this.body = body;
        lineVelocity = body.getLinearVelocity();
        if (properties.get("distance") != null) {
            distance = properties.get("distance");
        } else {
            distance = properties.get("move_distance");
        }
        bodyPosition.set(body.getPosition());
        rotationLine = body.getLinearVelocity().angle();
        time = distance / lineVelocity.len();

        TextureRegion tmpRegion = Assets.instance.game.findRegion("move_platform_line");
        lineRegionHeight = tmpRegion.getRegionHeight();
        lineRegion = new TextureRegion();
        lineRegion.setRegion(tmpRegion, 0, 0, (int) (distance / Constant.SCALE * 2), tmpRegion.getRegionHeight());
        Vector2 vector2 = new Vector2();

        this.region = Assets.instance.game.findRegion("move_platform");
        ((PolygonShape) body.getFixtureList().get(0).getShape()).getVertex(0, vector2);
        width = Math.abs(vector2.x * 2);
        height = Math.abs(vector2.y * 2);

        this.angle = body.getAngle();
        rotation = (float) (angle / Math.PI * 180);
    }

    public void updateUI(float delta) {
        if (gamePanel.getState() == GamePanel.GameState.gaming
                ||gamePanel.getState() == GamePanel.GameState.end) {
//            if (body.getLinearVelocity().len() != 0) {
            stateTime += delta;
//            }
            if (stateTime >= time) {
                body.setLinearVelocity(lineVelocity.scl(-1));
                stateTime = 0;
            }
        }
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        float x = body.getPosition().x;
        float y = body.getPosition().y;
        batch.draw(lineRegion,
                bodyPosition.x,
                bodyPosition.y - lineRegionHeight / 2,
                0,
                lineRegionHeight / 2,
                (int) (distance / Constant.SCALE * 2),
                lineRegionHeight,
                Constant.SCALE / 2,
                Constant.SCALE / 2,
                rotationLine);
        batch.draw(region, x - width / 2, y - height / 2, width / 2, height / 2, width, height, 1, 1, rotation);

    }
}
