package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.MathUtils;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.assets.GoodData;
import com.zyb.loveball.audio.AudioProcess;

public class Pen {
    private float inkMax;
    private float inkLeft;
    private float inkUsed;
    private int writeTimes;
    Sprite region;
    float x, y;
    private boolean visible;
    private float startX, startY;
    private float paintX, paintY;
    PenState state = PenState.none;
    float lastTime;
    GoodData data;
    long soundId;
    private String soundName = "draw";

    public Pen(int inkMax, int inkLeft) {
        this.inkMax = inkMax;
        this.inkLeft = inkLeft;
        data = Assets.instance.getGoodData(Configuration.settingData.getPenId(), 0);
        region = Assets.instance.game.createSprite(data.getResources());
        region.setScale(0.003F);
        region.setOrigin(region.getWidth() / 2, 0);
        region.setRotation(-10);
        soundId = AudioProcess.playSoundLoop(soundName,0);
        AudioProcess.setVolume(soundId,0,"draw");
    }

    public boolean begin(float x, float y) {
        this.visible = true;
        startX = x;
        startY = y;
        paintX = startX;
        paintY = startY;
        lastTime = System.nanoTime();
        audioX = -100;
        audioY = -100;
        AudioProcess.setVolume(soundId,0,"draw");
        return true;
    }

    public boolean end() {
        this.visible = false;
        if (state == PenState.writing) {
            state = PenState.none;
            writeTimes++;
        }
        AudioProcess.setVolume(soundId,0,"draw");
        return true;
    }

    public void addInk() {
        inkLeft = inkMax;
    }

    public boolean paint(float x, float y) {
        if (!canWrite()) return false;
        float offset = (float) Math.sqrt((x - paintX) * (x - paintX) + (y - paintY) * (y - paintY));
        inkUsed += 10 * offset;
        inkLeft -= 10 * offset;
        paintX = x;
        paintY = y;
        state = PenState.writing;
        if (inkLeft < 0) inkLeft = 0;
        return true;

    }

    public boolean canWrite() {
        if (writeTimes >= PenConfig.max_write_times) return false;
        if (!PenConfig.ink_off_write && inkLeft == 0) return false;
        return true;
    }

    public void update(float x, float y) {
        this.x = x;
        this.y = y;
    }

    public float getInkLeft() {
        return inkLeft;
    }

    public float getInkMax() {
        return inkMax + Float.MIN_VALUE;
    }

    public float getInkUsed() {
        return inkUsed;
    }

    public float inkLeftPercent() {
        return inkLeft / inkMax;
    }

    int i = 0;
    float audioX;
    float audioY;
    float volume;

    private void updateAudio() {
        i++;
        if (i < 3) return;
        i = 0;
        if (audioX == -100) {
            audioX = paintX;
            audioY = paintY;
        }
        float offset = (float) Math.sqrt((audioX - paintX) * (audioX - paintX) + (audioY - paintY) * (audioY - paintY));
        audioY = paintY;
        audioX = paintX;
        if (soundId != -1) {
            float pitch = (float) Math.sqrt(offset) * 3;
            float dstVolume = (float) Math.sqrt(offset) * 2;
            pitch = MathUtils.clamp(pitch, 0.5F, 1.5F);
            dstVolume = MathUtils.clamp(dstVolume, 0F, 1F);
            if(offset< 0.01F){
                pitch = 0;
                dstVolume = 0;
            }
            float volumeOffset = Math.abs(dstVolume-volume)/3;
            System.out.println(volumeOffset);
            volumeOffset = MathUtils.clamp(volumeOffset,0.1F,0.4F);

            if(volume<dstVolume){
                volume+= volumeOffset;
            }
            if(volume>dstVolume){
                volume-=volumeOffset;
            }
            AudioProcess.setVolume(soundId,volume,"draw");
//            AudioProcess.setPitch(soundId, pitch, "draw");
        }
    }

    public void draw(Batch batch) {
        if (!visible) return;
        updateAudio();
        batch.setColor(Color.WHITE);
        region.setPosition(x - region.getOriginX(), y);
        region.draw(batch, 1);
    }



    public void reset(float inkLeft){
        this.inkLeft = inkLeft;
        inkUsed = 0;
    }

    enum PenState {
        none, writing
    }

    public static class PenConfig {
        public static int max_write_times = Integer.MAX_VALUE;
        public static boolean ink_off_write = true;
    }
}
