package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.JointDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.joints.RevoluteJointDef;
import com.rube.CustomProperties;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.constants.Constant;

/**
 * Created by Doodle on 2018/9/7.
 */

public class Seesaw extends BaseObject {
    private Body jointBody;
    private RevoluteJointDef jointDef;
    private NinePatch ninePatch;
    private Vector2 tmpVector = new Vector2();
    private TextureRegion seesawPoint1;
    private TextureRegion seesawPoint2;
    private Vector2 centerVector;
    private float targetScale = 1/160f;
    public Seesaw(Body body, GamePanel gamePanel, CustomProperties properties) {
        super(gamePanel);
        TextureRegion seesaw = Assets.instance.game.findRegion("seesaw");
        seesawPoint1 = Assets.instance.game.findRegion("seesaw_point1");
        seesawPoint2 = Assets.instance.game.findRegion("seesaw_point3");
        ninePatch = new NinePatch(seesaw,25,25,3,3);
        this.body = body;
        body.getFixtureList().get(0).setDensity(Constant.seesaw.density);
        body.resetMassData();
        jointBody = gamePanel.createBody(new BodyDef());
        jointDef = new RevoluteJointDef();
        jointDef.type = JointDef.JointType.RevoluteJoint;
        centerVector = properties.get("center");
        if (centerVector == null) {
            centerVector = body.getPosition();
        }
        jointDef.initialize(body, jointBody, centerVector);
        gamePanel.getWorld().createJoint(jointDef);
        body.setAngularDamping(0.1F);
        tmpVector.set(body.getPosition());
    }
    Vector2 point1 = new Vector2();
    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch,parentAlpha);
        ((PolygonShape)body.getFixtureList().get(0).getShape()).getVertex(1,point1);
        float bodyWidth = Math.abs(point1.x*2/targetScale);
        float bodyHeight = Math.abs(point1.y*2/targetScale);
        ninePatch.draw(batch,
                tmpVector.x -bodyWidth/2,
                tmpVector.y -bodyHeight/2,
                bodyWidth/2,
                bodyHeight/2,
                bodyWidth,
                bodyHeight,
                targetScale,
                targetScale,
                body.getAngle()/(float) Math.PI*180f);
        batch.draw(seesawPoint1,
                centerVector.x- seesawPoint1.getRegionWidth()/2,
                centerVector.y- seesawPoint1.getRegionHeight()/2,
                seesawPoint1.getRegionWidth()/2,
                seesawPoint1.getRegionHeight()/2,
                seesawPoint1.getRegionWidth(),
                seesawPoint1.getRegionHeight(),
                targetScale,
                targetScale,
                0);
        batch.draw(seesawPoint2,
                centerVector.x- seesawPoint2.getRegionWidth()/2,
                centerVector.y- seesawPoint2.getRegionHeight()/2,
                seesawPoint2.getRegionWidth()/2,
                seesawPoint2.getRegionHeight()/2,
                seesawPoint2.getRegionWidth(),
                seesawPoint2.getRegionHeight(),
                targetScale,
                targetScale,
                0);
    }
}
