package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.PolygonRegion;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.*;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.DelayedRemovalArray;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.attributes.StaticObjectAttribute;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.utils.MathUtil;
import com.zyb.loveball.utils.MergePolygon;

import java.util.ArrayList;

public class StaticScene extends BaseObject {
    Array<StaticObject> objects = new Array<>();
    TextureRegion baseRegion;
    DelayedRemovalArray<float[]> vertexes = new DelayedRemovalArray<>();
    TextureRegion circleRegion;
    Array<PolygonRegion> regions = new Array<>();
    Color color = new Color(150 / 255F, 98 / 255F, 93 / 255F, 1);

    public StaticScene(GamePanel gamePanel) {
        super(gamePanel);
        circleRegion = Assets.instance.game.findRegion("circle_black");
        baseRegion = Assets.instance.ui.findRegion("white");
    }

    public void addImage(Body body) {
        Array<Fixture> fixtures = body.getFixtureList();
        Vector2 position = body.getPosition();
        float angle = body.getAngle();
        DelayedRemovalArray<float[]> tempArray = new DelayedRemovalArray<>();
        for (int i = 0; i < fixtures.size; i++) {
            Fixture fixture = fixtures.get(i);
            initGround(fixture);
            Shape shape = fixture.getShape();
            float[] tmpVertexes = getVertexesFromShape(shape, position, angle);
            if(tmpVertexes == null) return;
            if (tempArray.size == 0) {
                tempArray.add(tmpVertexes);
            } else {
                boolean overlapping = false;
                tempArray.begin();
                for (int j = 0; j < tempArray.size; j++) {
                    float[] tempVertex = tempArray.get(j);
                    if (MathUtil.checkOverlapping(tempVertex, tmpVertexes)) {
                        overlapping = true;
                        tempArray.removeValue(tempVertex, true);
                        ArrayList<double[]> result = MergePolygon.mergePolygonPoints(tempVertex, tmpVertexes);
                        tempVertex = new float[result.size() * 2];
                        for (int k = 0; k < result.size(); k++) {
                            tempVertex[k * 2] = (float) (result.get(k)[0]);
                            tempVertex[k * 2 + 1] = (float) (result.get(k)[1]);
                        }
                        tempArray.add(tempVertex);
                        break;
                    }
                }
                tempArray.end();
                if (!overlapping) {
                    tempArray.add(tmpVertexes);
                }else{
                    checkPolygonOverlapping(tempArray);
                }
            }
        }
        for (int i = 0;i<tempArray.size;i++){
            vertexes.add(tempArray.get(i));
            checkPolygonOverlapping(vertexes);
        }

    }

    private void initGround(Fixture fixture){
        fixture.setFriction(Constant.ground.friction);
        fixture.setFilterData(StaticObjectAttribute.filter);
    }

    Vector2 tmpVector = new Vector2();

    private float[] getVertexesFromShape(Shape shape, Vector2 position, float angle) {
        float[] vertexes = null;
        if (shape instanceof PolygonShape) {
            int size = ((PolygonShape) shape).getVertexCount();
            vertexes = new float[size * 2];
            for (int i = 0; i < size; i++) {
                ((PolygonShape) shape).getVertex(i, tmpVector);
                float length = tmpVector.len();
                float localAngle = (float) (MathUtil.getAngle(0, 0, tmpVector.x, tmpVector.y));
                vertexes[2 * i] = (float) (position.x + Math.cos(localAngle + angle) * length);
                vertexes[2 * i + 1] = (float) (position.y + Math.sin(localAngle + angle) * length);
            }
            ChainShape chainShape = new ChainShape();
            chainShape.createLoop(vertexes);
        }else if(shape instanceof CircleShape){
            float radius = shape.getRadius();
            int size = (int)(50 * radius);
            size = MathUtils.clamp(size,20,Integer.MAX_VALUE);
            vertexes = new float[size*2];
            float perOffset = 360F/size;
            for (int i = 0;i<size;i++){
                double tempAngle = (2*Math.PI/360) * i * perOffset;
                vertexes[2*i] =  position.x+ (float) Math.sin(tempAngle)* radius;
                vertexes[2*i+1] =position.y -  (float)Math.cos(tempAngle)* radius;
            }
        }else if(shape instanceof ChainShape){
            int size = ((ChainShape) shape).getVertexCount();
            vertexes = new float[size*2];
            for (int i = 0;i<size;i++){
                 ((ChainShape) shape).getVertex(i,tmpVector);
                float length = tmpVector.len();
                float localAngle = (float) (MathUtil.getAngle(0, 0, tmpVector.x, tmpVector.y));
                vertexes[2 * i] = (float) (position.x + Math.cos(localAngle + angle) * length);
                vertexes[2 * i + 1] = (float) (position.y + Math.sin(localAngle + angle) * length);
            }

        }
        return vertexes;
    }

    private void checkPolygonOverlapping(DelayedRemovalArray<float[]> vertexesCheck) {
        boolean merged = false;
        vertexesCheck.begin();
        for (int i = 0; i < vertexesCheck.size && !merged; i++) {
            float[] vertices1 = vertexesCheck.get(i);
            for (int j = i + 1; j < vertexesCheck.size && !merged; j++) {
                float[] vertices2 = vertexesCheck.get(j);
                if (MathUtil.checkOverlapping(vertices1, vertices2)) {
                    merged = true;
                    vertexesCheck.removeValue(vertices1, true);
                    vertexesCheck.removeValue(vertices2, true);
                    ArrayList<double[]> result = MergePolygon.mergePolygonPoints(vertices1, vertices2);
                    float[] tempVertex = new float[result.size() * 2];
                    for (int k = 0; k < result.size(); k++) {
                        tempVertex[k * 2] = (float) (result.get(k)[0]);
                        tempVertex[k * 2 + 1] = (float) (result.get(k)[1]);
                    }
                    vertexesCheck.add(tempVertex);
                    break;
                }
            }
        }
        vertexesCheck.end();
        if (merged) {
            checkPolygonOverlapping(vertexesCheck);
        }

    }

    public void end() {
        for (int i = 0; i < vertexes.size; i++) {
            float[] tempVertexes = vertexes.get(i);
            PolygonRegion polygonRegion = new PolygonRegion(baseRegion, tempVertexes, MathUtil.computeTriangles(tempVertexes).toArray());
            float[] textureCoords = polygonRegion.getTextureCoords();
            float u = baseRegion.getU();
            float v = baseRegion.getV();
            float u2 = baseRegion.getU2();
            float v2 = baseRegion.getV2();

            for (int j = 0; j < textureCoords.length; j += 2) {
                textureCoords[j] = MathUtils.clamp(textureCoords[j], u+0.001F, u2-0.001F);
                textureCoords[j + 1] = MathUtils.clamp(textureCoords[j + 1], v+0.001F, v2-0.001F);
            }
            regions.add(polygonRegion);
        }
    }


    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        Color oldColor = batch.getColor();
        batch.setColor(color);
        for (int i = 0; i < regions.size; i++) {
            ((PolygonSpriteBatch) batch).draw(regions.get(i), 0, 0);
        }
        for (int i = 0; i < vertexes.size; i++) {
            float[] temp = vertexes.get(i);
            if (temp != null && temp.length >= 6) {
                for (int j = 0; j < temp.length / 2; j++) {
                    float x = temp[j * 2];
                    float y = temp[j * 2 + 1];
                    batch.draw(circleRegion, x - 5, y - 5, 5, 5, 10, 10, 1 / 400F, 1 / 400F, 0);
                }
            }

        }
        batch.setColor(oldColor);
        renderOutline(batch);
    }

    private void renderOutline(Batch batch) {
        if (vertexes.size > 0) {
            batch.end();
            ShapeRenderer renderer = HelloZombieGame.getShapeRenderer();
            renderer.begin(ShapeRenderer.ShapeType.Filled);
            renderer.setColor(Color.BLACK);
            renderer.setProjectionMatrix(batch.getProjectionMatrix());
            for (int i = 0; i < vertexes.size; i++) {
                float[] temp = vertexes.get(i);
                if (temp != null && temp.length >= 6) {
                    for (int j = 0; j <= temp.length - 2; j += 2) {
                        float x1 = temp[j];
                        float y1 = temp[j + 1];
                        float x2 = temp[0];
                        float y2 = temp[1];
                        if (j == temp.length - 2) {
                        } else {
                            x2 = temp[j + 2];
                            y2 = temp[j + 3];
                        }
                        renderer.rectLine(x1, y1, x2, y2, 0.035F);
                    }
                }
            }
            renderer.end();
            batch.begin();
        }
    }

}
