package com.zyb.loveball.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.*;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.ArrayMap;
import com.badlogic.gdx.utils.ObjectMap;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.animations.SkeletonAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.constants.Constant;

import java.util.*;

/**
 * Created by Doodle on 2018/11/6.
 */

public class Vortex extends BaseObject {
    SkeletonAnimation vortex;
    TextureRegion region;
    Vector2 position;
    float width, height;
    Set<Body> bodies = new HashSet<>();
    float targetScale = 1 / 160f;
    Filter filter = new Filter();

    public Vortex(Body body, GamePanel gamePanel) {
        super(gamePanel);
        filter.maskBits = 0;
        this.body = body;
        this.region = Assets.instance.game.findRegion("vortex");
        position = body.getPosition();
        width = region.getRegionWidth();
        height = region.getRegionHeight();
        init();
        initAnimation();
    }

    public void init() {
        body.getFixtureList().get(0).setSensor(true);
        body.setContactListener(contactListener);
    }

    public void initAnimation() {
        vortex = new SkeletonAnimation("animations/xuanwo.skel", 1 / 400f);
        vortex.setPosition(position.x, position.y);
        vortex.getState().setAnimation(0, "animation", true);
    }

    BodyContactAdapter contactListener = new BodyContactAdapter() {
        public void beginContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
            super.beginContact(contact, fixtureA, fixtureB, isA);
            Body tmpBody;
            if (isA) {
                tmpBody = fixtureB.getBody();
            } else {
                tmpBody = fixtureA.getBody();
            }

            String name = gamePanel.getScene().getName(tmpBody);
            if (name == null || name.endsWith("dynamic") || name.startsWith("ball") || name.startsWith("heart")) {
                if(bodies.add(tmpBody)){
                    noCollision(tmpBody);
                }
            }
        }
    };

    float angle = 0;
    private void noCollision(Body body){
        Array<Fixture> list = body.getFixtureList();
        for (int i = 0;i<list.size;i++){
            Fixture fixture = list.get(i);
            fixture.setFilterData(filter);
        }
    }

    @Override
    public void updateUI(float delta) {
        super.updateUI(delta);
        angle -= 10;
        angle %= -360;
        vortex.update();
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        batch.draw(region, position.x - width / 2, position.y - height / 2, width / 2, height / 2, width, height, targetScale, targetScale, angle);
        vortex.draw(batch);
    }

    public void updateWorld(float delta) {
        super.updateWorld(delta);
        Iterator<Body> it = bodies.iterator();
        while (it.hasNext()) {
            final Body tmpBody = it.next();
            Vector2 tmpVector2 = tmpBody.getWorldPoint(tmpBody.getMassData().center).sub(position);
            if(tmpVector2.len()<0.05F){
                it.remove();
                gamePanel.removeObject(tmpBody);
            }else {
                tmpBody.setLinearVelocity(tmpVector2.scl(-5));
            }
        }

    }
}
