package com.zyb.loveball.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyContactAdapter;
import com.badlogic.gdx.physics.box2d.Contact;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.animations.WarpGateAnimation;

/**
 * Created by Doodle on 2018/9/10.
 */

public class WarpGate extends BaseObject {
    WarpGateAnimation inAni;
    WarpGateAnimation outAni;
    float outX, outY, outAngle;

    public WarpGate(Body body, GamePanel gamePanel) {
        super(gamePanel);
        this.body = body;
        this.gamePanel = gamePanel;
        body.setContactListener(contactListener);
        Vector2 temp = body.getPosition();
        inAni = new WarpGateAnimation(temp.x, temp.y, body.getAngle() / (float) Math.PI * 180, true);
    }

    public void setOut(Body body) {
        Vector2 temp = body.getPosition();
        outX = temp.x;
        outY = temp.y;
        outAngle = body.getAngle();
        outAni = new WarpGateAnimation(temp.x, temp.y, outAngle / (float) Math.PI * 180, false);
    }

    @Override
    public void updateUI(float delta) {
        super.updateUI(delta);
        inAni.update();
        outAni.update();;
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        inAni.draw(batch);
        outAni.draw(batch);
    }

    Vector2 outVelocity = new Vector2();
    BodyContactAdapter contactListener = new BodyContactAdapter() {
        @Override
        public void beginContact(Contact contact, Fixture fixtureA, Fixture fixtureB, boolean isA) {
            final Body tmpBody;
            inAni.in();
            Fixture fixture;
            if (isA) {
                fixture = fixtureB;
            } else {
                fixture = fixtureA;
            }
            tmpBody = fixture.getBody();
            String name = gamePanel.getScene().getName(tmpBody);
            if (name != null && (name.startsWith("ball") || name.startsWith("heart"))) {
                outVelocity.set(tmpBody.getLinearVelocity());
                Gdx.app.postRunnable(new Runnable() {
                    @Override
                    public void run() {
                        float speed = outVelocity.len();
                        if(speed>10){
                            speed = 10;
                        }
                        tmpBody.setTransform(outX, outY, tmpBody.getAngle());
                        tmpBody.setLinearVelocity((float) Math.cos(outAngle) * speed, (float) Math.sin(outAngle) * speed);
                        outAni.out();
                    }
                });
            } else if (name == null || name.endsWith("dynamic")) {
                gamePanel.removeObject(tmpBody);
            }
        }
    };


}
