package com.zyb.loveball.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.Filter;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.physics.box2d.JointDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.joints.WheelJointDef;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.animations.FlyZombieAnimation;
import com.zyb.loveball.animations.StandZombieAnimation;
import com.zyb.loveball.animations.ZombieAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.attributes.ZombieAttribute;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.utils.Log;

public class Zombie extends BaseObject {
    TextureRegion region;
    String bodyName;
    Vector2 position;
    ZombieAnimation zombieAnimation;
    private ZombieListener zombieListener;
    Vector2 tmpVector2 = new Vector2();
    ZombieState zombieState = ZombieState.none;
    ZombieBoom zombieBoom;
    int type = 1;
    public static boolean outDie = true;

    public Zombie(Body body, String name, GamePanel gamePanel,int type) {
        super(gamePanel);
        this.body = body;
        this.gamePanel = gamePanel;
        this.bodyName = name;
        this.type = type;
        region = Assets.instance.game.findRegion("heart");
        position = body.getPosition();
        body.destroyFixture(body.getFixtureList().get(0));
        body.setFixedRotation(true);
        PolygonShape polygonShape = new PolygonShape();
        if(type==1) {
            polygonShape.set(new float[]{-0.1f, -0.25f, 0.2f, -0.25f, 0.2f, 0.55f, -0.1f, 0.55f});
        }else if(type == 2){
            polygonShape.set(new float[]{-0.15f, -0.28f, 0.25f, -0.28f, 0.25f, 0.7f, -0.15f, 0.7f});
        }
        Fixture fixture = body.createFixture(polygonShape, Constant.zombie.density);
        fixture.setFriction(Constant.zombie.friction);
        fixture.setFilterData(ZombieAttribute.filter);
        body.getFixtureList().get(0).setSensor(true);
        if(type==1) {
            body.setType(BodyDef.BodyType.DynamicBody);
            zombieAnimation = new StandZombieAnimation();
            body.getFixtureList().get(0).setSensor(false);
        }else if(type==2){
            body.setType(BodyDef.BodyType.StaticBody);
            zombieAnimation = new FlyZombieAnimation();
            body.getFixtureList().get(0).setSensor(true);
        }
        body.setFixedRotation(true);
        zombieBoom = new ZombieBoom(gamePanel,type);
        if(position.x>3){
            zombieAnimation.faceLeft(true);
        }else{
            zombieAnimation.faceLeft(false);
        }
        zombieState = ZombieState.idle;
        zombieAnimation.idle();

    }


    public Ball.BombListener initBombListener() {
        return new Ball.BombListener() {
            @Override
            public void getPosition(Vector2 position) {
                tmpVector2.set(Zombie.this.position.x - position.x, Zombie.this.position.y - position.y);
                boolean faceLeft = tmpVector2.x > 0 ? true : false;
                if (tmpVector2.len() < 2) {
                    if (zombieState == ZombieState.idle || zombieState == ZombieState.none) {
                        zombieState = ZombieState.scared;
                        System.out.println("scared");
                        zombieAnimation.scared();
                    }
                } else {
//                    if (zombieState == ZombieState.scared || zombieState == ZombieState.none) {
//                        zombieState = ZombieState.idle;
//                        zombieAnimation.idle();
//                    }
                }
                if(tmpVector2.len()<3){
                    zombieAnimation.faceLeft(faceLeft);
                }
            }

            @Override
            public void boom(int ballId) {
                Zombie.this.boom(ballId);
            }

            @Override
            public void ballOut() {
                zombieAnimation.happy();
            }
        };
    }


    public void boom(int ballId) {
        Log.log("Zombie","boom()");
        if(zombieState == ZombieState.boom) return;
        zombieState = ZombieState.boom;
        boomDung(ballId);
    }

    private void boomDung(int ballId){
        zombieAnimation.boom(ballId);
        gamePanel.activeBody(body, false);
        if(ballId == Constant.BOMB_ID_1|| ballId == Constant.BOMB_ID_2) return;
        Gdx.app.postRunnable(new Runnable() {
            @Override
            public void run() {
                zombieBoom.load(position);
            }
        });
        Gdx.input.vibrate(100);
    }

    public void setZombieListener(ZombieListener zombieListener) {
        this.zombieListener = zombieListener;
        zombieListener.getPosition(position);
    }

    public Vector2 getPosition() {
        return position;
    }

    float rotation;
    Vector2 velocity = new Vector2();
    float angularVelocity;

    @Override
    public void updateUI(float delta) {
        super.updateUI(delta);
        position.set(body.getPosition());
        rotation = body.getAngle() / (float) Math.PI * 180;
        velocity.set(body.getLinearVelocity());
        angularVelocity = body.getAngularVelocity();
        zombieAnimation.update(position.x, position.y);
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        if (zombieState == ZombieState.boom) {
            zombieBoom.draw(batch, parentAlpha);
        }
        zombieAnimation.draw(batch);
    }

    public void out(){
        Log.log("Zombie","out()");
        if(outDie){
            gamePanel.end(false);
        }else{
            gamePanel.end(true);
        }
    }

    public interface ZombieListener {
        public void getPosition(Vector2 position);
    }

    enum ZombieState {
        none, idle, scared, boom
    }

}
