package com.zyb.loveball.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.DelayedRemovalArray;
import com.rube.*;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.attributes.DynamicObjectAttribute;
import com.zyb.loveball.attributes.ZombieAttribute;
import com.zyb.loveball.utils.MathUtil;
import com.zyb.loveball.utils.MyRandom;

public class ZombieBoom extends BaseObject{
    Vector2 position = new Vector2();
    DelayedRemovalArray<Body> bodies = new DelayedRemovalArray<>();
    Drawer drawer;
    WorldData data;
    Vector2 tmpVector = new Vector2();
    float positionOffsetY = 0.25F;
    protected DelayedRemovalArray<ImageUtils> images = new DelayedRemovalArray<>();
    public ZombieBoom(GamePanel gamePanel,int type){
        super(gamePanel);
        drawer = gamePanel.getDrawer();
        if(type == 1) {
//            data = HelloZombieGame.getData1();
            data = JsonLoader.load("animations/zombieBoom.json");
        }else if(type==2){
//            data = HelloZombieGame.getData2();
            data = JsonLoader.load("animations/zombieBoom2.json");
        }
    }

    public void load(Vector2 vector2){
        position.set(vector2.x,vector2.y-positionOffsetY);
        JsonLoader.loadImages(data, Assets.instance.game);
        gamePanel.getBox2dLoader().load(gamePanel.getWorld(),data,this.position);
        bodies.addAll(gamePanel.getBox2dLoader().getBodies());
        loadImages(data);
        initBodies();
        boom();
    }


    private void initBodies(){
        for(Body body:bodies) {
            Array<Fixture> fixtureList = body.getFixtureList();
            for (Fixture fixture : fixtureList) {
                fixture.setFilterData(ZombieAttribute.piecefilter);
                fixture.setRestitution(0.3f);
                fixture.setFriction(0.4f);
            }
        }
    }


    private void boom(){
        for(Body body:bodies){
            Vector2 bodyPosition = body.getPosition();
            tmpVector.set(bodyPosition);
            tmpVector.sub(position.x,position.y+positionOffsetY);
            tmpVector.scl(70);
            float mass = body.getMass();
            mass =(float) Math.sqrt(mass);
            tmpVector.scl(mass);
            mass = mass* mass;
            boolean left = MyRandom.getInstance().nextBoolean();
            if(left) {
                body.applyAngularImpulse(mass / 30F, true);
            }else{
                body.applyAngularImpulse(-mass / 30F, true);
            }
            body.applyForceToCenter(tmpVector,true);
        }
    }

    protected void loadImages(WorldData data) {
        for (int i = 0; i < data.getImageList().size; i++) {
            ImageData temp = data.getImageList().get(i);
            if (temp.getBodyIndex() != -1) {
                images.add(new ImageUtils(bodies.get(temp.getBodyIndex()), temp));
            } else {
                images.add(new ImageUtils(null, temp));
            }
        }
    }


    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
        for (ImageUtils imageUtils : images) {
            imageUtils.draw(drawer, batch);
        }
    }
}
