package com.zyb.loveball.screens;

import com.badlogic.gdx.*;
import com.badlogic.gdx.scenes.scene2d.*;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.animations.CommonAnimation;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.dialogs.BaseDialog;
import com.zyb.loveball.dialogs.CoinDialog;
import com.zyb.loveball.dialogs.SuccessDialog;
import com.zyb.loveball.ui.Grid;
import com.zyb.loveball.utils.CocosUtil;
import com.zyb.loveball.utils.FlurryUtils;
import com.zyb.loveball.utils.LayerUtils;
import com.zyb.loveball.utils.Log;
import com.zyb.loveball.utils.MScreenUtil;
import com.zyb.loveball.utils.listener.SoundButtonListener;

import org.freyja.libgdx.cocostudio.ui.model.CCExport;
import org.freyja.libgdx.cocostudio.ui.parser.group.CCScene;

import java.util.Iterator;
import java.util.Stack;

/**
 * Created by lilexiang on 17-8-15.
 */
public class BaseScreen extends CocosUtil implements Screen {
    protected HelloZombieGame game;
    protected Stage stage;
    protected Actor layer;
    protected Stack<BaseDialog> dialogs = new Stack<>();
    protected String TAG = "";
    protected Group dialogsGroup;
    protected Label coinLabel;
    protected boolean screenPaused;
    private PendingAction pendingAction = PendingAction.NONE;
    protected int pendingValue = 0;
    protected ScreenState state;
    protected Class pendingScreen;
    float stateTime;
    protected float outTime = 0.3F;
    protected float inTime = 0.3F;


    public BaseScreen(HelloZombieGame game) {
        this.game = game;
        stage = new Stage(game.getViewport(), HelloZombieGame.getBatch());
    }

    @Override
    public void show() {
        state = ScreenState.show;
        layer = LayerUtils.createLayer();
        loadFromJson(TAG);
        if (scene != null) {
            stage.addActor(scene);
        }
        if (TAG.equals("load")) {
            start();
        } else {
            scene.addActor(layer);
            layer.setColor(0, 0, 0, 1);
            layer.addAction(Actions.sequence(Actions.fadeOut(inTime), Actions.removeActor()));
        }
        Actor bg = findActor("bg");
        if (bg != null && !(this instanceof GameScreen)) {
            Grid grid = new Grid();
            grid.setSize(480, 1000);
            scene.addActorAfter(bg, grid);
        }
        initAnimation();
//        initAds();
        Log.log("BaseScreen", "show()" + getClass());
        stateTime = 0;
    }

//    private void initAds() {
//        if (this instanceof GameScreen) {
//            HelloZombieGame.launcherListener.showBanner(false);
//            return;
//        }
////        scene.addAction(Actions.forever(Actions.sequence(Actions.run(bannerCheckRunnable), Actions.delay(1))));
//    }

//    Runnable bannerCheckRunnable = new Runnable() {
//        @Override
//        public void run() {
//            checkBanner();
//        }
//    };
//
//    private void checkBanner() {
//        if (Configuration.settingData.isFirstPlay()&&Configuration.banner_times == 0) {
//            if (Configuration.banner_time > 1 * 60) {
//                showBanner(true);
//            }
//        } else {
//            if (Configuration.bannerShow && Configuration.banner_time > 15) {
//                showBanner(false);
//            }
//
//            if (!Configuration.bannerShow && Configuration.banner_time > 1 * 60) {
//                showBanner(true);
//            }
//        }
//    }
//
//    private void showBanner(boolean show) {
//        Gdx.app.log("BaseScreen", "show banner!" + show);
//        Configuration.bannerShow = show;
//        Configuration.banner_time = 0;
//        HelloZombieGame.launcherListener.showBanner(show);
//        if (show) {
//            Configuration.banner_times++;
//        }
//    }


    protected void start() {
        if (state == ScreenState.start) return;
        Log.log("BaseScreen", "start()--->" + getClass() + "   gra:" + Gdx.graphics.getWidth() + "  " + Gdx.graphics.getHeight());
        state = ScreenState.start;
        HelloZombieGame.addInputProcessor(stage);
        HelloZombieGame.resumeInputProcessor();
        if (!(this instanceof GameScreen)) {
            HelloZombieGame.launcherListener.showBanner(true);
        } else {
            HelloZombieGame.launcherListener.showBanner(false);
        }
        initDebug();
        if (layer != null) {
            layer.remove();
        }
    }

    public void initAnimation() {
        if (Configuration.poor) return;
        if (!(this instanceof GameScreen)) {
            CommonAnimation potAnimation = new CommonAnimation("animations/pot.skel", 0.7f);
            potAnimation.setColor(1, 1, 1, 0.6F);
            groups[BOTTOM].addActor(potAnimation);
            potAnimation.setPosition(338, 75);
            potAnimation.getState().setAnimation(0, "1", true);
        }
    }

    public void loadFromJson(String name) {
        CCExport export = Assets.instance.assetManager.get("ui/" + name + ".json");
        if (export == null) {
            newScene();
        } else {
            try {
                scene = game.getCocoStudioUIEditor().createGroup(export);
            } catch (Exception e) {
                e.printStackTrace();
                newScene();
            }
        }
        scene.setTransform(false);
        scene.setTouchable(Touchable.enabled);
        for (int i = 0; i < GROUP_NAME.length; i++) {
            groups[i] = scene.findActor(GROUP_NAME[i]);
            if (groups[i] == null) {
                groups[i] = new Group();
                scene.addActor(groups[i]);
            }
            groups[i].setTouchable(Touchable.childrenOnly);
            groups[i].setTransform(false);
        }
        dialogsGroup = (Group) findActor("dialogs");
        dialogsGroup.setZIndex(scene.getChildren().size);
        dialogsGroup.setTouchable(Touchable.childrenOnly);
        layout();

        Actor actor = findActor("coin_font");
        if (actor != null) {
            coinLabel = (Label) actor;
        }
    }

    private void initDebug() {
        InputAdapter keyAdapter = new InputAdapter() {
            @Override
            public boolean keyUp(int keycode) {
                if (keycode == Input.Keys.ESCAPE || keycode == Input.Keys.BACK) {
                    back();
                }
                if (Configuration.debug || Gdx.app.getType() == Application.ApplicationType.Desktop) {
                    if (keycode == Input.Keys.P) {
                        Gdx.app.postRunnable(new Runnable() {
                            @Override
                            public void run() {
                                MScreenUtil.createLevelPic(game);
                            }
                        });

                    } else if (keycode == Input.Keys.R) {
                        Configuration.resetData();
                    } else if (keycode == Input.Keys.UP) {
                        Configuration.settingData.addCoin(1000);
                        update();
                    }
                }
                return super.keyDown(keycode);
            }

        };
        HelloZombieGame.addInputProcessor(keyAdapter);
    }

    private void newScene() {
        if (TAG.equals("load")) return;
        try {
            scene = parseScene("ui/" + TAG + ".json");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void setPendingAction(PendingAction action) {
        pendingAction = action;
        game.setPendingActionState(0);
    }

    public void layout() {
        float height = stage.getHeight();
        if (height <= 800) return;
        Actor bg = findActor("bg");
        bg.setHeight(height + 2);
        groups[TOP].setY(height - 800);
        groups[CENTER].setY((height - 800) / 2);
        dialogsGroup.setY((height - 800) / 2F);
    }

    public void update() {
        Log.log("BaseScreen()", "update() --->" + getClass());
        if (game.getPendingActionState() == 1) {
            handlePendingAction(true);
        } else if (game.getPendingActionState() == -1) {
            handlePendingAction(false);
        }
        Iterator<BaseDialog> iterator = dialogs.iterator();
        while (iterator.hasNext()) {
            iterator.next().update();
        }
        if (coinLabel != null) {
            coinLabel.setText(Configuration.numType.format(Configuration.settingData.getCoin()));
        }
    }

    @Override
    public void dispose() {
        if (stage != null) {
            stage.dispose();
            stage = null;
        }
    }

    int logTime = 0;

    @Override
    public void render(float delta) {
        if (stage != null) {
            stage.act();
        }
        if (stage != null) {
            stage.draw();
        }
        if (!screenPaused)
            stateTime += Gdx.graphics.getDeltaTime();
        logTime++;
        if (logTime > 10 * 60) {
            Gdx.app.log("BaseScreen", "render()----> stateTime = " + stateTime + "  state = " + state + "  assetManager update " + Assets.instance.assetManager.update());
            logTime = 0;
        }
        if (state == ScreenState.end && Assets.instance.assetManager.update(10) && stateTime > outTime + 0.02F) {
            state = ScreenState.out;
            out();
        }

        if (state == ScreenState.show && stateTime > inTime + 0.02F) {
            start();
        }


    }


    protected void initButtons() {
        Actor btn = findActor("money", Touchable.enabled);
        if (btn != null) {
            btn.addListener(new ClickListener() {
                @Override
                public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                    showCoinDialog();
                    return super.touchDown(event, x, y, pointer, button);
                }
            });
        }
        btn = findActor("btn_back", Touchable.enabled);
        if (btn != null) {
            btn.addListener(new SoundButtonListener() {
                @Override
                public void touchUpEffect() {
                    super.touchUpEffect();
                    back();
                }
            });
        }
    }

    public BaseDialog showDialog(String attribute, Class<? extends BaseDialog> tClass) {
        return showDialog(attribute, tClass, null);
    }


    public BaseDialog showDialog(String attribute, Class<? extends BaseDialog> tClass, BaseDialog.DialogListener listener) {
        try {
            BaseDialog dialog = tClass.newInstance();
            Group group = parseScene(attribute);
            dialog.load(group);
            dialog.setBaseListener(getBaseDialogListener());
            dialogs.push(dialog);
            dialog.show(dialogsGroup);
            dialog.update();
            return dialog;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    protected void removeDialog() {
        if (!dialogs.empty()) {
            dialogs.pop();
            update();
        }
    }

    private BaseDialog.DialogListener getBaseDialogListener() {
        BaseDialog.DialogListener dialogListener = new BaseDialog.DialogAdapter() {
            @Override
            public void close() {
                removeDialog();
            }

            @Override
            public void update(boolean updated) {
                BaseScreen.this.update();
            }

            @Override
            public void screenEnd(Class screen) {
                super.screenEnd(screen);
                end(screen);
            }

            @Override
            public void showDialog(String attribute, Class<? extends BaseDialog> tClass) {
                super.showDialog(attribute, tClass);
                BaseScreen.this.showDialog(attribute, tClass);
            }
        };
        return dialogListener;
    }


    public HelloZombieGame getGame() {
        return game;
    }


    protected boolean back() {
//        if (game.isFullScreenShow()) {
//            game.closeFullScreenAd();
//            return false;
//        } else {
        if (dialogs.size() > 0) {
            BaseDialog dialog = dialogs.peek();
            if (dialog != null) {
                dialog.close();
                return false;
            }
        }
//        }
        return true;
    }


    @Override
    public void resize(int width, int height) {
        stage.getViewport().update(width, height);
    }

    @Override
    public void hide() {

    }

    @Override
    public void pause() {
        screenPaused = true;
    }


    @Override
    public void resume() {
        update();
        screenPaused = false;
    }

    protected void out() {
        Log.log("BaseScreen", "out() ---> " + pendingScreen);
        if (pendingScreen == MenuScreen.class) {
            game.setScreen(new MenuScreen(game));
        } else if (pendingScreen == GameScreen.class) {
            game.setScreen(new GameScreen(game));
        } else if (pendingScreen == ChapterScreen.class) {
            game.setScreen(new ChapterScreen(game));
        } else if (pendingScreen == LevelScreen.class) {
            game.setScreen(new LevelScreen(game));
        } else if (pendingScreen == ChallengeScreen.class) {
            game.setScreen(new ChallengeScreen(game));
        } else if (pendingScreen == StoreScreen.class) {
            game.setScreen(new StoreScreen(game));
        }
        dispose();
    }

    private void loadResource() {
        if (pendingScreen == ChapterScreen.class) {
            Assets.instance.loadChaptersAtlas();
        } else if (pendingScreen == LevelScreen.class) {
            Assets.instance.loadLevelAtlas(GameInfo.chapter);
        }
//        if (pendingScreen == LevelScreen.class) {
//            Assets.instance.loadLevelAtlas(GameInfo.chapter);
//        }
    }

    public void end(Class screen) {
        if (state == ScreenState.end) return;
        if (screen == null) return;
        Log.log("BaseScreen", "end()----> " + screen);
        stateTime = 0;
        HelloZombieGame.launcherListener.showBanner(false);
        stateTime = 0;
        pendingScreen = screen;
        state = ScreenState.end;
        HelloZombieGame.clearInputProcessor();
        layer.remove();
        layer.setColor(0, 0, 0, 0);
        stage.addActor(layer);
        layer.getActions().clear();
        layer.addAction(Actions.sequence(Actions.fadeIn(outTime)));
        loadResource();
    }


    public void showCoinDialog() {
        CoinDialog coinDialog = (CoinDialog) showDialog("ui/dialogs/coin.json", CoinDialog.class);
        coinDialog.setCoinDialogListener(new CoinDialog.CoinDialogListener() {
            @Override
            public void playVideoAds() {
                setPendingAction(PendingAction.ADD_COIN);
                pendingValue = 200;
                HelloZombieGame.launcherListener.playUnityAds();
            }
        });
    }


    public Group parseScene(String name) {
        return CCScene.parse(game.getCocoStudioUIEditor(), name);
    }

    public enum PendingAction {
        NONE, ADD_COIN, HINT, RESTART, CLEAR, BALL_UNLOCK, REVIVE,SUCCESS_COIN
    }


    public void handlePendingAction(boolean done) {
        Log.log("BaseScreen", "handle pending action :" + pendingAction + "   done ---> " + done);
        if (!done) setPendingAction(PendingAction.NONE);
        switch (pendingAction) {
            case ADD_COIN:
                coinGot();
                break;
            case BALL_UNLOCK:
                unlockBall();
                break;
            case REVIVE:
                revive();
                break;
            case SUCCESS_COIN:
                successCoinGot();
        }
    }

    private void successCoinGot() {
        FlurryUtils.Ads_Reward("Coin");
        setPendingAction(PendingAction.NONE);
        Log.log("BaseScreen", "coin got !!!");
        Iterator<BaseDialog> iterator = dialogs.iterator();
        while(iterator.hasNext()){
            BaseDialog dialog = iterator.next();
            if(dialog instanceof SuccessDialog){
                ((SuccessDialog)dialog).extraCoinGot();
            }
        }
    }

    private void coinGot() {
        FlurryUtils.Ads_Reward("Coin");
        setPendingAction(PendingAction.NONE);
        Log.log("BaseScreen", "coin got !!!");
        Configuration.settingData.addCoin(pendingValue);
        final CommonAnimation coin = new CommonAnimation("animations/jinbi_fly.skel", 0.5F);
        final Actor actor = LayerUtils.createLayer();
        dialogsGroup.addActor(actor);
        actor.addAction(Actions.sequence(Actions.delay(1.7F), Actions.fadeOut(0.3F),Actions.removeActor(),Actions.run(new Runnable() {
            @Override
            public void run() {
                coin.remove();
            }
        })));
        coin.getState().setTimeScale(1);
        coin.setPosition(0, (Configuration.height - 800) / 2);
        coin.getState().setAnimation(0, "animation", false);
        dialogsGroup.addActor(coin);
    }

    private void revive() {
        setPendingAction(PendingAction.NONE);
        GameInfo.life++;
        end(GameScreen.class);
    }

    public void showMessage(String message) {
        Actor temp = dialogsGroup.findActor(message);
        if (temp != null) return;
        Group group = parseScene("ui/objects/text_hint.json");
        group.setY(400);
        Label label = group.findActor("hint_label");
        float ff = label.getX(Align.center);
        label.setWidth(360);
        label.setX(ff, Align.center);
        label.setWrap(true);
        label.setText(message);
        group.setName(message);
        dialogsGroup.addActor(group);
        group.addAction(Actions.sequence(Actions.delay(2), Actions.fadeOut(0.3F), Actions.removeActor()));
    }

    private void unlockBall() {
        setPendingAction(PendingAction.NONE);
        Configuration.settingData.cost(pendingValue);
        int id = 1;
        int goodNum = Assets.instance.getGoodsNum(1);
        for (int i = 1; i <= goodNum; i++) {
            if (Configuration.settingData.isGoodUnlocked(i, 1)) continue;
            id = i;
            break;
        }
        Configuration.settingData.unlockGood(id, 1);
        Log.log("BaseScreen", "unlockBall()-----> " + id);
    }

    public enum ScreenState {
        show, start, end, dark, out
    }

    public PendingAction getPendingAction() {
        return pendingAction;
    }


}
