package com.zyb.loveball.screens;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Array;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.utils.listener.ButtonListener;

public class ChallengeScreen extends BaseScreen {
    ScrollPane pane;
    Table table;
    private Array<ChapterObject> chapterObjects = new Array<>();

    public ChallengeScreen(HelloZombieGame game) {
        super(game);
        TAG = "challenge";
    }

    @Override
    public void show() {
        super.show();
        init();
    }

    private void init() {
        table = new Table();
        ScrollPane.ScrollPaneStyle scrollPaneStyle = new ScrollPane.ScrollPaneStyle();
        pane = new ScrollPane(table, scrollPaneStyle);
        pane.setSize(480, 650);
        pane.setY(70);
        groups[CENTER].addActor(pane);
        initChapters();
        layer.remove();
        scene.addActor(layer);
        initButtons();
    }

    private void initChapters() {
        long start = System.nanoTime();
        long end = System.nanoTime();
        System.out.println("init chapters : " + (end - start));
        start = System.nanoTime();
        Actor blank = new Actor();
        table.add(blank).width(80);
        for (int i = 1; i <= Constant.CHALLENGE_NUN; i++) {
            Group group = parseScene("ui/objects/challenge_object.json");
            chapterObjects.add(new ChapterObject(group, i));
            table.add(group).width(380).height(600);
            Color color = new Color(Constant.challengeColors[i - 1]);
            group.findActor("challenges_title").setColor(color);
            group.findActor("bg_frame1").setColor(color);
            group.findActor("bg_frame2").setColor(color.r, color.g, color.b, 0.8f);
            group.findActor("bg_frame3").setColor(color.r, color.g, color.b, 0.7F);
            group.findActor("bg_frame4").setColor(color.r, color.g, color.b, 0.7F);
        }
        end = System.nanoTime();
        System.out.println("init chapters : " + (end - start));
    }

    @Override
    protected void initButtons() {
        super.initButtons();
        findActor("btn_back", Touchable.enabled).addListener(new ButtonListener() {
            @Override
            public void touchUpEffect() {
                super.touchUpEffect();
                back();
            }
        });
    }

    @Override
    protected boolean back() {
        if (super.back()) {
            end(MenuScreen.class);
            return true;
        } else {
            return false;
        }
    }

    class ChapterObject {
        private final int id;
        private Group root;

        public ChapterObject(Group root, int id) {
            this.id = id;
            this.root = root;
            root.setSize(380, 630);
            root.setTouchable(Touchable.enabled);
            root.addListener(chapterSelectListener);
            root.setTransform(false);
            if (Configuration.settingData.isChallengePassed(id)) {
                root.findActor("clear").setVisible(true);
            } else {
                root.findActor("clear").setVisible(false);
            }
        }

        private ClickListener chapterSelectListener = new ClickListener() {
            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                super.touchUp(event, x, y, pointer, button);
                if (!pane.isPanning()) {
                    GameInfo.challenge(id);
                    end(GameScreen.class);
                }
            }
        };
    }
}
