package com.zyb.loveball.screens;

import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Array;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.ChapterData;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;
import com.zyb.loveball.dialogs.ConfirmDialog;
import com.zyb.loveball.ui.AtlasImage;
import com.zyb.loveball.ui.Grid;
import com.zyb.loveball.utils.LevelPicUtil;

public class ChapterScreen extends BaseScreen {
    ScrollPane pane;
    Table table;
    TextureAtlas atlas;
    String atlasName;

    private Array<ChapterObject> chapterObjects = new Array<>();

    public ChapterScreen(HelloZombieGame game) {
        super(game);
        TAG = "chapter";
    }

    @Override
    public void show() {
        super.show();
        atlasName = Assets.instance.prefix + "chapters.atlas";
        atlas = Assets.instance.assetManager.get(atlasName);
        init();
        pane.validate();
        pane.setScrollX(380 * (GameInfo.chapter - 1));
        pane.updateVisualScroll();
    }

    private void init() {
        table = new Table();
        ScrollPane.ScrollPaneStyle scrollPaneStyle = new ScrollPane.ScrollPaneStyle();
        pane = new ScrollPane(table, scrollPaneStyle);
        pane.setSize(480, 650);
        pane.setY(90+(Configuration.height-800) * 0.2F);
        groups[CENTER].addActor(pane);
        initChapters();
        layer.remove();
        scene.addActor(layer);
        initButtons();
    }

    private void initChapters() {
        Actor blank = new Actor();
        table.add(blank).width(80);
        for (int i = 1; i <= Constant.CHAPTER_NUN; i++) {
            Group group = parseScene("ui/objects/chapter_object.json");
            chapterObjects.add(new ChapterObject(group, i));
            table.add(group).width(380).height(630);
        }
    }

    @Override
    protected boolean back() {
        if (super.back()) {
            end(MenuScreen.class);
            return true;
        } else {
            return false;
        }
    }

    class ChapterObject {
        private final int id;
        private Group root;
        private boolean unlock;

        public ChapterObject(Group root, int id) {
            this.id = id;
            this.root = root;
            root.findActor("clear").setVisible(false);
            unlock = Configuration.unlock || Configuration.settingData.getUnlockChapter() >= id;
            root.findActor("lock").setVisible(!unlock);
            root.setSize(380, 630);
            root.setTransform(false);
            root.addListener(new ClickListener() {
                @Override
                public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                    super.touchUp(event, x, y, pointer, button);
                    if (!pane.isPanning()) {
                        touched();
                    }
                }
            });
            initStar();
            loadLevelPic();
            initTitleFont(id);
        }

        private void touched() {
            if (unlock) {
                GameInfo.chapter = ChapterObject.this.id;
                end(LevelScreen.class);
            } else {
                ConfirmDialog dialog = (ConfirmDialog) showDialog("ui/dialogs/confirm.json", ConfirmDialog.class);
                dialog.setTitle("Locked");
                dialog.setContent("Please pass all levels of the last chapter.");
            }
        }

        private void loadLevelPic() {
            Actor bg = root.findActor("bg");
            if (bg != null) {
                float bgWidth = bg.getWidth() / 2;
                float bgHeight = bg.getHeight() / 2;
                float centerX = bg.getX() + bg.getWidth() / 2;
                float centerY = bg.getY() + bg.getHeight() / 2;
                Grid grid = new Grid();
                grid.setNeedBg(true);
                grid.setScale(0.9F);
                grid.setColor(249 / 255f, 250 / 255f, 233 / 255f, 1);
                grid.setSize((bgWidth - 8) / 0.9F, (bgHeight - 8) / 0.9F);
                grid.setPosition(centerX - grid.getWidth() / 2, centerY - grid.getHeight() / 2);
                root.addActorBefore(root.findActor("lock"), grid);
            }
            int levelId = Assets.instance.chaptersData.get(id).getLevelId(1);
            Sprite region = LevelPicUtil.getLevelPic(levelId, atlas);
            if (region != null) {
                AtlasImage image = new AtlasImage(region);
                image.setName("level_pic");
                image.setOrigin(0);
                image.setSize(284, 480);
                root.addActorBefore(root.findActor("lock"), image);
                image.setPosition(8, 8);
            }
        }

        private void initStar() {
            int total = Configuration.settingData.getStarsChapter(id);
            ((Label) root.findActor("star_font")).setText(total + "/30");
            if (total >= 30) root.findActor("clear").setVisible(true);
        }

        private void initTitleFont(int id) {
            ChapterData data = Assets.instance.chaptersData.get(id);
            ((Label) root.findActor("chapter_title_font")).setText(data.getName());
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        if(Configuration.poor) {
            Assets.instance.assetManager.unload(atlasName);
        }
    }
}
