package com.zyb.loveball.screens;

import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.GameScreen;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.ChapterData;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.dialogs.ConfirmDialog;
import com.zyb.loveball.ui.AtlasImage;
import com.zyb.loveball.ui.Grid;
import com.zyb.loveball.utils.LevelPicUtil;

public class LevelScreen extends BaseScreen {
    ScrollPane pane;
    Table table;
    TextureAtlas atlas;
    private String atlasName;

    public LevelScreen(HelloZombieGame game) {
        super(game);
        TAG = "level";
    }


    @Override
    public void show() {
        super.show();
        atlasName = Assets.instance.prefix + "chapter_" + GameInfo.chapter + ".atlas";
        atlas = Assets.instance.assetManager.get(atlasName);
        init();
    }

    private void init() {
        table = new Table();
        ScrollPane.ScrollPaneStyle scrollPaneStyle = new ScrollPane.ScrollPaneStyle();
        pane = new ScrollPane(table, scrollPaneStyle);
        pane.setSize(480, 635);
        pane.setY(65+(Configuration.height-800) * 0.2F);
        groups[CENTER].addActor(pane);
        initButtons();
        initLevels();
        layer.remove();
        scene.addActor(layer);
        ChapterData chapterData = Assets.instance.chaptersData.get(GameInfo.chapter);
        ((Label) findActor("level_title")).setText(chapterData.getName());
        ((Label) findActor("star_font")).setText(Configuration.settingData.getStarsChapter(GameInfo.chapter) + "/30");
        if (GameInfo.chapter == Configuration.settingData.getLastChapterInt()) {
            pane.validate();
            pane.setScrollX(200 * ((GameInfo.id - 1) / 2));
            pane.updateVisualScroll();
        }
    }


    private void initLevels() {
        int levelNum = 10;
        Actor tmpActor = new Actor();
        table.add(tmpActor).width(45);
        for (int i = 1; i <= levelNum; i += 2) {
            Group group = parseScene("ui/objects/level_object.json");
            group.setTouchable(Touchable.enabled);
            new LevelObject(group, i);
            table.add(group).width(173).height(252);
            table.add(tmpActor).width(40);
        }
        table.row();
        table.add(tmpActor).height(60);
        table.row();
        tmpActor = new Actor();
        table.add(tmpActor).width(45);
        for (int i = 2; i <= levelNum; i += 2) {
            Group group = parseScene("ui/objects/level_object.json");
            group.setTouchable(Touchable.enabled);
            new LevelObject(group, i);
            table.add(group).width(173).height(252);
            table.add(tmpActor).width(40);
        }
    }

    @Override
    protected boolean back() {
        if (super.back()) {
            end(ChapterScreen.class);
            return true;
        } else {

            return false;
        }
    }

    class LevelObject {
        private final int id;
        private Group root;
        private boolean unlock;

        public LevelObject(Group root, int id) {
            this.id = id;
            this.root = root;
            root.findActor("badge").setVisible(false);
            unlock = Configuration.unlock || Configuration.settingData.getUnlockChapter() > GameInfo.chapter
                    || (Configuration.settingData.getUnlockChapter() == GameInfo.chapter
                    && id <= Configuration.settingData.getUnlockLevel());
            root.findActor("lock").setVisible(!unlock);
            root.addListener(new ClickListener() {
                @Override
                public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                    super.touchUp(event, x, y, pointer, button);
                    if (!pane.isPanning()) {
                        touched();
                    }
                }
            });
            initImage();
            initStars();
            initLevelFont();

        }

        private void touched() {
            if (unlock) {
                GameInfo.id = LevelScreen.LevelObject.this.id;
                end(GameScreen.class);
            } else {
                ConfirmDialog dialog = (ConfirmDialog) showDialog("ui/dialogs/confirm.json", ConfirmDialog.class);
                dialog.setTitle("Locked");
                dialog.setContent("Complete previous levels to unlock.");
            }
        }

        private void initLevelFont() {
            ((Label) root.findActor("level_font")).setText(GameInfo.chapter + "-" + id);
        }

        private void initStars() {
            int starInt = Configuration.settingData.getStars(GameInfo.chapter, id);
            for (int j = starInt; j < 3; j++) {
                root.findActor("star_" + j).setVisible(false);
            }
            if (starInt >= 3) {
                root.findActor("badge").setVisible(true);
            }
        }

        private void initImage() {
            Actor bg = root.findActor("Image_1");
            if (bg != null) {
                float bgWidth = bg.getWidth() / 2;
                float bgHeight = bg.getHeight() / 2;
                float centerX = bg.getX() + bg.getWidth() / 2;
                float centerY = bg.getY() + bg.getHeight() / 2;
                Grid grid = new Grid();
                grid.setNeedBg(true);
                grid.setScale(0.5F);
                grid.setColor(249 / 255f, 250 / 255f, 233 / 255f, 1);
                grid.setSize((bgWidth - 8) / 0.5F, (bgHeight - 8) / 0.5F);
                grid.setPosition(centerX - grid.getWidth() / 2, centerY - grid.getHeight() / 2);
                root.addActorBefore(root.findActor("lock"), grid);
            }
            int levelId = Assets.instance.chaptersData.get(GameInfo.chapter).getLevelId(id);
            Sprite sprite = LevelPicUtil.getLevelPic(levelId, atlas);
            if (sprite != null) {
                AtlasImage image = new AtlasImage(sprite);
                image.setSize(160, 240);
                root.addActorBefore(root.findActor("lock"), image);
                image.setPosition(8, 8);
            }
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        Assets.instance.assetManager.unload(atlasName);
        for(int i =2;i<=10;i++){
            int levelId = Assets.instance.chaptersData.get(GameInfo.chapter).getLevelId(i);
            LevelPicUtil.disposeLocalPic(levelId);
        }
    }
}
