package com.zyb.loveball.screens;

import com.badlogic.gdx.Application;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.NinePatchDrawable;
import com.badlogic.gdx.utils.Align;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.utils.Log;

/**
 * Created by lilexiang on 17-8-15.
 */
public class LoadScreen extends BaseScreen {

    Image bg;
    String prefix = "desktop/";
    Actor layer;
    TextureAtlas atlas;

    public LoadScreen(HelloZombieGame game) {
        super(game);
        TAG = "load";
    }


    @Override
    public void show() {
        if (Gdx.app.getType() == Application.ApplicationType.Android) {
            prefix = "image/";
        }
        Assets.instance.assetManager.load(prefix + "load.atlas", TextureAtlas.class);
        Assets.instance.assetManager.finishLoadingAsset(prefix + "load.atlas");
        atlas = Assets.instance.assetManager.get(prefix + "load.atlas");
        NinePatch ninePatch = new NinePatch(atlas.findRegion("white"));
        bg = new Image(ninePatch);
        bg.setColor(232/255F,250/255F,233/255F,1);
        bg.setSize(HelloZombieGame.getViewport().getWorldWidth(), HelloZombieGame.getViewport().getWorldHeight());
        stage.addActor(bg);
        Image mark = new Image(atlas.findRegion("title_hello zombie"));
        mark.setSize(mark.getWidth() / 2, mark.getHeight() / 2);
        mark.setColor(1, 1, 1, 0);
        mark.setPosition(240, stage.getHeight() / 2, Align.center);
        mark.setOrigin(mark.getWidth() / 2, mark.getHeight() / 2);
        mark.addAction(Actions.sequence(Actions.parallel(Actions.fadeIn(1), Actions.scaleBy(0.05F, 0.05F, 1)), Actions.scaleBy(0.2f, 0.2F, 4), Actions.forever(Actions.sequence(Actions.scaleBy(-0.05F, -0.05F, 2), Actions.scaleBy(0.05F, 0.05F, 2)))));
        stage.addActor(mark);
        Assets.instance.load();
        initLayer();
    }


    private void initLayer() {
        TextureRegion region = atlas.findRegion("white");
        NinePatch patch = new NinePatch(region, 4, 4, 4, 4);
        NinePatchDrawable drawable = new NinePatchDrawable(patch);
        layer = new Image(drawable);
        layer.setSize(500, 1160);
        layer.setPosition(-10, -120);
        layer.setColor(0, 0, 0, 0);
    }

    public float stateTime = 0;
    boolean loaded = false;
    boolean end;

    @Override
    public void render(float delta) {
        if (stage != null) {
            stage.act();
        }
        if (stage != null) {
            stage.draw();
        }
        stateTime += delta;
        if (!loaded && Assets.instance.assetManager.update(100) && stateTime >= 3) {
            Log.log("LoadScreen","render()---> load resource end"  );
            loaded = true;
            Assets.instance.loadFinish();
            stage.addActor(layer);
            layer.addAction(Actions.fadeIn(0.3F));
            stateTime = 0;
        }
        if(loaded && !end && stateTime>0.6F){
            Log.log("LoadScreen","render()---> load screen end"  );
            end = true;
            game.setScreen(new MenuScreen(game));
            Assets.instance.assetManager.unload(prefix + "load.atlas");
        }
    }

}
