package com.zyb.loveball.ui;

import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.utils.Array;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.assets.Assets;
import com.zyb.loveball.assets.Configuration;
import org.freyja.libgdx.cocostudio.ui.parser.group.CCScene;

public class StorePanel {
    ScrollPane pane;
    Table table;
    Array<StoreObject> objects;
    int type;
    StorePanelListener listener;

    public StorePanel(int type) {
        this.type = type;
        load();
    }

    public void show(StorePanelListener listener, Group parent) {
        this.listener = listener;
        parent.addActor(pane);
    }

    public void load() {
        table = new Table();
        objects = new Array<>();
        ScrollPane.ScrollPaneStyle scrollPaneStyle = new ScrollPane.ScrollPaneStyle();
        pane = new ScrollPane(table, scrollPaneStyle);
        pane.setSize(480, 580);
        pane.setY(120);
        if (type == 0) {
            initPenTable();
        } else {
            initBallTable();
        }
    }

    private void initPenTable() {
        Actor tmpActor = new Actor();
        table.add(tmpActor).width(60);
        for (int i = 1; i <= Assets.instance.getGoodsNum(type); i++) {
            Group groupPen = CCScene.parse(HelloZombieGame.getCocoStudioUIEditor(), "ui/objects/pen_skin_object.json");
            groupPen.setTouchable(Touchable.enabled);

            objects.add(new StoreObject(groupPen,i,type,this));
            table.add(groupPen).width(200).height(450);
        }

        pane.validate();
        pane.setScrollX(180 * (Configuration.settingData.getGoodId(type)-1));
        pane.updateVisualScroll();
    }

    private void initBallTable() {
        Actor tmpActor = new Actor();
        table.add(tmpActor).width(60);
        for (int i = 1; i <= Assets.instance.getGoodsNum(type); i += 2) {
            Group groupBall = CCScene.parse(HelloZombieGame.getCocoStudioUIEditor(), "ui/objects/ball_skin_object.json");
            groupBall.setTouchable(Touchable.enabled);
            objects.add(new BallStoreObject(groupBall,i,type,this));
            table.add(groupBall).width(200).height(280);
        }
        table.row();
        tmpActor = new Actor();
        table.add(tmpActor).width(60);
        for (int i = 2; i <= Assets.instance.getGoodsNum(type); i += 2) {
            Group groupBall = CCScene.parse(HelloZombieGame.getCocoStudioUIEditor(), "ui/objects/ball_skin_object.json");
            groupBall.setTouchable(Touchable.enabled);
            objects.add(new BallStoreObject(groupBall,i,type,this));
            table.add(groupBall).width(200).height(280);
        }
        pane.validate();
        pane.setScrollX(165 * ((Configuration.settingData.getGoodId(type)) / 2-1));
        pane.updateVisualScroll();
    }

    public void update() {
        for (StoreObject object : objects) {
            object.update();
        }
        listener.update();
    }

    public void showCoinDialog(){
        listener.showCoinDialog();
    }

    public void setVisible(boolean visible) {
        pane.setVisible(visible);
    }

    public interface StorePanelListener {
        void update();

        void showCoinDialog();
    }

    public boolean isPanning(){
        return pane.isPanning();
    }
}
