package com.zyb.loveball.utils;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.utils.ArrayMap;

import java.io.IOException;

/**
 * Created by lilexiang on 17-9-10.
 */
public class CarDataLoader {
    public static ArrayMap<Integer, ArrayMap<Integer, CarData>> carDatas;
    public static ArrayMap<String, ArrayMap<String,Integer>> poseDatas;
    public static ArrayMap<String,CarConfig> carconfigs;

    public static void loadData() {
        carDatas = new ArrayMap<>();

        CsvReader reader = null; // 定义Csvreader
        reader = new CsvReader(Gdx.files.internal("Car Data.csv").reader());
        try {
            reader.readHeaders();
            while (reader.readRecord()) {
                String str = reader.get("CarId");
                if (str == null || str.equals("")) continue;
                try {
                    int id = Integer.parseInt(str);
                    CarData data = new CarData();
                    data.carId = (Integer.parseInt(str) - 1);
                    data.level = (Integer.parseInt(reader.get("Level")));
                    data.speedUp = (Integer.parseInt(reader.get("SpeedUp")));
                    data.turn = (Float.parseFloat(reader.get("Turn")));
                    data.cost[0] = (Integer.parseInt(reader.get("Cost1")));
                    data.fuel = (Integer.parseInt(reader.get("Fuel")));
                    data.cost[1] = (Integer.parseInt(reader.get("Cost2")));
                    data.cost[2] = (Integer.parseInt(reader.get("Cost3")));
                    ArrayMap<Integer, CarData> array = carDatas.get(data.carId);
                    if (array == null) {
                        array = new ArrayMap<>();
                        carDatas.put(data.carId, array);
                    }
                    array.put(data.level, data);
                } catch (Exception e) {
                    e.printStackTrace();
                }


            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        loadPoses();
        loadConfig();
    }

    private static void loadPoses() {
        poseDatas = new ArrayMap<>();

        CsvReader reader = null; // 定义Csvreader
        reader = new CsvReader(Gdx.files.internal("data/vehicle/carposData.csv").reader());
        try {
            reader.readHeaders();
            ArrayMap<String,Integer> temp = null;
            String carName;
            while (reader.readRecord()) {
                String str = reader.get("carName");
                if(str!=null && !str.equals("")){
                    carName = str;
                    temp = poseDatas.get(carName);
                    if(temp == null){
                        temp = new ArrayMap<>();
                        poseDatas.put(carName,temp);
                    }
                }
                str = reader.get("poseName");
                String poseName = null;
                int posePosition = 0;
                if(str!=null && !str.equals("")){
                    poseName = str;
                }
                str = reader.get("position");
                if(str!=null || !str.equals("")){
                    posePosition = Integer.parseInt(str);
                }
                temp.put(poseName,posePosition);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    private static void loadConfig() {
        carconfigs  = new ArrayMap<>();

        CsvReader reader = null; // 定义Csvreader
        reader = new CsvReader(Gdx.files.internal("data/vehicle/carconfig.csv").reader());
        try {
            reader.readHeaders();
            String carName;

            while (reader.readRecord()) {
                String str = reader.get("carName");
                if(str!=null && !str.equals("")){
                    carName = str;
                    CarConfig config = new CarConfig();
                    config.name = carName;
                    str = reader.get("weight");
                    if(str!=null && !str.equals("")){
                        config.weight = Float.parseFloat(str);
                    }
                    str = reader.get("force");
                    if(str!=null && !str.equals("")){
                        config.force = Float.parseFloat(str);
                    }
                    str = reader.get("broken");
                    if(str!=null && !str.equals("")){
                        config.broken = Float.parseFloat(str);
                    }
                    carconfigs.put(carName,config);
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static class CarData {
        public int carId;
        public int level;
        public int speedUp;
        public float turn;
        public int []cost = new int[3];
        public int fuel;
    }

    public static class CarConfig{
        public String name;
        public float weight;
        public float force;
        public float broken;
    }
}
