package com.zyb.loveball.utils;

import com.badlogic.gdx.Files;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.glutils.FrameBuffer;
import com.badlogic.gdx.math.Matrix4;
import com.badlogic.gdx.utils.ArrayMap;
import com.badlogic.gdx.utils.BufferUtils;
import com.badlogic.gdx.utils.ObjectMap;
import com.badlogic.gdx.utils.ScreenUtils;
import com.zyb.loveball.HelloZombieGame;
import com.zyb.loveball.GamePanel;
import com.zyb.loveball.GameInfo;
import com.zyb.loveball.assets.Configuration;
import com.zyb.loveball.constants.Constant;

import java.io.File;
import java.util.Iterator;

public class MScreenUtil {
    static Matrix4 matrix4 = new Matrix4();
    private static boolean bufferDone = true;

    public static void screenShot(HelloZombieGame game, GamePanel gamePanel, FileHandle file,int width,int height, boolean newThread) {
        FrameBuffer buffer = game.getFrameBuffer();
        if (buffer == null) {
            Log.log("MScreenUtil", "screenShot() ---> framebuffer is null ");
            return;
        }
        if (file == null) {
            String pngFileName = "level_pic_" + GameInfo.levelId;
            file = Gdx.files.local(Constant.LEVEL_PIC_PATH + pngFileName + ".png");
        }
        Batch batch = game.getBatch();
        Log.log("MScreenUtil", "screenShot() ---> " + file.path());
        boolean batchBegin = false;
        matrix4.set(batch.getProjectionMatrix());
        batch.setProjectionMatrix(gamePanel.getCamera().combined);
        if (!batch.isDrawing()) {
            batch.begin();
            batchBegin = true;
        }
        buffer.begin();
        gamePanel.getBg().setDrawable(false);
        batch.setColor(1, 1, 1, 1);
        gamePanel.draw(batch, 1);
        if (batchBegin) {
            batch.end();
        }
        gamePanel.getBg().setDrawable(true);
        batch.setProjectionMatrix(matrix4);
        buffer.end();
        bufferDone = false;
        if (!newThread) {
            writePngFromBuffer(file, width,height,newThread);
        }
    }

    private static void writePng(final FileHandle fileHandle, final int width, final int height, Pixmap pixmap) {
        Pixmap temp = new Pixmap(width, height, Pixmap.Format.RGBA8888);
        temp.setBlending(Pixmap.Blending.None);
        temp.drawPixmap(pixmap, 0, 0, pixmap.getWidth(), pixmap.getHeight(), 0, 0, temp.getWidth(), temp.getHeight());
        HelloZombieGame.imageUtil.saveJpg(fileHandle, temp);
        pixmap.dispose();
        temp.dispose();
    }


    public static void writePngFromBuffer(final FileHandle file, final int width, final int height, boolean newThread) {
        if(bufferDone) return;
        try {
            FrameBuffer buffer = HelloZombieGame.getFrameBuffer();
            buffer.begin();
            final byte[] pixels = ScreenUtils.getFrameBufferPixels(0, 0,
                    Configuration.width,
                    Configuration.height, true);
            final Pixmap pixmap = new Pixmap(Configuration.width,
                    Configuration.height, Pixmap.Format.RGBA8888);
            BufferUtils.copy(pixels, 0, pixmap.getPixels(), pixels.length);
            if (newThread) {
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        writePng(file, width, height, pixmap);
                    }
                }).start();

            } else {
                writePng(file, width, height, pixmap);
            }
            buffer.begin();
            Gdx.gl20.glClearColor(0, 0, 0, 0);
            Gdx.gl20.glClear(GL20.GL_COLOR_BUFFER_BIT);
            buffer.end();
            bufferDone = true;
        } catch (Throwable e) {
            e.printStackTrace();
        }

    }

    public static void createLevelPic(HelloZombieGame game) {
        File file = new File("");
        File dstFile = new File(new File(file.getAbsolutePath()).getParentFile().getParentFile().getAbsolutePath() + "/Assets/pic/");
        GameInfo.normal();
        for (int i = 1; i <= Constant.CHAPTER_NUN; i++) {
            GameInfo.id = 1;
            GameInfo.chapter = i;
            GameInfo.start();
            String pngFileName = "level_pic_" + GameInfo.levelId;
            FileHandle dir = Gdx.files.absolute(dstFile.getPath() + "/chapters");
            if (!dir.exists()) {
                dir.mkdirs();
            }
            String filePath = dstFile.getPath() + "/chapters/" + pngFileName + ".png";
            FileHandle pngFileHandle = Gdx.files.getFileHandle(filePath, Files.FileType.Absolute);
            try {
                GameInfo.hint = false;
                GamePanel gamePanel = new GamePanel(game.getViewport().getWorldWidth() * Constant.SCALE, game.getViewport().getWorldHeight() * Constant.SCALE, null);
                screenShot(game, gamePanel, pngFileHandle, Constant.SCREEN_SHOT_W_BIG,Constant.SCREEN_SHOT_H_BIG, false);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        ArrayMap<Integer, Float> length = new ArrayMap<>();

        for (int i = 1; i <= Constant.CHAPTER_NUN; i++) {
            for (int k = 1; k <= 10; k++) {
                GameInfo.id = k;
                GameInfo.chapter = i;
                GameInfo.start();
                String pngFileName = "level_pic_" + GameInfo.levelId;
                String filePath = dstFile.getPath() + "/chapter_" + i + "/" + pngFileName + ".png";
                FileHandle dir = Gdx.files.absolute(dstFile.getPath() + "/chapter_" + i);
                if (!dir.exists()) {
                    dir.mkdirs();
                }
                FileHandle pngFileHandle = Gdx.files.getFileHandle(filePath, Files.FileType.Absolute);
                try {
                    GameInfo.hint = false;
                    GamePanel gamePanel = new GamePanel(game.getViewport().getWorldWidth() * Constant.SCALE, game.getViewport().getWorldHeight() * Constant.SCALE, null);
                    length.put(GameInfo.levelId, gamePanel.getHintLength());
                    screenShot(game, gamePanel, pngFileHandle, Constant.SCREEN_SHOT_W_SMALL,Constant.SCREEN_SHOT_H_SMALL, false);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        Iterator<ObjectMap.Entry<Integer, Float>> iterator = length.iterator();
        System.out.println("---------------------------------------->");
        while (iterator.hasNext()) {
            ObjectMap.Entry<Integer, Float> entry = iterator.next();
            System.out.println(entry.key + "   " + entry.value);
        }
        System.out.println("---------------------------------------->");
    }


}
